/*--------------------------------------------------------------------------
**      module.h                 Twine           From:  Wine/debugger
**
**              
**------------------------- < License Information > ------------------------
**
**      This file is a direct copy of the equivalent file in the
**  Wine project (www.winehq.com).  The Wine license permits
**  copying of source code, so long as all copyright and credit
**  information is preserved.  Any and all copyrights originally
**  in the Wine source code file are preserved exactly as they were
**  when this copy was made from the Wine source tree.
**  (see the 'Original Copyright(s)', below).
**  An exacty copy of the Wine license is contained in WINE-LICENSE.
**  
**      For a more general description of the Wine license, and
**  for a full list of the authors who deserve credit for this
**  code, please see www.winehq.com, or WINE-AUTHORS.
**  
**      However, because this file is released with Twine, we
**  are obliged to also release it under the LGPL license.
**  In short, you may make a copy or republish this code so long as
**  all of your changes to source code are again made publicly available.
**  Further, absolutely no warranty of any kind is given or implied.
**  Refer to the LGPL description at www.fsf.org or the file LICENSE 
**  for further details.
**  
**  
**      ==========================================================
**      IF YOU MAKE CHANGES TO THIS CODE, PLEASE REFLECT
**      THOSE CHANGES IN THE ORIGINAL WINE CODE, AT WWW.WINEHQ.COM
**      ==========================================================
**
**      
**------------------------ < Original Copyright(s) > -----------------------
**      
**------------------------ < File Content Description > --------------------
**
**  Module:	debugger/module.h
**
**  Description:
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/debugger/module.h,v 1.3 1999/07/08 18:53:02 mwalsh Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/


/* Macros for easier access to context registers */

#define EAX_reg(context)     ((context)->Eax)
#define EBX_reg(context)     ((context)->Ebx)
#define ECX_reg(context)     ((context)->Ecx)
#define EDX_reg(context)     ((context)->Edx)
#define ESI_reg(context)     ((context)->Esi)
#define EDI_reg(context)     ((context)->Edi)
#define EBP_reg(context)     ((context)->Ebp)

#define CS_reg(context)      ((context)->SegCs)
#define DS_reg(context)      ((context)->SegDs)
#define ES_reg(context)      ((context)->SegEs)
#define FS_reg(context)      ((context)->SegFs)
#define GS_reg(context)      ((context)->SegGs)
#define SS_reg(context)      ((context)->SegSs)

#define EFL_reg(context)     ((context)->EFlags)
#define EIP_reg(context)     ((context)->Eip)
#define ESP_reg(context)     ((context)->Esp)

#define AX_reg(context)      (*(WORD*)&EAX_reg(context))
#define BX_reg(context)      (*(WORD*)&EBX_reg(context))
#define CX_reg(context)      (*(WORD*)&ECX_reg(context))
#define DX_reg(context)      (*(WORD*)&EDX_reg(context))
#define SI_reg(context)      (*(WORD*)&ESI_reg(context))
#define DI_reg(context)      (*(WORD*)&EDI_reg(context))
#define BP_reg(context)      (*(WORD*)&EBP_reg(context))

#define AL_reg(context)      (*(BYTE*)&EAX_reg(context))
#define AH_reg(context)      (*((BYTE*)&EAX_reg(context)+1))
#define BL_reg(context)      (*(BYTE*)&EBX_reg(context))
#define BH_reg(context)      (*((BYTE*)&EBX_reg(context)+1))
#define CL_reg(context)      (*(BYTE*)&ECX_reg(context))
#define CH_reg(context)      (*((BYTE*)&ECX_reg(context)+1))
#define DL_reg(context)      (*(BYTE*)&EDX_reg(context))
#define DH_reg(context)      (*((BYTE*)&EDX_reg(context)+1))
                            
#define IP_reg(context)      (*(WORD*)&EIP_reg(context))
#define SP_reg(context)      (*(WORD*)&ESP_reg(context))
                            
#define FL_reg(context)      (*(WORD*)&EFL_reg(context))

#define SET_CFLAG(context)   (EFL_reg(context) |= 0x0001)
#define RESET_CFLAG(context) (EFL_reg(context) &= 0xfffffffe)

/*------------------------- < Full Revision History > ----------------------
** $Log: module.h,v $
** Revision 1.3  1999/07/08 18:53:02  mwalsh
** Updated Comments
**
**-------------------------------------------------------------------------*/
