/*--------------------------------------------------------------------------
**      GetUserName.c                   Twine           
**              
**------------------------- < License Information > ------------------------
**
**  
**      This file is released under terms of an LGPL.  Readers of 
**  this file may modify or use this source code so long as they make all
**  modifications publically available.  Please refer to LICENSE 
**  for further details.
**  
**
**      The original Copyright is in 1999, by CodeWeavers, Inc.
**
**      
**------------------------ < File Content Description > --------------------
**
**  Module:	dlls/advapi32/GetUserName
**	
**  Description:
**    Contains functions to return a user name.
**
**  Functions defined:
**      GetUserName
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/dlls/advapi32/GetUserName.c,v 1.1 1999/07/27 02:13:01 jwhite Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/



/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/
#include <windows.h>

#include <pwd.h>



/*------< Function GetUserName >------------------------------------------
**
**  Description:
**      This function returns the username of the current user.
**		
**  Parameters:
**      1.  lpBuffer       Pointer to the output buffer
**      2.  nSize          Size of the output buffer
**
**  Returns:
**      TRUE if it worked, FALSE if it didn't
**
**--------------------------------------------------------------------------*/
BOOL GetUserName(LPTSTR lpBuffer, LPDWORD nSize)
{
    struct passwd *p;

    /*------------------------------------------------------------------------
    **  Get the passwd file entry, if we can.
    **----------------------------------------------------------------------*/
    p = getpwuid(getuid());
    if (p)
    {
        /*--------------------------------------------------------------------
        **  The spec requires that we test the length and return the
        **      size required.
        **------------------------------------------------------------------*/
        if (*nSize > (strlen(p->pw_name) + 1))
        {
            *nSize = strlen(p->pw_name) + 1;
            return FALSE;
        }

        /*--------------------------------------------------------------------
        **  Copy in as much of the username as we can, including nul byte
        **------------------------------------------------------------------*/
        memcpy(lpBuffer, p->pw_name, strlen(p->pw_name) + 1);
        *nSize = strlen(p->pw_name) + 1;
        return TRUE;
    }
    else
        return FALSE;
}

/*------------------------- < Full Revision History > ----------------------
** $Log: GetUserName.c,v $
** Revision 1.1  1999/07/27 02:13:01  jwhite
** Added the GetUserName function.
**
**
**-------------------------------------------------------------------------*/


