/*--------------------------------------------------------------------------
**      security.c                   Twine           
**              
**------------------------- < License Information > ------------------------
**
**  
**      This file is released under terms of an LGPL.  Readers of 
**  this file may modify or use this source code so long as they make all
**  modifications publically available.  Please refer to LICENSE 
**  for further details.
**  
**
**      The original Copyright is in 1999, by CodeWeavers, Inc.
**
**      
**------------------------ < File Content Description > --------------------
**
**  Module:	dlls/advapi32/security.c
**	
**  Description:
**      This file contains implementations for as many of the NT
**  security functions as is possible.
**		
**
**  Functions defined:
**      InitializeSecurityDescription
**
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/dlls/advapi32/security.c,v 1.2 1999/07/12 15:38:13 mwalsh Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/



/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#include "windows.h"
#include "winnt.h"



/*------< Function >---------------------------------------------------------
**
**                  InitializeSecurityDescritpion  
**
**  Description:
**      This function initializes a security structures.
**		
**  Parameters:
**      1.  pSecurityDescriptor     A pointer to a security descriptor
**                                  This must be at least xxx bytes long.
**      2.  dwRevision              The revision of the security descriptor
**                                  Generally is SECURITY_DESCRIPTOR_REVISION
**
**  Returns:
**      True on success, False on failure
**
**  Notes:
**    
**--------------------------------------------------------------------------*/
BOOL32 WINAPI InitializeSecurityDescriptor(
        PSECURITY_DESCRIPTOR pSecurityDescriptor,
        DWORD                dwRevision)
{
    if (pSecurityDescriptor)
    {
        /*-----------------------------------------------------------------
        ** Clear the struct out to zero values.  That's what NT does.
        **----------------------------------------------------------------*/
        memset(pSecurityDescriptor, '\0', sizeof(pSecurityDescriptor)); 
        pSecurityDescriptor->Revision = dwRevision;
    }

    return(TRUE);
}


/*------< Function >---------------------------------------------------------
**
**                  SetSecurityDescriptorDacl
**
**  Description:
**      This function sets a discretionary ACL into a security
**  security descriptor.
**		
**  Parameters:
**      1.  pSecurityDescriptor     A pointer to a security descriptor
**                                  This must be at least xxx bytes long.
**      2.  bDaclPresent            A boolean, indicating whether or not
**                                  the discretionary ACL is present.
**      3.  pDacl                   Points to a discretionary ACL that
**                                  is subsequently used inside of this
**                                  descriptor.  If NULL, the ACL
**                                  of this descriptor is cleared.
**                                  Note that the caller is required
**                                  to keep this storage until the
**                                  Descriptor is cleared.
**      4.  bDaclDefaulted          Whether or not this ACL is a default,
**                                  or has been user specified.
**
**
**  Returns:
**      True on success, False on failure
**
**  Notes:
**    
**--------------------------------------------------------------------------*/
BOOL32 WINAPI SetSecurityDescriptorDacl(
        PSECURITY_DESCRIPTOR pSecurityDescriptor,
        BOOL                 bDaclPresent,
        LPACL                pDacl,
        BOOL                 bDaclDefaulted)
{
    if (pSecurityDescriptor)
    {
        /*-----------------------------------------------------------------
        **  If they gave us an ACL, use it...
        **----------------------------------------------------------------*/
        if (bDaclPresent)
        {
            pSecurityDescriptor->Control |= SE_DACL_PRESENT;
            pSecurityDescriptor->Dacl = pDacl;
            if (bDaclDefaulted)
                pSecurityDescriptor->Control |= SE_DACL_DEFAULTED;
            else
                pSecurityDescriptor->Control &= ~SE_DACL_DEFAULTED;
        }
        else
            pSecurityDescriptor->Control &= ~SE_DACL_PRESENT;
    }

    return(TRUE);
}

/*------------------------- < Full Revision History > ----------------------
** $Log: security.c,v $
** Revision 1.2  1999/07/12 15:38:13  mwalsh
** Updated comments to standard header and footer style
**
** Revision 1.1  1999/03/23 02:49:10  jwhite
** Added InitializeSecurityDescriptor and SetSecurityDescriptorDacl
**
**-------------------------------------------------------------------------*/


