/*--------------------------------------------------------------------------
**      WToolbar.c                Twin           From:  Twin/dlls/commctrl
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      Copyright 1997 Willows Software, Inc.
**------------------------ < File Content Description > --------------------
**
**  Module:	dlls/commctrl/WToolbar.c
**
**  Description:
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/dlls/commctrl/WToolbar.c,v 1.18 2000/01/21 06:33:50 jwhite Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/
#ifdef _WINDOWS
#include <windows.h>
#include <commctrl.h>

/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#else
#include "WCommCtrl.h"
#endif
#include "string.h"
#include "WToolbar.h"
#include "WLists.h"
#include "WTooltip.h"
#include "WImgList.h"
#include "WResourc.h"
#include "WTlbCust.h"
#include "Log.h"
#include "SysMetrics.h"

/*----------------------------------------------------------------------------
**  C?_BORDER
**      Specifies how much white space to leave outside _all_ of the buttons
**  when there is no divider
**--------------------------------------------------------------------------*/
#define  CX_BORDER		1
#define  CY_BORDER		1

/*----------------------------------------------------------------------------
**  C?_BORDER_DIVIDER
**      Specifies how much white space to leave outside _all_ of the buttons
**  when there is a divider.
**--------------------------------------------------------------------------*/
#define  CX_BORDER_DIVIDER	1
#define  CY_BORDER_DIVIDER	2

/*----------------------------------------------------------------------------
**  VERT_PADDING, HORZ_PADDING
**    Amount of whitespace to place between each button.
**--------------------------------------------------------------------------*/
#define  VERT_PADDING		1
#define	 HORZ_PADDING		1

#define	 MAX_LABELLENGTH	20

#define  SMALLIMAGE_CX		16
#define  SMALLIMAGE_CY		15

#define  LARGEIMAGE_CX		24
#define  LARGEIMAGE_CY		21

#define  TOOLBMP_CX		16
#define  TOOLBMP_CY		15

#define  TOOLBTN_CX		24
#define  TOOLBTN_CY		22

#define	 STDTOOLBAR_IMAGES	15
#define	 VIEWTOOLBAR_IMAGES	12

extern HINSTANCE hLibInstance;
static char TOOLBAR_PROP[] = "Toolbar_Data";	
static char szCustomizeDlgTemplate[] = "WTLBCUSTOMIZETEMPLATE";	
static HCURSOR	g_hDragCursor = ( HCURSOR )NULL;
static HCURSOR	g_hDefCursor = ( HCURSOR )NULL;
enum { eNoDrag = 0, eEndDrag = eNoDrag, eCancelDrag,
       eLBtnDown, eBeginDrag, eDragging, 
       eShftLBtnDown, eShftBeginDrag, eShftDragging  };
enum { eHitOutside = -1 };

struct TBCDLGBTN;
typedef struct 
{
	int		HitBtn;
	int		HitRow;  /* meaningless unless BtnIndx < 0 */
} TLBHITTEST;

typedef struct 
{
	TBBUTTON	Btn;
	RECT		Rect;  
	int		Row;
/*	LPSTR		pszText;*/
} TLBITEM;

typedef struct 
{
	char		Label[MAX_LABELLENGTH];
} TLBSTR;

typedef struct 
{
	HDC		hbmMonoOld;
	HDC		hbmOldGlyphs;
	HBITMAP		hbmMono;
} TLBGDI;

typedef struct 
{
	UINT		uStyles;
	UINT		uExtStyles;
	UINT		maxrows;
	UINT		buttonstructsize;
	POINT		Pos;
	SIZE		Size;
	int		Rows;

	int		enDrag;	 /* for dragging */
	TBNOTIFY	tbn;
	char		cBtnText[MAX_LABELLENGTH]; /* holds text of button being dragged */

	HWND		hwTTip;
	HFONT		hFont;
	HDC		hDCGlyphs;
	HDC		hDCMono;
	HBITMAP		hbmImage;

	int		ImgCX;
	int		ImgCY;
	int		ImgCount;
	int		StrCount;

        /*--------------------------------------------------------------------
        **  nXPadding, nYPadding
        **    Space around the buttons inside the window
        **  NOT space between buttons.  That is XXXX_PAD
        **------------------------------------------------------------------*/
	int		nXPadding;
	int		nYPadding;

	int		ItmHeight;
	int		ItmWidth;

	int		ItmCount;
	int		ItmSelected;
	int		ItmHot;
	int		ItmTip; /*the button for which a tooltip is going to be displayed*/

    	BOOL        	bNeedSetRects;
	BOOL		bStandardBmp; /*toolbar with predefined bitmap*/

	
	WLIST		ItmList; /*array of Toolbar buttons*/
	WLIST		StrList;

        /*--------------------------------------------------------------------
        ** FIXME:  Only a first pass has been implemented of this
        **------------------------------------------------------------------*/
        HIMAGELIST      himl;       /* Handle to image list for buttons     */
        HIMAGELIST      himlHot;    /* Handle to image list for hot buttons */
} TOOLBARDATA;

#define GETTHISDATA(hwnd) \
   GetProp(hwnd, TOOLBAR_PROP )
#define SETTHISDATA(hwnd, pthisNew) \
   SetProp(hwnd, TOOLBAR_PROP, pthisNew)
#define REMOVETHISDATA(hwnd) \
   RemoveProp(hwnd, TOOLBAR_PROP )


/*prototypes*/
extern void
Toolbr_GetLBxBtnInfo  
(
    	HWND            		hwDlg,
    	HWND            		hwLB,
    	int             		iLBIndx,
    	TBCDLGBTN*      		pBtnInfo
);

extern BOOL 
Tooltip_SetToolbarItemID ( HWND hwTooltip, HWND hwToolbar, UINT newID, LPRECT pnewRect );


static int
Toolbr_AddBitmap
( 
	HWND				hwToolbar,
	TOOLBARDATA*			pthis,
	UINT				nBitmaps,
	LPTBADDBITMAP			lptba
);

static BOOL
Toolbr_AddButtons
( 
	HWND				hwToolbar,
	TOOLBARDATA*			pthis,
	UINT				uButtons,
	LPTBBUTTON			lpButtons
);

static int
Toolbr_CommandToIndex
( 
	HWND				hWnd,
	TOOLBARDATA*			pthis,
	int				idCommand
);

static BOOL
Toolbr_InsertBtn 
( 
	HWND				hwToolbar,
	TOOLBARDATA*			pthis, 
	int				ItmIndx, 
	LPTBBUTTON			lpBtn 
);


static BOOL 
W_IAddImage
( 
	HBITMAP*			phbmImages,
	int*				pImgWidth,
	int*				pImgHeight,
	int*				pImgCount, 
	HBITMAP				hBitmap,
	int				NoOfImages
);

static BOOL
Toolbr_IBeginDrag 
(
	HWND				hWnd,
	TOOLBARDATA*			pthis,
	TLBITEM*			pItem,
	int				idBtn
);

static BOOL 
Toolbr_IBeginPaintBtn
(
	TOOLBARDATA*			pthis, 
	TLBGDI*				ptbg
);

static BOOL
Toolbr_IEndDrag 
(
	HWND				hWnd,
	TOOLBARDATA*			pthis,
	BOOL*				pbInsert
);

static void 
Toolbr_IEndPaintBtn
(
	TOOLBARDATA*			pthis, 
	TLBGDI*				ptbg
);

static void
Toolbr_IResetLayout
(
	HWND				hWnd,
	TOOLBARDATA*			pthis,
	TLBHITTEST			tlbht
);

static void
Toolbr_IResetCheckGroup
(
	HWND				hWnd,
	TLBITEM*			pToolBItmList,
	int				nSelected,
	int				nBtnCount
);


static void
Toolbr_SetTooltip 
( 
	HWND				hWnd, 
	HWND				hwTTip 
);


static void
Toolbr_IPaintBtn 
( 
	HWND				hWnd,
	HDC			    	hDC, 
	TOOLBARDATA*			pthis, 
	TLBITEM*			pItem,
	TLBSTR*				pStr
);

static void
Toolbr_IPaintDragLbxItem 
( 
	HWND				hWnd,
	LPDRAWITEMSTRUCT		lpdi, 
	TOOLBARDATA*			pthis, 
	TLBITEM*			pItem,
	LPSTR				pStr
);



static void
Toolbr_ISetItemRects 
( 
	HWND				hwToolbar,
	TOOLBARDATA*			pthis
);

static int
Toolbr_IBtnFromPt 
( 
	HWND				hwToolbar,
	TOOLBARDATA*			pthis, 
	int			    	x,
	int			    	y,
	TLBHITTEST*			pHitTest
);

static LRESULT  
Toolbar_WndProc
(
	HWND				hWnd,
    	UINT				uMessage,
    	WPARAM				wParam,
    	LPARAM				lParam
);

/*Win32 API */
HWND
WCreateToolbarEx
(
	HWND				hWnd,
	DWORD				ws,
	UINT				wID,
	int				nBitmaps,
	HINSTANCE			hBMInst,
	UINT				wBMID,
	LPCTBBUTTON			lpButtons,
	int				iNumButtons,
	int				dxButton,
	int				dyButton,
	int				dxBitmap,
	int				dyBitmap,
	UINT				uStructSize
)
{
	HWND				hwToolbar;
	TBADDBITMAP			tba;
	HGLOBAL				hToolbar	= ( HGLOBAL )NULL;
	TOOLBARDATA*			pthis		= ( TOOLBARDATA* )NULL;

	int				ErrorCode	= 0;

    	ws |= WS_CHILD;
	ws &= ~WS_BORDER;
	hwToolbar = CreateWindowEx ( 0,WTOOLBARCLASSNAME, "Willows Toolbar", ws, 
			0, 0, 0, 0, hWnd, ( HMENU )wID,
			( HINSTANCE )GetWindowLong ( hWnd, GWL_HINSTANCE ), NULL );
	if ( hwToolbar )
	{
		if ( ! ( hToolbar = GETTHISDATA ( hwToolbar ) ) )
			ErrorCode = WTLB_ERR_GETWINPROP;
		else
		if ( ! ( pthis = ( TOOLBARDATA *) GlobalLock ( hToolbar ) ) )
			ErrorCode = WTLB_ERR_GLOBALLOCK;
		else
		{
			pthis->ImgCX = dxBitmap;
			pthis->ImgCY = dyBitmap;

                        /* FIXME: Use this for ItmHeight? */
			if ( dxButton == 0 )
				dxButton = dxBitmap;
			if ( dyButton == 0 )
				dyButton = dyBitmap;

			tba.hInst = hBMInst;
			tba.nID = wBMID;
			if ( wBMID == IDB_STD_LARGE_COLOR  || 
		     	     wBMID == IDB_STD_SMALL_COLOR  ||
		     	     wBMID == IDB_VIEW_LARGE_COLOR || 
		     	     wBMID == IDB_VIEW_SMALL_COLOR
		   	   )
			pthis->bStandardBmp = TRUE;
			SendMessage(hwToolbar,TB_BUTTONSTRUCTSIZE,
				sizeof(TBBUTTON),0);

			Toolbr_AddBitmap ( hwToolbar, pthis, nBitmaps, &tba ); 
			Toolbr_AddButtons ( hwToolbar, pthis, iNumButtons, 
				( LPTBBUTTON )lpButtons );
		}

	}

	return hwToolbar;

}
HBITMAP CreateMappedBitmap(
    HINSTANCE hInstance, 	
    int idBitmap, 	
    UINT wFlags, 	
    LPCOLORMAP lpColorMap, 	
    int iNumMaps	
   )
{
	APISTR((LF_APISTUB,"CreateMappedBitmap(...)\n"));
	return 0;
}

/* Tool bar Implementation*/

static int
Toolbr_AddBitmap
( 
	HWND				hwToolbar,
	TOOLBARDATA*			pthis,
	UINT				nBitmaps,
	LPTBADDBITMAP			lptba
)
{
	int				tmp = ( int )lptba->hInst;
	HINSTANCE			hBMInst = lptba->hInst;
	UINT				wBMID	= lptba->nID;
	HBITMAP				hBitmap = ( HBITMAP )NULL;
	int				Index	= -1;

	tmp = HINST_COMMCTRL;
	if ( hBMInst == HINST_COMMCTRL )
	{
		hBMInst = hLibInstance;

		if ( wBMID == IDB_STD_LARGE_COLOR  || 
		     wBMID == IDB_STD_SMALL_COLOR
		   )
			nBitmaps = 15;
		if ( wBMID == IDB_VIEW_LARGE_COLOR  || 
		     wBMID == IDB_VIEW_SMALL_COLOR
		   )
			nBitmaps = 12;
		if ( wBMID == IDB_STD_LARGE_COLOR  || 
		     wBMID == IDB_VIEW_LARGE_COLOR
		   )
		{
			pthis->ImgCX = LARGEIMAGE_CX;
			pthis->ImgCY = LARGEIMAGE_CY;
		}
		else
		if ( wBMID == IDB_STD_SMALL_COLOR  || 
		     wBMID == IDB_VIEW_SMALL_COLOR
		   )
		{
			pthis->ImgCX = SMALLIMAGE_CX;
			pthis->ImgCY = SMALLIMAGE_CY;
		}

		if ( wBMID == IDB_STD_SMALL_COLOR )
			wBMID = IDB_STDSMALLCOLOR;
	}

	if ( hBMInst ) /* need refinement to eliminate imagelist */
		hBitmap = LoadBitmap ( hBMInst, MAKEINTRESOURCE ( wBMID ) );
	else
		hBitmap = ( HWND )wBMID;

	if ( pthis->hbmImage == ( HBITMAP )NULL )
	{
		pthis->hbmImage = hBitmap;
		pthis->ImgCount = nBitmaps;
		Index = 0;
		if ( pthis->ImgCX <= 0 || pthis->ImgCY <= 0 )
		{
			BITMAP Bitmap;
			GetObject ( hBitmap, sizeof ( BITMAP ), &Bitmap );
			pthis->ImgCX = Bitmap.bmWidth / nBitmaps;
			pthis->ImgCY = Bitmap.bmHeight;
		}
	}
	else
	{
		Index = pthis->ImgCount; 
		W_IAddImage ( &pthis->hbmImage, &pthis->ImgCX, &pthis->ImgCY, 
				&pthis->ImgCount, hBitmap, nBitmaps );
		DeleteObject ( hBitmap );
	}

	return Index;
}

static BOOL
Toolbr_AddButtons
( 
	HWND				hwToolbar,
	TOOLBARDATA*			pthis,
	UINT				uButtons,
	LPTBBUTTON			lpButtons
)
{
	int				i;
	int				Count;
	
	if ( !lpButtons )
		return FALSE;
	
	Count = pthis->ItmCount;
	for ( i = 0; i < ( int )uButtons; i++, Count++ )
	{
		Toolbr_InsertBtn ( hwToolbar, pthis, Count, lpButtons + i );
	}

	pthis->ItmCount = Count;

	return TRUE;
}

static int
Toolbr_AddStrings
( 
	HWND				hwToolbar,
	TOOLBARDATA*			pthis,
	HINSTANCE			hInst,
	LPSTR				lpStrings
)
{
	int			    	i;
	int				Count;
	int				Index;
	int				Len;
	SIZE				Size;
	char				szLabel[MAX_LABELLENGTH];
	char*				pLabel		= ( char* )NULL;
	HDC				hDC		= GetDC ( hwToolbar );
	HFONT				hOldFont	= ( HFONT )NULL;

	
	if ( !lpStrings )
		return -1;

	Index = pthis->StrCount;
	
	memset ( &Size, 0, sizeof ( SIZE ) );
	if ( hInst )
	{
		if ( LoadString ( hInst, LOWORD ( lpStrings ), szLabel, MAX_LABELLENGTH ) == 0 )
			return -1;
		WLInsert ( &pthis->StrList, Index, szLabel);
		return Index;
	}

	pLabel = lpStrings;
	if(pthis->hFont)
		hOldFont = SelectObject ( hDC, pthis->hFont );
	for ( i = 0, Count = Index; ; i++, Count++ )
	{
		if ( *pLabel == '\0' )
			break;
		if ( WLInsert ( &pthis->StrList, Count, pLabel) )
			break;
		Len = lstrlen ( pLabel );

                /* JPW Experiment - HACK - Don't make it a minum of 20 chars */
		/* Len = min ( sizeof ( TLBSTR ), Len );   */

		GetTextExtentPoint32 ( hDC, pLabel, Len, &Size );
		if ( pthis->ItmWidth <= Size.cx )
			pthis->ItmWidth = Size.cx + HORZ_PADDING;
		while ( *pLabel++ );
	}
	
	pthis->StrCount = Count;

	if(hOldFont)
		SelectObject ( hDC, hOldFont );
	ReleaseDC ( hwToolbar, hDC );
	return Index;
}

static void
Toolbr_AutoSize
( 
	HWND				hwToolbar,
	TOOLBARDATA*			pthis
)
{
	Toolbr_ISetItemRects ( hwToolbar, pthis );

	InvalidateRect ( hwToolbar, NULL, TRUE );
}


static BOOL
Toolbr_ChangeBitmap
( 
	HWND				hwToolbar,
	TOOLBARDATA*			pthis,
	int				idButton,
	int				iBitmap
)
{
	TLBITEM*			pToolBItmList	= ( TLBITEM* )NULL;
	int				ErrorCode	= 0;

	if ( iBitmap >= pthis->ImgCount )
		ErrorCode = WTLB_ERR_OUTOFRANGE;	
	else
	if ( ( idButton = Toolbr_CommandToIndex ( hwToolbar, pthis, idButton ) ) == -1 )
		ErrorCode = WTLB_ERR_OUTOFRANGE;	
	else
	if ( WLLock ( &pthis->ItmList, ( LPVOID )&pToolBItmList ) )
		ErrorCode = WTLB_ERR_LISTLOCK;
	else
	{
		( pToolBItmList + idButton )->Btn.iBitmap = iBitmap;
	}


	if ( pToolBItmList )
		WLUnlock ( &pthis->ItmList, ( LPVOID )&pToolBItmList );
	
	return ( ErrorCode >= 0 );
}

static BOOL
Toolbr_CheckBtn
( 
	HWND				hWnd,
	TOOLBARDATA*			pthis,
	int				idButton,
	BOOL				fCheck
)
{
	TLBITEM*			pToolBItmList	= ( TLBITEM* )NULL;
	TLBITEM*			pItem		= ( TLBITEM* )NULL;
	int				BtnChecked;
	int				ErrorCode	= 0;
	if ( pthis->ItmCount == 0)
		ErrorCode = WTLB_ERR_OUTOFRANGE;	

	else
	if ( ( BtnChecked = Toolbr_CommandToIndex ( hWnd, pthis, idButton ) ) == -1 )
		ErrorCode = WTLB_ERR_OUTOFRANGE;
	else
	if ( WLLock ( &pthis->ItmList, ( LPVOID )&pToolBItmList ) )
		ErrorCode = WTLB_ERR_LISTLOCK;
	else
	{
		RECT	rcBtn;
		pthis->ItmSelected = BtnChecked;
		pItem = pToolBItmList + BtnChecked;
		if ( pItem && pItem->Btn.fsStyle & TBSTYLE_CHECK )
		{
			if ( fCheck )
			{
				if ( pItem->Btn.fsStyle & TBSTYLE_GROUP )
					Toolbr_IResetCheckGroup ( hWnd, pToolBItmList, BtnChecked, 
							pthis->ItmCount );
				pItem->Btn.fsState |= TBSTATE_PRESSED;
			}
			else
				pItem->Btn.fsState &= ~TBSTATE_PRESSED;

			CopyRect ( &rcBtn, &pItem->Rect );
			InflateRect ( &rcBtn, 1, 1 );
			InvalidateRect ( hWnd, &rcBtn, TRUE );

			SendMessage ( GetParent ( hWnd ), WM_COMMAND, 
				MAKELONG ( pItem->Btn.idCommand, 0 ), 0L );
		}
	}


	if ( pToolBItmList )
		WLUnlock ( &pthis->ItmList, ( LPVOID )&pToolBItmList );
	
	return ( ErrorCode >= 0 );
}


static int
Toolbr_CommandToTextIndex
( 
	HWND				hWnd,
	TOOLBARDATA*			pthis,
	int				idCommand
)
{
	int				j = 0;
	int				i;
	TLBITEM*			pToolBItmList	= ( TLBITEM* )NULL;
	int				ErrorCode	= 0;

	if ( WLLock ( &pthis->ItmList, ( LPVOID )&pToolBItmList ) )
		ErrorCode = WTLB_ERR_LISTLOCK;
	else
	{
		for ( i = 0, j = -1; i < pthis->ItmCount; i++ )
		{
			if ( ( pToolBItmList + i )->Btn.idCommand > 0 )
				j++;
			if ( ( pToolBItmList + i )->Btn.idCommand == idCommand )
				break;
		}
	}


	if ( pToolBItmList )
		WLUnlock ( &pthis->ItmList, ( LPVOID )&pToolBItmList );
	
	return ( j < pthis->ItmCount ? j : -1 );
}

static int
Toolbr_CommandToIndex
( 
	HWND				hWnd,
	TOOLBARDATA*			pthis,
	int				idCommand
)
{
	int				i = -1;
	TLBITEM*			pToolBItmList	= ( TLBITEM* )NULL;
	int				ErrorCode    	= 0;

	if (pthis->ItmCount == 0) 
		ErrorCode = WTLB_ERR_OUTOFRANGE;
        else	{
	    if ( WLLock ( &pthis->ItmList, ( LPVOID )&pToolBItmList ) )
		ErrorCode = WTLB_ERR_LISTLOCK;
	    else
	    {
		for ( i = 0; i < pthis->ItmCount; i++ )
		{
			if ( ( pToolBItmList + i )->Btn.idCommand == idCommand )
				break;
		}
	    }
	}


	if ( pToolBItmList )
		WLUnlock ( &pthis->ItmList, ( LPVOID )&pToolBItmList );
	
	return ( i < pthis->ItmCount ? i : -1 );
}

static BOOL
Toolbr_DeleteBtn
(
	HWND			hwToolbar,
	TOOLBARDATA*		pthis, 
	int			iButton 
)
{
	BOOL				bRedraw		    = FALSE;
	int				    ErrorCode	    = -1;

	if ( pthis->ItmCount == 0)
		return FALSE;

	if ( iButton < 0 || iButton >= pthis->ItmCount )
        	return FALSE;

    	if ( WLDelete ( &pthis->ItmList, iButton ) == 0 )
    	{
		pthis->ItmCount--;
		ErrorCode = 0;
		bRedraw = TRUE;
    	}

	if ( bRedraw )
	{
		Toolbr_ISetItemRects ( hwToolbar, pthis );
		InvalidateRect ( hwToolbar, NULL, TRUE );
	}
	
	return ( ErrorCode >= 0 );
}
/* to be called from within the customize dialogbox WM_DRAWITEM handler */
void
Toolbr_DrawDragLBxItem 
( 
    	HWND                hwDlg,
    	HWND                hwToolbar, 
    	LPDRAWITEMSTRUCT    lpdi,
    	LPSTR               pText
)
{
    	int                 index;
	HGLOBAL		    hToolbar	    = ( HGLOBAL )NULL;
	TOOLBARDATA*	    pthis	    = ( TOOLBARDATA* )NULL;
	TLBITEM*	    pToolBItmList   = ( TLBITEM* )NULL;
	TLBITEM*	    pItem	    = ( TLBITEM* )NULL;
	int				    ErrorCode	    = 0;
	
	if ( ! ( hToolbar = GETTHISDATA ( hwToolbar ) ) )
		ErrorCode = WTLB_ERR_GETWINPROP;
	else
	if ( ! ( pthis = ( TOOLBARDATA *) GlobalLock ( hToolbar ) ) )
		ErrorCode = WTLB_ERR_GLOBALLOCK;
	else
	if ( WLLock ( &pthis->ItmList, ( LPVOID )&pToolBItmList ) )
		ErrorCode = WTLB_ERR_LISTLOCK;
	else
    	{
        	TLBGDI  tbg;
        	TBCDLGBTN   BtnInfo;
        	Toolbr_GetLBxBtnInfo ( hwDlg, lpdi->hwndItem, lpdi->itemID, &BtnInfo );
        	index = Toolbr_CommandToIndex ( hwToolbar, pthis, BtnInfo.iCmdID );
        	pItem = index != -1 ? pToolBItmList + index : NULL;
        	Toolbr_IBeginPaintBtn ( pthis, &tbg );
        	Toolbr_IPaintDragLbxItem ( hwToolbar, lpdi, pthis, pItem, BtnInfo.Text );
        	Toolbr_IEndPaintBtn ( pthis, &tbg );
    	}

    	if ( pToolBItmList )
		WLUnlock ( &pthis->ItmList, ( LPVOID )&pToolBItmList );
    	if ( pthis )
		GlobalUnlock ( hToolbar );
}

static BOOL
Toolbr_EnableBtn
(
	HWND			hwToolbar,
	TOOLBARDATA*	pthis, 
	int				idButton,
	BOOL			bEnable
)
{
	TLBITEM*		pToolBItmList	= ( TLBITEM* )NULL;
	BOOL			bResult = FALSE;
	int				ErrorCode	= 0;

	if ( ( idButton = Toolbr_CommandToIndex ( hwToolbar, pthis, idButton ) ) == -1 )
		ErrorCode = WTLB_ERR_OUTOFRANGE;
	else
	if ( WLLock ( &pthis->ItmList, ( LPVOID )&pToolBItmList ) )
		ErrorCode = WTLB_ERR_LISTLOCK;
	else
	{
		RECT rcBtn;

                /*------------------------------------------------------------
                **  First, turn off the enabled and indeterminate bits.
                **----------------------------------------------------------*/
		( pToolBItmList + idButton )->Btn.fsState &= ~(TBSTATE_ENABLED | TBSTATE_INDETERMINATE);

                /*------------------------------------------------------------
                **  Then, just turn on the correct bit.
                **----------------------------------------------------------*/
		( pToolBItmList + idButton )->Btn.fsState |= bEnable ? TBSTATE_ENABLED : 
								       TBSTATE_INDETERMINATE;

		CopyRect ( &rcBtn, &( ( pToolBItmList + idButton )->Rect) );
		InflateRect ( &rcBtn, 1, 1 );
		InvalidateRect ( hwToolbar, &rcBtn, TRUE );
		bResult = TRUE;

	}

	if ( pToolBItmList )
		WLUnlock ( &pthis->ItmList, ( LPVOID )&pToolBItmList );
	return ( ErrorCode >= 0 && bResult );
}

/*
 * Note: Microsoft docs state that if unsuccessful, this function would return 0. But 0 is a valid
 *       value for the index of a button's bitmap. So we assume that if unsuccessful, the return
 *       value would be -1, which is more reasonable.
 */
static int
Toolbr_GetBitmap
(
	HWND				hwToolbar,
	TOOLBARDATA*			pthis, 
	int				idButton
)
{
	TLBITEM*			pToolBItmList	= ( TLBITEM* )NULL;
	int				iBitmap		= -1;
	int				ErrorCode	= 0;

	if ( ( idButton = Toolbr_CommandToIndex ( hwToolbar, pthis, idButton ) ) == -1 )
		ErrorCode = WTLB_ERR_OUTOFRANGE;
	else
	if ( WLLock ( &pthis->ItmList, ( LPVOID )&pToolBItmList ) )
		ErrorCode = WTLB_ERR_LISTLOCK;
	else
	{
		iBitmap = ( pToolBItmList + idButton )->Btn.iBitmap;
	}

	if ( pToolBItmList )
		WLUnlock ( &pthis->ItmList, ( LPVOID )&pToolBItmList );
	return iBitmap;
}


static int
Toolbr_GetBitmapFlag
(
	HWND				hwToolbar
)
{
	int				iFlag	= 0;
	HDC				hDC	= GetDC ( ( HWND )NULL );

	if ( GetDeviceCaps ( hDC, LOGPIXELSX ) >= 120 )
		iFlag = TBBF_LARGE;
	ReleaseDC ( ( HWND )NULL, hDC );
	
	return iFlag;
}

static BOOL
Toolbr_GetButton
(
	HWND				hwToolbar,
	TOOLBARDATA*			pthis, 
	int				iButton,
	LPTBBUTTON			lpTBButton
)
{
	TLBITEM*			pToolBItmList	= ( TLBITEM* )NULL;
	BOOL				bResult = FALSE;
	int				ErrorCode	= 0;

	if(pthis->ItmCount == 0)
		ErrorCode = WTLB_ERR_OUTOFRANGE;
	else
	if ( WLLock ( &pthis->ItmList, ( LPVOID )&pToolBItmList ) )
		ErrorCode = WTLB_ERR_LISTLOCK;
	else
	if ( lpTBButton && iButton >= 0 && iButton <= pthis->ItmCount )
	{
		*lpTBButton =  ( pToolBItmList + iButton )->Btn;
		bResult = TRUE;
	}

	if ( pToolBItmList )
		WLUnlock ( &pthis->ItmList, ( LPVOID )&pToolBItmList );

	return ( ErrorCode >= 0 && bResult );
}

static int
Toolbr_GetButtonText
(
	HWND				hwToolbar,
	TOOLBARDATA*			pthis, 
	int				idButton,
	LPSTR				lpBtnText
)
{
	TLBSTR*				pToolBItmText	= ( TLBSTR* )NULL;
	TLBSTR*				pItmText	= ( TLBSTR* )NULL;
	int				len		= -1;
	LPVOID				lpText		= NULL;
	int				ErrorCode	= 0;

	if ( pthis->StrCount <= 0 )
		return -1;
	if ( ( idButton = Toolbr_CommandToTextIndex ( hwToolbar, pthis, idButton ) ) == -1 )
		ErrorCode = WTLB_ERR_OUTOFRANGE;
	else {
		if ( WLLock ( &pthis->StrList, ( LPVOID )&pToolBItmText ) )
			ErrorCode = WTLB_ERR_LISTLOCK;
		else
		if ( lpBtnText && idButton <= pthis->StrCount )
		{
			pItmText = pToolBItmText + idButton;
			lpText = lstrcpyn( lpBtnText, ( LPSTR )pItmText, sizeof ( TLBSTR ) );
		}
	}
	if ( pToolBItmText )
		WLUnlock ( &pthis->StrList, ( LPVOID )&pToolBItmText );

	if ( lpText )
		len = lstrlen ( ( LPSTR ) lpText );
	return len;
}


static BOOL
Toolbr_GetItemRect
(
	HWND				hwToolbar,
	TOOLBARDATA*			pthis, 
	int				idButton,
	LPRECT				lpRect
)
{
	TLBITEM*			pToolBItmList	= ( TLBITEM* )NULL;
	BOOL				bResult		= FALSE;
	int				ErrorCode	= 0;

 	if (pthis->ItmCount == 0)
		return 0;

	if ( WLLock ( &pthis->ItmList, ( LPVOID )&pToolBItmList ) )
		ErrorCode = WTLB_ERR_LISTLOCK;
	else
	if ( !( ( pToolBItmList + idButton )->Btn.fsState & TBSTATE_HIDDEN ) && 
	     lpRect && idButton >= 0 && idButton <= pthis->ItmCount )
	{
		CopyRect ( lpRect, &( ( pToolBItmList + idButton )->Rect ) );
		bResult = TRUE;
	}

	if ( pToolBItmList )
		WLUnlock ( &pthis->ItmList, ( LPVOID )&pToolBItmList );

	return ( ErrorCode >= 0 && bResult );
}

static BYTE
Toolbr_GetState
(
	HWND				hwToolbar,
	TOOLBARDATA*			pthis, 
	int				idButton
)
{
	TLBITEM*			pToolBItmList	= ( TLBITEM* )NULL;
	int				State		= -1;
	int				ErrorCode	= 0;

	if ( ( idButton = Toolbr_CommandToIndex ( hwToolbar, pthis, idButton ) ) == -1 )
		ErrorCode = WTLB_ERR_OUTOFRANGE;
	else
	if ( WLLock ( &pthis->ItmList, ( LPVOID )&pToolBItmList ) )
		ErrorCode = WTLB_ERR_LISTLOCK;
	else
		State =  ( pToolBItmList + idButton )->Btn.fsState;

	if ( pToolBItmList )
		WLUnlock ( &pthis->ItmList, ( LPVOID )&pToolBItmList );

	return State;
}

static BOOL
Toolbr_HideBtn
(
	HWND				hwToolbar,
	TOOLBARDATA*			pthis, 
	int				idButton,
	BOOL				bHide
)
{
	TLBITEM*			pToolBItmList	    = ( TLBITEM* )NULL;
	TLBITEM*			pItem		    = ( TLBITEM* )NULL;
	BOOL				bSkip		    = FALSE;
	int				ErrorCode	    = 0;

	if ( ( idButton = Toolbr_CommandToIndex ( hwToolbar, pthis, idButton ) ) == -1 )
		ErrorCode = WTLB_ERR_OUTOFRANGE;
	else
	if ( WLLock ( &pthis->ItmList, ( LPVOID )&pToolBItmList ) )
		ErrorCode = WTLB_ERR_LISTLOCK;
	else
    	if ( idButton < pthis->ItmCount )
	{
		pItem = pToolBItmList + idButton;
        	bSkip = ( bHide && pItem->Btn.fsState & TBSTATE_HIDDEN ) ||
		        ( !bHide && !( pItem->Btn.fsState & TBSTATE_HIDDEN ) );
		if ( !bSkip )
		{
			pItem->Btn.fsState ^= TBSTATE_HIDDEN;
		}
	}

	if ( pToolBItmList )
		WLUnlock ( &pthis->ItmList, ( LPVOID )&pToolBItmList );
	if ( !bSkip )
	{
		Toolbr_ISetItemRects ( hwToolbar, pthis );
		InvalidateRect ( hwToolbar, NULL, TRUE );
	}
	return ( ErrorCode >= 0 );
}

static BOOL
Toolbr_InDeterminate
(
	HWND				hwToolbar,
	TOOLBARDATA*			pthis, 
	int				idButton,
	BOOL				bInDeterminate
)
{
	RECT				rcBtn;
	TLBITEM*			pToolBItmList	= ( TLBITEM* )NULL;
	TLBITEM*			pItem		= ( TLBITEM* )NULL;
	BOOL				bSkip		= FALSE;

	int				ErrorCode	= 0;

	if ( ( idButton = Toolbr_CommandToIndex ( hwToolbar, pthis, idButton ) ) == -1 )
		ErrorCode = WTLB_ERR_OUTOFRANGE;
	else
	if ( WLLock ( &pthis->ItmList, ( LPVOID )&pToolBItmList ) )
		ErrorCode = WTLB_ERR_LISTLOCK;
	else
	{
		pItem = pToolBItmList + idButton;
		bSkip = ( bInDeterminate && pItem->Btn.fsState & TBSTATE_INDETERMINATE ) ||
		        ( !bInDeterminate && !( pItem->Btn.fsState & TBSTATE_INDETERMINATE ) );
		if ( !bSkip )
		{
			pItem->Btn.fsState ^= TBSTATE_INDETERMINATE;
			CopyRect ( &rcBtn, &( pItem->Rect) );
			InflateRect ( &rcBtn, 1, 1 );
		}
	}

	if ( pToolBItmList )
		WLUnlock ( &pthis->ItmList, ( LPVOID )&pToolBItmList );
	if ( !bSkip )
	{
		InvalidateRect ( hwToolbar, NULL, TRUE );
	}
	return ( ErrorCode >= 0 );
}

static BOOL
Toolbr_PressBtn
(
	HWND				hwToolbar,
	TOOLBARDATA*			pthis, 
	int				idButton,
	BOOL				bPress
)
{
	RECT				rcBtn;
	TLBITEM*			pToolBItmList	= ( TLBITEM* )NULL;
	TLBITEM*			pItem		= ( TLBITEM* )NULL;
	BOOL				bSkip		= FALSE;

	int				ErrorCode	= 0;

	if ( ( idButton = Toolbr_CommandToIndex ( hwToolbar, pthis, idButton ) ) == -1 )
		ErrorCode = WTLB_ERR_OUTOFRANGE;
	else
	if ( WLLock ( &pthis->ItmList, ( LPVOID )&pToolBItmList ) )
		ErrorCode = WTLB_ERR_LISTLOCK;
	else
	{
		pItem = pToolBItmList + idButton;
		bSkip = ( bPress && pItem->Btn.fsState & TBSTATE_PRESSED ) ||
		        ( !bPress && !( pItem->Btn.fsState & TBSTATE_PRESSED ) );
		if ( !bSkip )
		{
			pItem->Btn.fsState ^= TBSTATE_PRESSED;
			CopyRect ( &rcBtn, &( pItem->Rect) );
			InflateRect ( &rcBtn, 1, 1 );
		}
	}

	if ( pToolBItmList )
		WLUnlock ( &pthis->ItmList, ( LPVOID )&pToolBItmList );

        if (!bSkip) {
		InvalidateRect(hwToolbar,&rcBtn,TRUE);
	}
	return ( ErrorCode >= 0 );
}

static BOOL
Toolbr_SetBMPSize
(
	HWND				hwToolbar,
	TOOLBARDATA*			pthis, 
	int				Width,
	int				Height
)
{
        pthis->ImgCX = Width;
        pthis->ImgCY = Height;
	return TRUE;
}

static BOOL
Toolbr_SetBtnSize
(
	HWND				hwToolbar,
	TOOLBARDATA*			pthis, 
	int				Width,
	int				Height
)
{
    if (Height > 0)
	pthis->ItmHeight = Height;

    if (Width > 0)
	pthis->ItmWidth  = Width;

    return TRUE;
}


static BOOL
Toolbr_QueryStat
(
	HWND				hwToolbar,
	TOOLBARDATA*			pthis, 
	UINT				uMessage,
	int				idButton
)
{
	TLBITEM*			pToolBItmList	= ( TLBITEM* )NULL;
	TLBITEM*			pItem		= ( TLBITEM* )NULL;
	int				State		= 0;
	BOOL				bResult		= FALSE;

	int				ErrorCode	= 0;

	if ( ( idButton = Toolbr_CommandToIndex ( hwToolbar, pthis, idButton ) ) == -1 )
		ErrorCode = WTLB_ERR_OUTOFRANGE;
	else
	if ( WLLock ( &pthis->ItmList, ( LPVOID )&pToolBItmList ) )
		ErrorCode = WTLB_ERR_LISTLOCK;
	else
	{
		State = uMessage == TB_ISBUTTONCHECKED ? TBSTATE_CHECKED :	
			uMessage == TB_ISBUTTONENABLED ? TBSTATE_ENABLED :	
			uMessage == TB_ISBUTTONHIDDEN  ? TBSTATE_HIDDEN :	
			uMessage == TB_ISBUTTONINDETERMINATE ? TBSTATE_INDETERMINATE :	
			uMessage == TB_ISBUTTONPRESSED ? TBSTATE_PRESSED :
			uMessage == TB_ISBUTTONHIGHLIGHTED ? TBSTATE_HIGHLIGHTED : 0;
		bResult = pItem->Btn.fsState & State;
	}

	if ( pToolBItmList )
		WLUnlock ( &pthis->ItmList, ( LPVOID )&pToolBItmList );
	return bResult;
}

static BOOL
Toolbr_SetCmdID
(
	HWND				hwToolbar,
	TOOLBARDATA*			pthis, 
	UINT				indx,
	UINT				cmdID
)
{
	BOOL				bResult		= FALSE;
	TLBITEM*			pToolBItmList	= ( TLBITEM* )NULL;

	int				ErrorCode	= 0;

	if(pthis->ItmCount == 0)
		return 0;

	else if ( WLLock ( &pthis->ItmList, ( LPVOID )&pToolBItmList ) )
		ErrorCode = WTLB_ERR_LISTLOCK;
	else
	if ( indx >= 0 && indx <= ( UINT )pthis->ItmCount )
	{
		( pToolBItmList + indx )->Btn.idCommand = cmdID;
		bResult = TRUE;
	}

	if ( pToolBItmList )
		WLUnlock ( &pthis->ItmList, ( LPVOID )&pToolBItmList );

	return ( bResult );
}

static void
Toolbr_SetRows 
( 
	HWND				hwToolbar,
	TOOLBARDATA*			pthis,
	int				cRows,
	BOOL				bLarger,
	LPRECT				lpRect
)
{
	int				i;
	int				ItmsPerRow;
	int				Count		= 0;
	TLBITEM*			pItemList	= ( TLBITEM* )NULL;
	TLBITEM*			pItem		= ( TLBITEM* )NULL;

	if(pthis->ItmCount == 0)
		return;

	ItmsPerRow = pthis->ItmCount / cRows;
	Count = pthis->ItmCount;
	if ( WLLock ( &pthis->ItmList, ( LPVOID )&pItemList ) )
		return;

	for ( i = 0; i < Count; i++ )
	{
		pItem = pItemList + i;
		if ( ( pItem->Btn.fsState & TBSTATE_HIDDEN ) ||
		     ( pItem->Btn.fsStyle & TBSTYLE_SEP ) 
		   )
			continue;
		if ( ( i + 1 ) % ItmsPerRow == 0 )
			pItem->Btn.fsState |= TBSTATE_WRAP;
	}

	pthis->Rows = cRows;

        /*--------------------------------------------------------------------
        **  The height is the size of a row (ItmHeight + VERT_PADDING),
        **      times the number of rows, plus the total window
        **      padding
        **------------------------------------------------------------------*/
	pthis->Size.cy = pthis->Rows * (pthis->ItmHeight + VERT_PADDING )
                                     + (2 * pthis->nYPadding);

	if ( lpRect )
	{
		SetRect ( lpRect, pthis->Pos.x, pthis->Pos.y, 
			pthis->Pos.x + pthis->Size.cx, 
			pthis->Pos.y + pthis->Size.cy );
	}
		/*CopyRect ( lpRect, &pthis->Rect );*/
	if ( pItemList )
		WLUnlock ( &pthis->ItmList, ( LPVOID )&pItemList );

}


static BOOL 
Toolbr_IBeginPaintBtn
(
	TOOLBARDATA*		pthis, 
	TLBGDI*				ptbg
)
{

	ptbg->hbmOldGlyphs = ( HBITMAP )SelectObject ( pthis->hDCGlyphs, pthis->hbmImage );
	ptbg->hbmMono = CreateBitmap ( pthis->ImgCX, pthis->ImgCY, 1, 1, NULL );
	ptbg->hbmMonoOld = ( HBITMAP )SelectObject ( pthis->hDCMono, ptbg->hbmMono);
	if ( ptbg->hbmOldGlyphs == ( HBITMAP )NULL || 
         ptbg->hbmMono == ( HBITMAP )NULL || 
         ptbg->hbmMonoOld == ( HBITMAP )NULL )
	{
		if ( ptbg->hbmMono)
			DeleteObject ( ptbg->hbmMono );
		return FALSE;
	}
	return TRUE;
}

static void 
Toolbr_IEndPaintBtn
(
	TOOLBARDATA*		pthis, 
	TLBGDI*				ptbg
)
{
	SelectObject ( pthis->hDCMono, ptbg->hbmMonoOld );
	DeleteObject ( ptbg->hbmMono );
	SelectObject ( pthis->hDCGlyphs, ptbg->hbmOldGlyphs );
}


static BOOL
Toolbr_InsertBtn 
( 
	HWND				hwToolbar,
	TOOLBARDATA*			pthis, 
	int				ItmIndx, 
	LPTBBUTTON			lpBtn 
)
{
	TLBITEM				tlbi;
	UINT				Count		= 0;
	int				ErrorCode	= 0;

	memset ( &tlbi, 0, sizeof ( TLBITEM ) );
	tlbi.Btn = *lpBtn;
	if ( lpBtn->fsStyle & TBSTYLE_SEP && lpBtn->iBitmap <= 0 )
	{
		tlbi.Btn.iBitmap = 3 * HORZ_PADDING;
	}		
	WLCount ( &pthis->ItmList, &Count );
	if ( ItmIndx > Count )
		ItmIndx = Count;

	if ( WLInsert ( &pthis->ItmList, ItmIndx, &tlbi ) < 0 )
		ErrorCode = WTLB_ERR_LISTINSERT;
	else
	{
		pthis->ItmCount++;
        	pthis->bNeedSetRects = TRUE;
		Toolbr_ISetItemRects ( hwToolbar, pthis );
	}
	

	return  ( ErrorCode >= 0 );
}

static void
Toolbr_Resize_Toolbar(HWND hWnd, TOOLBARDATA *pthis, 
                      int suggested_x, int suggested_y)
{
    HWND hParent;
    RECT rect;
    RECT parentrect;
    POINT origin;
    int  toolbar_height = pthis->ItmHeight + 2 * pthis->nYPadding;
    int  toolbar_width = pthis->ItmWidth + 2 * pthis->nXPadding;
    int  oldcy;
    int  oldcx;
    BOOL bSizeChanged = FALSE;


    /*------------------------------------------------------------
    **  Get our current client rectangle
    **----------------------------------------------------------*/
    GetClientRect ( hWnd , &rect );

    /*------------------------------------------------------------
    ** If they have not disabled alignment, then align
    **    our rectangle with their parent, presuming it hasn't
    **    been done already.
    **----------------------------------------------------------*/
    origin = pthis->Pos;
    if ((pthis->Size.cx == 0 || pthis->Size.cy == 0) &&
        ! (pthis->uStyles & (CCS_NOPARENTALIGN | CCS_NORESIZE)))
    {

        hParent = GetParent ( hWnd );
        if (hParent)
        {
            GetClientRect ( hParent, &parentrect );
            if (pthis->uStyles & (CCS_LEFT | CCS_RIGHT))
            {
                pthis->Size.cx = toolbar_width;
                pthis->Size.cy = parentrect.bottom;
                if (pthis->uStyles & CCS_RIGHT)
                    origin.x = parentrect.right - toolbar_width;
            }
            else 
            {
                pthis->Size.cx = parentrect.right;
                pthis->Size.cy = toolbar_height;

                /*------------------------------------------------
                **  CCS_TOP is the default, so check for bottom
                **-----------------------------------------------*/
                if (pthis->uStyles & CCS_BOTTOM)
                    origin.y = parentrect.bottom - toolbar_height;
            }

            bSizeChanged = TRUE;
        }
    }

    /*------------------------------------------------------------
    **  Now, if they still haven't given us a size, let's make
    **    a good guess.
    **----------------------------------------------------------*/
    oldcx = pthis->Size.cx;
    oldcy = pthis->Size.cy;
    if (! bSizeChanged)
    {
        if (pthis->Size.cx == 0 || pthis->Size.cx != rect.right)
        {
            if (rect.right == 0)
            {
                if (suggested_x == 0)
                    pthis->Size.cx = toolbar_width;
                else
                    pthis->Size.cx = suggested_x;
            }
            else
                pthis->Size.cx = rect.right;
        }

        if (pthis->Size.cy == 0 || pthis->Size.cy != rect.bottom)
        {
            if (rect.bottom == 0)
            {
                if (suggested_y == 0)
                    pthis->Size.cy = toolbar_height;
                else
                    pthis->Size.cy = suggested_y;
            }
            else
                pthis->Size.cy = rect.bottom;
        }
    }


    /*------------------------------------------------------------
    **  Finally, adjust the size
    **----------------------------------------------------------*/
    if (bSizeChanged || oldcy != pthis->Size.cy || oldcx != pthis->Size.cx || 
        origin.x != pthis->Pos.x || origin.y != pthis->Pos.y)
    {
        pthis->Pos = origin;

        SetWindowPos ( hWnd, ( HWND )NULL, pthis->Pos.x, pthis->Pos.y,
               pthis->Size.cx + pthis->Pos.x, pthis->Size.cy + pthis->Pos.y,
               SWP_NOACTIVATE | SWP_NOMOVE |SWP_NOZORDER );
    }
}

static BOOL
Toolbr_OnCreate 
(
	HWND				hWnd, 
	LPCREATESTRUCT			pcs
) 
{
	HGLOBAL				hToolbar	= ( HGLOBAL )NULL;
	TOOLBARDATA*			pthis		= ( TOOLBARDATA* )NULL;

	int				ErrorCode	= 0;

	if ( ! ( hToolbar = GlobalAlloc ( GHND, sizeof ( TOOLBARDATA ) ) ) )
		ErrorCode = WTLB_ERR_GLOBALALLOC;
	else
	if ( ! ( pthis = ( TOOLBARDATA *) GlobalLock ( hToolbar ) ) )
		ErrorCode = WTLB_ERR_GLOBALLOCK;
	else
	{
		NONCLIENTMETRICS lpm;
		pthis->uStyles = pcs->style;
        	pthis->uStyles &= ~WS_BORDER;
    		pthis->uExtStyles = pcs->dwExStyle;

        	pthis->StrCount = 0;

                if (pthis->uStyles & CCS_NODIVIDER)
                {
                    pthis->nXPadding = CX_BORDER;
                    pthis->nYPadding = CY_BORDER;
                }
                else
                {
                    pthis->nXPadding = CX_BORDER_DIVIDER;
                    pthis->nYPadding = CY_BORDER_DIVIDER;
                }

		pthis->ImgCX     = TOOLBMP_CX;
		pthis->ImgCY     = TOOLBMP_CY;

		pthis->ItmHeight = TOOLBTN_CX; 
		pthis->ItmWidth  = TOOLBTN_CY;

		pthis->bStandardBmp = FALSE;

		pthis->himl = 0;
		pthis->himlHot = 0;
		
		SystemParametersInfo(SPI_GETNONCLIENTMETRICS, 0, &lpm, 0);
		pthis->hFont = CreateFontIndirect(&lpm.lfMenuFont);
	
		pthis->hDCGlyphs = CreateCompatibleDC ( ( HDC )NULL );
		pthis->hDCMono = CreateCompatibleDC ( ( HDC )NULL );
#if 0
		if ( ( hbmGray = CreateDitherBitmap () ) )
			pthis->hbrDither = CreatePatternBrush(hbmGray);
#endif
		WLCreate ( &pthis->ItmList, sizeof ( TLBITEM ) );
		WLCreate ( &pthis->StrList, sizeof ( TLBSTR ) );
		if ( pthis->uStyles & TBSTYLE_TOOLTIPS )
		{
			pthis->hwTTip = CreateWindowEx ( 0, WC_TOOLTIP, ( LPSTR ) NULL, 
				WS_POPUP | WS_BORDER | TTS_ALWAYSTIP, CW_USEDEFAULT, CW_USEDEFAULT, 
				CW_USEDEFAULT, CW_USEDEFAULT, hWnd, ( HMENU )NULL, 
				( HINSTANCE )GetWindowLong ( hWnd, GWL_HINSTANCE ), NULL ); 

			Toolbr_SetTooltip ( hWnd, pthis->hwTTip );
		}

		SETTHISDATA ( hWnd, hToolbar );

                Toolbr_Resize_Toolbar(hWnd, pthis, 0, 0);
	}

	if ( pthis )
		GlobalUnlock ( hToolbar );
	return ( ErrorCode >= 0 );
}


static void Toolbr_OnLButtonDblClk 
(	
	HWND				hWnd, 
	int				x, 
	int				y, 
	UINT				keyFlags
) 
{
	HGLOBAL				hToolbar	    = ( HGLOBAL )NULL;
	TOOLBARDATA*			pthis		    = ( TOOLBARDATA* )NULL;
	int				ErrorCode	    = 0;
	
	if ( ! ( hToolbar = GETTHISDATA ( hWnd ) ) )
		ErrorCode = WTLB_ERR_GETWINPROP;
	else
	if ( ! ( pthis = ( TOOLBARDATA *) GlobalLock ( hToolbar ) ) )
		ErrorCode = WTLB_ERR_GLOBALLOCK;
	else
	if ( pthis->uStyles & CCS_ADJUSTABLE )
	{
        NMHDR hdr;
        hdr.hwndFrom = hWnd;
        hdr.idFrom = GetDlgCtrlID ( hWnd );
        hdr.code = TBN_BEGINADJUST;
        SendMessage ( GetParent ( hWnd ), WM_NOTIFY, 0, ( LPARAM )&hdr ); 
        /*somehow hWnd would never become the dialog's parent */
        /*so we have to pass it as lParam */
#ifdef _WINDOWS
        DialogBoxParam ( ( HINSTANCE )GetWindowLong ( hWnd, GWL_HINSTANCE ), szCustomizeDlgTemplate,
                    hWnd, ( DLGPROC )Toolbr_CustomizeWndProc, ( LPARAM )hWnd ); 
#else
        DialogBoxParam ( hLibInstance, szCustomizeDlgTemplate,
                    hWnd, ( DLGPROC )Toolbr_CustomizeWndProc, ( LPARAM )hWnd ); 
#endif
        hdr.code = TBN_ENDADJUST;
        SendMessage ( GetParent ( hWnd ), WM_NOTIFY, 0, ( LPARAM )&hdr ); 
	}

    if ( pthis )
		GlobalUnlock ( hToolbar );
}


static void Toolbr_OnLButtonDown 
(	
	HWND				hWnd, 
	int				x, 
	int				y, 
	UINT				keyFlags
) 
{
	unsigned char				cKeyBuf[256];
	HGLOBAL				hToolbar	    = ( HGLOBAL )NULL;
	TOOLBARDATA*			pthis		    = ( TOOLBARDATA* )NULL;
	TLBITEM*			pToolBItmList	    = ( TLBITEM* )NULL;
	TLBITEM*			pItem		    = ( TLBITEM* )NULL;
	int				ErrorCode	    = 0;
	
	if ( ! ( hToolbar = GETTHISDATA ( hWnd ) ) )
		ErrorCode = WTLB_ERR_GETWINPROP;
	else
	if ( ! ( pthis = ( TOOLBARDATA *) GlobalLock ( hToolbar ) ) )
		ErrorCode = WTLB_ERR_GLOBALLOCK;
	else
	if ( WLLock ( &pthis->ItmList, ( LPVOID )&pToolBItmList ) )
		ErrorCode = WTLB_ERR_LISTLOCK;
	else
	if ( ( pthis->ItmSelected = Toolbr_IBtnFromPt ( hWnd, pthis, x, y, NULL ) ) >= 0 )
	{
		RECT	rcBtn;
		pItem = pToolBItmList + pthis->ItmSelected;
		
        	if ( !( pItem->Btn.fsStyle & TBSTYLE_SEP ) )
		{
			if ( pthis->uStyles & TBSTYLE_ALTDRAG )
			{
				cKeyBuf[VK_MENU] = 0;
#ifdef _WINDOWS
                		if ( GetKeyboardState ( cKeyBuf ) )
#else
                		GetKeyboardState ( cKeyBuf ); 
#endif
				    pthis->enDrag = ( cKeyBuf[VK_MENU] & 0x80 )? eShftLBtnDown :
									     eLBtnDown;
			}
			else	
			    pthis->enDrag = keyFlags & MK_SHIFT ? eShftLBtnDown : eLBtnDown;
		}

		if ( pthis->enDrag == eLBtnDown &&
             !( pItem->Btn.fsStyle & TBSTYLE_SEP ) &&
		     !( pItem->Btn.fsState & TBSTATE_INDETERMINATE ) &&
		     ( pItem->Btn.fsState & TBSTATE_ENABLED ) 
		   )
		{
			if ( pItem->Btn.fsStyle & TBSTYLE_GROUP )
			{
				Toolbr_IResetCheckGroup ( hWnd, pToolBItmList, pthis->ItmSelected, 
								pthis->ItmCount );
				pItem->Btn.fsState |= TBSTATE_PRESSED;
			}
			else
			if ( pItem->Btn.fsStyle & TBSTYLE_CHECK )
				pItem->Btn.fsState ^= TBSTATE_PRESSED;
			else
				pItem->Btn.fsState |= TBSTATE_PRESSED;
			CopyRect ( &rcBtn, &pItem->Rect );
			InflateRect ( &rcBtn, 1, 1 );
			InvalidateRect ( hWnd, NULL, FALSE );
		}

	}

	if ( pToolBItmList )
		WLUnlock ( &pthis->ItmList, ( LPVOID )&pToolBItmList );
	if ( pthis )
		GlobalUnlock ( hToolbar );
}

static void Toolbr_OnLButtonUp 
(	
	HWND				hWnd, 
	int				    x, 
	int				    y, 
	UINT				keyFlags
) 
{
	RECT				rcBtn;
	HGLOBAL				hToolbar	    = ( HGLOBAL )NULL;
	TOOLBARDATA*		pthis		    = ( TOOLBARDATA* )NULL;
	TLBITEM*			pToolBItmList	= ( TLBITEM* )NULL;
	TLBITEM*			pItem		    = ( TLBITEM* )NULL;
	BOOL				bSendMessage	= FALSE;
	BOOL				bInsert		    = FALSE;

	int				    ErrorCode	    = 0;
	
	if ( ! ( hToolbar = GETTHISDATA ( hWnd ) ) )
		ErrorCode = WTLB_ERR_GETWINPROP;
	else
	if ( ! ( pthis = ( TOOLBARDATA* ) GlobalLock ( hToolbar ) ) )
		ErrorCode = WTLB_ERR_GLOBALLOCK;
	else
	if ( WLLock ( &pthis->ItmList, ( LPVOID )&pToolBItmList ) )
		ErrorCode = WTLB_ERR_LISTLOCK;
	else
        if ( pthis->ItmSelected >= 0 )
	{
		pItem = pToolBItmList + pthis->ItmSelected;
		if ( !( pItem->Btn.fsState & TBSTATE_INDETERMINATE ) &&
		     ( pItem->Btn.fsState & TBSTATE_ENABLED ) &&
                     !( pItem->Btn.fsStyle & TBSTYLE_SEP )
		   )
		{
			if ( pItem->Btn.fsStyle ==  TBSTYLE_BUTTON )
			{
				pItem->Btn.fsState ^= TBSTATE_PRESSED;
				CopyRect ( &rcBtn, &pItem->Rect );
				InflateRect ( &rcBtn, 1, 1 );
				InvalidateRect ( hWnd, &rcBtn, FALSE );
			}
			bSendMessage = TRUE;
		}

		if ( pthis->enDrag == eLBtnDown )
		{
			ReleaseCapture ();
			pthis->enDrag = eEndDrag;
		}
		if ( pthis->enDrag != eNoDrag )
			bSendMessage = FALSE;

		Toolbr_IEndDrag ( hWnd, pthis, &bInsert );
		if ( bSendMessage )
			SendMessage ( GetParent ( hWnd ), WM_COMMAND, 
				MAKELONG ( pItem->Btn.idCommand, 0 ), 0L );

	}

	if ( pToolBItmList )
		WLUnlock ( &pthis->ItmList, ( LPVOID )&pToolBItmList );
	if ( bInsert )
	{
		TLBHITTEST	tbht;
		Toolbr_IBtnFromPt ( hWnd, pthis, x, y, &tbht ); 
		Toolbr_IResetLayout ( hWnd, pthis, tbht );
	}
	if ( pthis )
		GlobalUnlock ( hToolbar );
}

static void 
Toolbr_OnDestroy 
(	
	HWND				hWnd 
)
{
	HGLOBAL				hToolbar	= ( HGLOBAL )NULL;
	TOOLBARDATA*			pthis		= ( TOOLBARDATA* )NULL;

	int				ErrorCode	= 0;

	if ( ! ( hToolbar = GETTHISDATA ( hWnd ) ) )
		ErrorCode = WTLB_ERR_GETWINPROP;
	else
	if ( ! ( pthis = ( TOOLBARDATA *) GlobalLock ( hToolbar ) ) )
		ErrorCode = WTLB_ERR_GLOBALLOCK;
	else
	{
		REMOVETHISDATA ( hWnd );
		WLDestroy ( &pthis->ItmList );
		if ( pthis->hFont )
			DeleteObject ( pthis->hFont );
		DeleteDC ( pthis->hDCGlyphs );
		DeleteDC ( pthis->hDCMono );

		/*DeleteObject ( pthis->hbrDither );*/
		GlobalFree ( hToolbar );
	}
}

/* to be called by customize dialog WM_MEASUREITEM event handler only */
void 
Toolbr_OnMeasureItem 
(
	HWND			    hWnd, 
	LPMEASUREITEMSTRUCT	lpmi
) 
{
	HGLOBAL				hToolbar	= ( HGLOBAL )NULL;
	TOOLBARDATA*		pthis		= ( TOOLBARDATA* )NULL;
    int                 ErrorCode   = 0;

	if ( ! ( hToolbar = GETTHISDATA ( hWnd ) ) )
		ErrorCode = WTLB_ERR_GETWINPROP;
	else
	if ( ! ( pthis = ( TOOLBARDATA *) GlobalLock ( hToolbar ) ) )
		ErrorCode = WTLB_ERR_GLOBALLOCK;
    else
        lpmi->itemHeight = pthis->ItmHeight + VERT_PADDING;

    if ( pthis )
		GlobalUnlock ( hToolbar );
}

/*----------------------------------------------------------------------------
** Toolbr_OnMouseMove
**  Handle mouse movements.
**
**  Parameters
**      1.  hWnd        Toolbar window
**      2.  wParam      key indicators - currently unused
**      3.  lParam      X and Y coordinates of the point where the mouse is.
**                      it will be -1 when we are actually processing
**                      a WM_CAPTURECHANGED message.
**     
**--------------------------------------------------------------------------*/
static void 
Toolbr_OnMouseMove 
(
	HWND			hWnd, 
	UINT			wParam,
	LPARAM			lParam 
) 
{
	POINT			pt;
	TLBITEM*		pItemList	= ( TLBITEM* )NULL;
	TLBITEM*		pItem		= ( TLBITEM* )NULL;
	HGLOBAL			hToolbar	= ( HGLOBAL )NULL;
	TOOLBARDATA*		pthis		= ( TOOLBARDATA* )NULL;
	int		    	i	    	= 0;
	int		    	ErrorCode	= 0;
        BOOL                    bRepaint          = FALSE;
        MSG                     out_msg;
        BOOL                    bNeedCapture    = FALSE;
        BOOL                    bNeedRelease    = FALSE;

        /*--------------------------------------------------------------------
        **   Optimization:  if the next message in our queue is
        **                  a mouse move item, pretend that this
        **                  event never happened.  Same goes for
        **                  a capture changed message.
        **------------------------------------------------------------------*/
        if (PeekMessage(&out_msg, hWnd, WM_MOUSEMOVE, WM_CAPTURECHANGED, 
                    PM_NOREMOVE))
        {
            if (out_msg.message == WM_MOUSEMOVE ||
                out_msg.message == WM_CAPTURECHANGED)
                return;
        }

	if ( ! ( hToolbar = GETTHISDATA ( hWnd ) ) )
		ErrorCode = WTLB_ERR_GETWINPROP;
	else
	if ( ! ( pthis = ( TOOLBARDATA *) GlobalLock ( hToolbar ) ) )
		ErrorCode = WTLB_ERR_GLOBALLOCK;
	else
	if ( WLLock ( &pthis->ItmList, ( LPVOID )&pItemList ) )
		ErrorCode = WTLB_ERR_LISTLOCK;
	else
	{
		pt.x = LOWORD ( lParam );
		pt.y = HIWORD ( lParam );
		for ( i = 0; i < pthis->ItmCount; i++ )
		{
		    pItem = pItemList + i;

                    /*--------------------------------------------------------
                    **  By default, presume that the button doesn't
                    **      need updating
                    **------------------------------------------------------*/
                    bRepaint = FALSE;

                    /*--------------------------------------------------------
                    **  If this is a legitimate mouse move, and the mouse
                    **    is in our button, try to highlight it.  Note
                    **    that if pt.x == -1 && pt.y == -1, we're
                    **    not really processing a move move, but
                    **    instead we're processing a capture changed
                    **    message, and so we're just turning off highlights.
                    **------------------------------------------------------*/
                    if ( pt.x >= 0 && pt.y >=0 && PtInRect ( &pItem->Rect, pt ))
                    {
                        if (   !( pItem->Btn.fsStyle & TBSTYLE_SEP ) )
			{
                            /*----------------------------------------------------
                            ** If this button is enabled, but not yet
                            **    highlighted, indicate that it is
                            **--------------------------------------------------*/
                            if ((pItem->Btn.fsState & TBSTATE_ENABLED) &&
                                ! (pItem->Btn.fsState & TBSTATE_HIGHLIGHTED))
                            {
                                pItem->Btn.fsState |= TBSTATE_HIGHLIGHTED;
                                bRepaint = TRUE;
                                bNeedCapture = TRUE;
                            }

                            if ( pthis->hwTTip ) 
                            {
                                    MSG msg;
                                    
                                    if ( pthis->ItmTip != pItem->Btn.idCommand )
                                    {
                                            Tooltip_SetToolbarItemID ( pthis->hwTTip, hWnd,
                                                    pItem->Btn.idCommand, &pItem->Rect );
                                            pthis->ItmTip = pItem->Btn.idCommand;
                                    }
                                    msg.hwnd = hWnd;
                                    msg.message = WM_MOUSEMOVE;
                                    msg.wParam  = wParam;
                                    msg.lParam  = lParam;
                                    SendMessage ( pthis->hwTTip, TTM_RELAYEVENT, 0, 
                                        ( LPARAM ) ( LPMSG )&msg); 
                            }

                            if ( pthis->uStyles & CCS_ADJUSTABLE )
                                    Toolbr_IBeginDrag ( hWnd, pthis, pItem, i );
			}
                    }
                    else
                    {
                        /*----------------------------------------------------
                        ** If we've moved out of the area of this button,
                        **   it can't be highlighted anymore
                        **--------------------------------------------------*/
                        if (pItem->Btn.fsState & TBSTATE_HIGHLIGHTED)
			{
                            pItem->Btn.fsState &= ~TBSTATE_HIGHLIGHTED;
                            bRepaint = TRUE;
                            bNeedRelease = TRUE;
                        }

                        /*----------------------------------------------------
                        ** If we've moved out of the area of this button,
                        **   it can't be pressed any longer.
                        **--------------------------------------------------*/
                        if (pItem->Btn.fsState & TBSTATE_PRESSED)
			{
                            pItem->Btn.fsState &= ~TBSTATE_PRESSED;
                            bRepaint = TRUE;
                        }

                    }

                    /*----------------------------------------------------
                    ** If we've changed the rectangle, invalidate it
                    **   to force a repaint
                    **--------------------------------------------------*/
                    if (bRepaint)
                    {
                        RECT rcBtn;
			CopyRect ( &rcBtn, &( pItem->Rect) );
		        InflateRect ( &rcBtn, 1, 1 );
                        RedrawWindow( hWnd, &rcBtn, 0, 
                                RDW_INVALIDATE | RDW_UPDATENOW );
                    }

                    /*--------------------------------------------------------
                    ** If we've started a highlight, grab the capture so
                    **   we know when our window is exited
                    **------------------------------------------------------*/
                    if (bNeedRelease)
                        ReleaseCapture();

                    if (bNeedCapture)
                        SetCapture(hWnd);
		}
	}

	if ( pItemList )
		WLUnlock ( &pthis->ItmList, ( LPVOID )&pItemList );

	if ( pthis )
		GlobalUnlock ( hToolbar );
}

static void Toolbr_OnNotify ( HWND hWnd, UINT uCtlID, LPARAM lParam ) 
{ 

	TLBITEM*			pItemList	= ( TLBITEM* )NULL;
	HGLOBAL				hToolbar	= ( HGLOBAL )NULL;
	TOOLBARDATA*			pthis		= ( TOOLBARDATA* )NULL;
	int				ErrorCode	= 0;

	LPTOOLTIPTEXT			lpttt; 

	if ( ! ( hToolbar = GETTHISDATA ( hWnd ) ) )
		ErrorCode = WTLB_ERR_GETWINPROP;
	else
	if ( ! ( pthis = ( TOOLBARDATA *) GlobalLock ( hToolbar ) ) )
		ErrorCode = WTLB_ERR_GLOBALLOCK;
	else
	if ( WLLock ( &pthis->ItmList, ( LPVOID )&pItemList ) )
		ErrorCode = WTLB_ERR_LISTLOCK;
	else
	if ( ( ( ( LPNMHDR ) lParam )->code ) == TTN_NEEDTEXT ) 
	{ 
		lpttt = ( LPTOOLTIPTEXT )lParam;
		lpttt->hdr.idFrom = pthis->ItmTip;
		lpttt->uFlags &= ~TTF_IDISHWND;
		SendMessage ( GetParent ( hWnd ), WM_NOTIFY, ( WPARAM )pthis->ItmTip,
				( LPARAM )lpttt );
	} 

	if ( pItemList )
		WLUnlock ( &pthis->ItmList, ( LPVOID )&pItemList );
	if ( pthis )
		GlobalUnlock ( hToolbar );
} 

static void Toolbr_DrawBorder
(
	HWND			hWnd,
	HDC			hDC,
	UINT			uStyle
)
{
	RECT				rc;
	HPEN				hbPen	= CreatePen ( PS_SOLID, 1, GetSysColor ( COLOR_BTNSHADOW ) );
	HPEN				hOPen;

	GetClientRect ( hWnd, &rc );
	hOPen = SelectObject ( hDC, hbPen );
	MoveToEx ( hDC, rc.left, rc.top, NULL );
	LineTo ( hDC, rc.right, rc.top );
	SelectObject ( hDC, GetStockObject ( WHITE_PEN ) );
	MoveToEx ( hDC, rc.left, rc.top + 1, NULL );
	LineTo ( hDC, rc.right, rc.top + 1 );
	SelectObject ( hDC, hOPen );
	
	DeleteObject ( hbPen );
}

static void Toolbr_OnPaint 
(
	HWND				hWnd, 
	HDC				hPaintDC 
)
{
	PAINTSTRUCT			ps;
	TLBITEM*			pToolBItmList	= ( TLBITEM* )NULL;
	TLBITEM*			pItem		= ( TLBITEM* )NULL;
	TLBSTR*	 			pToolBStrList	= ( TLBSTR* )NULL;
	TLBSTR*				pStr		= ( TLBSTR* )NULL;
	HGLOBAL				hToolbar	= ( HGLOBAL )NULL;
	TOOLBARDATA*			pthis		= ( TOOLBARDATA* )NULL;
	int				ErrorCode	= 0;


	if(hPaintDC)
		ps.hdc = hPaintDC;
	else
		BeginPaint(hWnd, &ps);

	if ( !IsWindowVisible ( hWnd ) )
		goto EndToolbarPaint;
	if ( ! ( hToolbar = GETTHISDATA ( hWnd ) ) )
		ErrorCode = WTLB_ERR_GETWINPROP;
	else
	if ( ! ( pthis = ( TOOLBARDATA *) GlobalLock ( hToolbar ) ) )
		ErrorCode = WTLB_ERR_GLOBALLOCK;
	else
	if ( WLLock ( &pthis->ItmList, ( LPVOID )&pToolBItmList ) )
		ErrorCode = WTLB_ERR_LISTLOCK;
	else
	if ( WLLock ( &pthis->StrList, ( LPVOID )&pToolBStrList ) )
		ErrorCode = WTLB_ERR_LISTLOCK;
	else
	{
                /*------------------------------------------------------------
                **  If they haven't turned off the divider, draw it on...
                **----------------------------------------------------------*/
                if (!(pthis->uStyles & CCS_NODIVIDER))
                    Toolbr_DrawBorder(hWnd, ps.hdc, pthis->uStyles);

		if ( pthis->ItmCount > 0 )
		{
			int i;
			TLBGDI	tbg;
			HFONT	hOldFont		= ( HFONT )NULL;
            		Toolbr_IBeginPaintBtn ( pthis, &tbg );
			if(pthis->hFont)
				hOldFont = SelectObject ( ps.hdc, pthis->hFont );
			SetBkMode ( ps.hdc, TRANSPARENT );
			for ( i = 0; i < pthis->ItmCount ; i++ )
			{
				pItem = pToolBItmList + i;
                                if ( pItem->Btn.fsState & TBSTATE_HIDDEN ) 
					continue;

				if ( pItem->Btn.iString >= 0 && 
				     pItem->Btn.iString < pthis->StrCount )
                                {
					pStr = pToolBStrList + pItem->Btn.iString;
                                }
				else
					pStr = NULL;
				Toolbr_IPaintBtn ( hWnd, ps.hdc, pthis, pItem, pStr ); 
			}


			if(hOldFont)
				SelectObject ( ps.hdc, hOldFont );
			Toolbr_IEndPaintBtn ( pthis, &tbg );
		}

	}


	if ( pToolBStrList )
		WLUnlock ( &pthis->StrList, ( LPVOID )&pToolBStrList );
	if ( pToolBItmList )
		WLUnlock ( &pthis->ItmList, ( LPVOID )&pToolBItmList );
	if ( pthis )
		GlobalUnlock ( hToolbar );
EndToolbarPaint:
	if(hPaintDC == 0)
		EndPaint ( hWnd, &ps );
}

static int 
Toolbr_OnSize 
(
	HWND			hWnd,
	DWORD			fwSizeType,
	int			nWidth,
	int			nHeight
) 
{
	HGLOBAL				hToolbar	= ( HGLOBAL )NULL;
	TOOLBARDATA*			pthis		= ( TOOLBARDATA* )NULL;
	int				ErrorCode	= 0;

	if ( ! ( hToolbar = GETTHISDATA ( hWnd ) ) )
		ErrorCode = WTLB_ERR_GETWINPROP;
	else
	if ( ! ( pthis = ( TOOLBARDATA *) GlobalLock ( hToolbar ) ) )
		ErrorCode = WTLB_ERR_GLOBALLOCK;
	else
	{
            pthis->Size.cx = nWidth;
            pthis->Size.cy = nHeight;
            Toolbr_AutoSize ( hWnd, pthis );
	}

	if ( pthis )
		GlobalUnlock ( hToolbar );

	return 0;	
}

static int 
Toolbr_OnWndPosChanging 
(
	HWND			hWnd,
	LPWINDOWPOS		lpWndPos
) 
{
	HGLOBAL				hToolbar	= ( HGLOBAL )NULL;
	TOOLBARDATA*			pthis		= ( TOOLBARDATA* )NULL;
	int				ErrorCode	= 0;

	if ( ! ( hToolbar = GETTHISDATA ( hWnd ) ) )
		ErrorCode = WTLB_ERR_GETWINPROP;
	else
	if ( ! ( pthis = ( TOOLBARDATA *) GlobalLock ( hToolbar ) ) )
		ErrorCode = WTLB_ERR_GLOBALLOCK;
	else
	if ( pthis->bStandardBmp )
	{
		lpWndPos->cy = pthis->Size.cy;
	}

	if ( pthis )
		GlobalUnlock ( hToolbar );

	return 0;	
}

/* create a mono bitmap mask:*/
#if 1
static void 
ToolBr_ICreateMask
(
	TOOLBARDATA*			pthis,
	TLBITEM*			pItem,
	POINT				ptOffset,
	BOOL				bHilite, 
	BOOL				bHiliteShadow
)
{
	/* initalize whole area with 0's				      */
	PatBlt ( pthis->hDCMono, 0, 0, pthis->ImgCX, pthis->ImgCY, WHITENESS );

	/* create mask based on color bitmap*/
	/* convert this to 1's*/
	SetBkColor ( pthis->hDCGlyphs, GetSysColor ( COLOR_BTNFACE ) );
        if (pthis->himl && pItem->Btn.iBitmap != -1)
            ImageList_Draw(pthis->himl, pItem->Btn.iBitmap, 
                                pthis->hDCMono, ptOffset.x, ptOffset.y,
                                ILD_NORMAL);
        else if ( pthis->hbmImage && pItem->Btn.iBitmap != -1 )
            BitBlt ( pthis->hDCMono, ptOffset.x, ptOffset.y, pthis->ImgCX, pthis->ImgCY,
                     pthis->hDCGlyphs, pItem->Btn.iBitmap * pthis->ImgCX, 0, SRCCOPY);

	if (bHilite)
	{
		/* convert this to 1's*/
		SetBkColor ( pthis->hDCGlyphs, RGB ( 255, 255, 255 ) );
		/* OR in the new 1's*/

                /* FIXME:  Implement IMAGELIST highlights */
		BitBlt ( pthis->hDCMono, ptOffset.x, ptOffset.y, pthis->ImgCX, pthis->ImgCY,
			 pthis->hDCGlyphs,  pItem->Btn.iBitmap * pthis->ImgCX, 0, SRCPAINT);
		if ( bHiliteShadow )
			BitBlt ( pthis->hDCMono, 1, 1, pthis->ImgCX - 3, pthis->ImgCY - 3,
				 pthis->hDCMono, 0, 0, SRCAND);
	}
}
#endif


static void
Toolbr_IGetBtnRects 
( 
	HDC				hDC,
	TOOLBARDATA*			pthis,
	RECT*   			prcBtn,
    	BYTE                		fsState,
	RECT*				prcIcon,
	RECT*				prcText
)
{                                             
	SIZE				Off;


        /* FIXME this is probably not correct for text buttons, but it works for now */
    	if ( prcText )
	    CopyRect ( prcText, prcBtn );

        if (prcIcon)
            CopyRect ( prcIcon, prcBtn );

	Off.cx = ( pthis->ItmWidth - pthis->ImgCX ) / 2;
        if (Off.cx < 0)
            Off.cx = 0;

        Off.cy = ( pthis->ItmHeight - pthis->ImgCY ) / 2;
        if (Off.cy < 0)
            Off.cy = 0;


        if (prcIcon)
        {
            InflateRect(prcIcon, -1 * Off.cx, -1 * Off.cy);

            if ( fsState & TBSTATE_PRESSED )
                OffsetRect ( prcIcon, 2, 2 );
	}

}

static void
Toolbr_IDrawBtnBorder 
( 
	HDC				hDC, 
	UINT				uState, 
	UINT				uStyle, 
	RECT*				pOuterRect,
	RECT*				pInnerRect
)
{
    RECT    rcIcon;
    int     inner_pad = 2;

    /*------------------------------------------------------------------------
    **  For FLAT style highlights, we want to draw a highlight just around
    **    the inner rectangle, because it seems to look better that way.
    **    FIXME:  Compare this with Windows and reconcile
    **----------------------------------------------------------------------*/
    if (uStyle & TBSTYLE_FLAT)
    {
        CopyRect ( &rcIcon, pInnerRect);

        if (pInnerRect->left - pOuterRect->left < inner_pad)
            CopyRect (&rcIcon, pOuterRect);
        else
            InflateRect(&rcIcon, inner_pad, inner_pad);

        if ( uState & TBSTATE_PRESSED )
            DrawEdge(hDC, &rcIcon, BDR_SUNKENOUTER, BF_RECT);
        else if ( uState & TBSTATE_HIGHLIGHTED)
            DrawEdge(hDC, &rcIcon, BDR_RAISEDINNER, BF_RECT);
    }
    
    /*------------------------------------------------------------------------
    **  For 3D buttons, they always have a highlight, and then it seems
    **    best if it's around the outer edge.
    **----------------------------------------------------------------------*/
    else
    {
        CopyRect ( &rcIcon, pOuterRect);
        InflateRect(&rcIcon, -1, -1);

        if ( uState & TBSTATE_PRESSED )
            DrawEdge(hDC, &rcIcon, EDGE_SUNKEN, BF_RECT);
        else
            DrawEdge(hDC, pOuterRect, EDGE_RAISED, BF_SOFT | BF_RECT);
    }
}

static void
Toolbr_IPaintDragLbxItem 
( 
	HWND				hWnd,
	LPDRAWITEMSTRUCT		lpdi, 
	TOOLBARDATA*			pthis, 
	TLBITEM*			pItem,
	LPSTR				pStr
)
{
    	RECT                		rcIcon;
	RECT				rcLabel;
	RECT				rcBtn;
    	HDC                 		hDC;
	HBRUSH              		hBkBr;
    	BYTE                		fsState = 0;
    	COLORREF            		txtColor;
	HPEN				hbPen	= CreatePen ( PS_SOLID, 1, GetSysColor ( COLOR_BTNSHADOW ) );
	HPEN				htxtPen = CreatePen ( PS_SOLID, 1, GetSysColor ( COLOR_BTNTEXT ) );
	HPEN				hSelPen = CreatePen ( PS_SOLID, 1, GetSysColor ( COLOR_HIGHLIGHT ) );
	HPEN				htmpPen = ( HPEN )NULL;
    	HBRUSH              		hHLBr   = CreateSolidBrush ( GetSysColor ( COLOR_HIGHLIGHT ) );

    	hDC = lpdi->hDC;
    	SetBkMode ( hDC, TRANSPARENT ); 
    	hBkBr = CreateSolidBrush ( GetBkColor ( hDC ) );

    	FillRect ( hDC, &lpdi->rcItem, hBkBr );
	txtColor = RGB ( 0, 0, 0 );
    	if ( ( lpdi->itemState & ODS_SELECTED ) &&
    	     !( lpdi->itemState & ODS_FOCUS )  )
    	{
    		htmpPen = SelectObject ( hDC, hSelPen );
    	        Rectangle ( hDC, lpdi->rcItem.left, lpdi->rcItem.top,
    	                         lpdi->rcItem.right, lpdi->rcItem.bottom );
    	        SelectObject ( hDC, htmpPen );
    	}
    	else
    	if ( ( lpdi->itemState & ODS_SELECTED ) &&
    	     ( lpdi->itemState & ODS_FOCUS )  )
    	{
    	        FillRect ( hDC, &lpdi->rcItem, hHLBr );
    	        DrawFocusRect ( hDC, &lpdi->rcItem );
		txtColor = RGB ( 255,255,255 );
    	}
    	SetRect ( &rcLabel, lpdi->rcItem.left + pthis->ItmWidth + 5, lpdi->rcItem.top,
                        lpdi->rcItem.right, lpdi->rcItem.bottom );
    	SetRect ( &rcBtn, lpdi->rcItem.left, lpdi->rcItem.top,
    	                  lpdi->rcItem.left + pthis->ItmWidth,
    	                  lpdi->rcItem.bottom
    	        );
    	InflateRect ( &rcBtn, 0, -1 * VERT_PADDING );
    	OffsetRect ( &rcBtn, 2, 0 );
    	FillRect ( hDC, &rcBtn, GetStockObject ( LTGRAY_BRUSH ) );

    	if ( pItem ) /* if not a list box separator item */
    	{
    	    Toolbr_IGetBtnRects ( hDC, pthis, &rcBtn,
    	                      pItem->Btn.fsState, &rcIcon, NULL );
    	    if ( pItem->Btn.fsState & TBSTATE_INDETERMINATE )
	    {
		    HGDIOBJ hbrOld;
		    POINT ptOffset;
       	     	    COLORREF txtColor, bkColor;
		    ptOffset.x = ptOffset.y = 0;
		    ToolBr_ICreateMask ( pthis, pItem, ptOffset, 
                            TRUE, FALSE);

		    txtColor = SetTextColor(hDC, 0L);                  /* 0's in mono -> 0 (for ROP)*/
		    bkColor = SetBkColor(hDC, (COLORREF)0x00FFFFFFL); /* 1's in mono -> 1*/

		    hbrOld = SelectObject(hDC, GetStockObject ( WHITE_BRUSH ) );
		    if (hbrOld != ( HGDIOBJ )NULL)
		    {
			    /* draw hilight color where we have 0's in the mask*/
			    BitBlt ( hDC, rcIcon.left, rcIcon.top,
				     pthis->ImgCX, pthis->ImgCY,
				     pthis->hDCMono, 0, 0, 0x00B8074AL);
			    SelectObject(hDC, hbrOld);
		    }

		    /*BLOCK: always draw the shadow*/
		    {
			    HGDIOBJ hbrOld = CreateSolidBrush ( GetSysColor ( COLOR_BTNSHADOW ) );
			    hbrOld = SelectObject(hDC, hbrOld );
			    if ( hbrOld != ( HGDIOBJ )NULL )
			    {
				    /* draw the shadow color where we have 0's in the mask*/
				    BitBlt ( hDC, rcIcon.left, rcIcon.top, pthis->ImgCX-2, pthis->ImgCY-2,
				             pthis->hDCMono, 0, 0, 0x00B8074AL );
				    hbrOld = SelectObject ( hDC, hbrOld );
				    DeleteObject ( hbrOld );
			    }
		    }
            	SetTextColor ( hDC, txtColor );
            	SetBkColor ( hDC, bkColor );
	    }
            else
	    {
		    if ( pthis->hbmImage && !( pItem->Btn.fsStyle & TBSTYLE_SEP ) && pItem->Btn.iBitmap != -1 ) 
			    BitBlt ( hDC, rcIcon.left, rcIcon.top, pthis->ImgCX, pthis->ImgCY, 
				     pthis->hDCGlyphs, pItem->Btn.iBitmap * pthis->ImgCX, 0, SRCCOPY );
	    }
    	}

	if ( pStr && *pStr != 0 )
	{
		HFONT	hOldFont = ( HFONT )NULL;
#if 1
        if ( pItem &&
             ( !( pItem->Btn.fsState & TBSTATE_ENABLED ) ||
	       ( pItem->Btn.fsState & TBSTATE_INDETERMINATE ) 
             )
           )
			SetTextColor ( hDC, GetSysColor ( COLOR_BTNSHADOW ) );
#endif
		if(pthis->hFont)
			hOldFont = SelectObject ( hDC, pthis->hFont );
		SetTextColor ( hDC, txtColor );
		DrawText ( hDC, pStr, -1, &rcLabel, DT_SINGLELINE | DT_LEFT | DT_VCENTER );
		if(hOldFont)
			SelectObject ( hDC, hOldFont );
	/*	SetTextColor ( hDC, GetSysColor ( COLOR_BTNTEXT ) );*/
	}

    	fsState = pItem ? pItem->Btn.fsState : TBSTATE_INDETERMINATE;

        /* FIXME:  Send the right parameters in here */
	Toolbr_IDrawBtnBorder ( hDC, fsState, pthis->uStyles, &rcBtn, &rcBtn );

	/*cleanup*/
	DeleteObject ( hbPen );
	DeleteObject ( htxtPen );
	DeleteObject ( hSelPen );
    	DeleteObject ( hHLBr );
    	DeleteObject ( hBkBr );
}

/* from Wine dlls/comctl32/toolbar.c, thank you very much */
static void
TOOLBAR_DrawFlatSeparator (LPRECT lpRect, HDC hdc)
{
    INT x = (lpRect->left + lpRect->right) / 2 - 1;
    INT yBottom = lpRect->bottom - 3;
    INT yTop = lpRect->top + 1;

    SelectObject ( hdc, GetSysColorPen (COLOR_3DSHADOW));
    MoveToEx (hdc, x, yBottom, NULL);
    LineTo (hdc, x, yTop);
    x++;
    SelectObject ( hdc, GetSysColorPen (COLOR_3DHILIGHT));
    MoveToEx (hdc, x, yBottom, NULL);
    LineTo (hdc, x, yTop);
}


/*new*/
static void
Toolbr_IPaintBtn 
( 
	HWND				hWnd,
	HDC				    hDC, 
	TOOLBARDATA*		pthis, 
	TLBITEM*			pItem,
	TLBSTR*				pStr
)
{
    RECT				rcLabel;
    RECT				rcIcon;
    HPEN				hbPen	= CreatePen ( PS_SOLID, 1, GetSysColor ( COLOR_BTNSHADOW ) );
    HPEN				htxtPen = CreatePen ( PS_SOLID, 1, GetSysColor ( COLOR_BTNTEXT ) );
    RECT*				pRect   = &pItem->Rect;

    FillRect ( hDC, pRect, GetStockObject ( LTGRAY_BRUSH ) );


    /*draw content of a toolbar button*/
    Toolbr_IGetBtnRects ( hDC, pthis, &pItem->Rect,
                      pItem->Btn.fsState, &rcIcon, &rcLabel );

    if ( pItem->Btn.fsStyle & TBSTYLE_SEP)
    {
        if (pthis->uStyles & TBSTYLE_FLAT) 
        {
            if (1/*pItem->Btn.iBitmap == 0*/)
                TOOLBAR_DrawFlatSeparator(&rcIcon, hDC);
            else if (pthis->himl)
                ImageList_Draw(pthis->himl, pItem->Btn.iBitmap, 
                                    hDC, rcIcon.left, rcIcon.top,
                                    ILD_NORMAL);
        }
    }
    else
    {
	if ( pItem->Btn.fsState & TBSTATE_ENABLED )
	{
            if (pthis->himl && pItem->Btn.iBitmap != -1)
            {
                ImageList_Draw(pthis->himl, pItem->Btn.iBitmap, 
                                    hDC, rcIcon.left, rcIcon.top,
                                    ILD_NORMAL);
            }
            else if ( pthis->hbmImage && pItem->Btn.iBitmap != -1 )
                BitBlt ( hDC, rcIcon.left, rcIcon.top, pthis->ImgCX, pthis->ImgCY, 
			 pthis->hDCGlyphs, pItem->Btn.iBitmap * pthis->ImgCX, 0, SRCCOPY );
	}
	else
	{
		HGDIOBJ hbrOld;
		POINT ptOffset;
		ptOffset.x = ptOffset.y = 0;
		ToolBr_ICreateMask ( pthis, pItem, ptOffset, TRUE, FALSE);

		SetTextColor(hDC, 0L);                  /* 0's in mono -> 0 (for ROP)*/
		SetBkColor(hDC, (COLORREF)0x00FFFFFFL); /* 1's in mono -> 1*/

		hbrOld = SelectObject(hDC, GetStockObject ( WHITE_BRUSH ) );
		if (hbrOld != ( HGDIOBJ )NULL)
		{
			/* draw hilight color where we have 0's in the mask*/
			BitBlt ( hDC, rcIcon.left, rcIcon.top,
				 pthis->ImgCX, pthis->ImgCY,
				 pthis->hDCMono, 0, 0, 0x00B8074AL);
			SelectObject(hDC, hbrOld);
		}

		/*BLOCK: always draw the shadow*/
		{
			HGDIOBJ hbrOld = CreateSolidBrush ( GetSysColor ( COLOR_BTNSHADOW ) );
			hbrOld = SelectObject(hDC, hbrOld );
			if (hbrOld != ( HGDIOBJ )NULL)
			{
				/* draw the shadow color where we have 0's in the mask*/
				BitBlt ( hDC, rcIcon.left, rcIcon.top, pthis->ImgCX-2, pthis->ImgCY-2,
				         pthis->hDCMono, 0, 0, 0x00B8074AL );
				hbrOld = SelectObject ( hDC, hbrOld );
				DeleteObject ( hbrOld );
			}
		}
	}


	if ( pStr )
	{
		HFONT hOldFont = (HFONT) 0;
		if ( !( pItem->Btn.fsState & TBSTATE_ENABLED ) ||
		     ( pItem->Btn.fsState & TBSTATE_INDETERMINATE ) )
			SetTextColor ( hDC, GetSysColor ( COLOR_BTNSHADOW ) );
		if(pthis->hFont)
			hOldFont = SelectObject(hDC,pthis->hFont);	
		DrawText ( hDC, pStr->Label, -1, &rcLabel, DT_SINGLELINE | DT_CENTER );
		if(hOldFont)
			SelectObject(hDC,hOldFont);	

		SetTextColor ( hDC, GetSysColor ( COLOR_BTNTEXT ) );
	}

        /*--------------------------------------------------------------------
        **  Draw a border around our button.  Note for certain styles
        **     there is no border, but that is handled inside the function.
        **------------------------------------------------------------------*/
        if (pStr)
        {
            /*------------------------------------------------------------
            **  For text buttons, make sure that we're not going out
            **      of the client window.
            **----------------------------------------------------------*/
            RECT rect;
            RECT client_rect;
            GetClientRect ( hWnd , &client_rect );
            IntersectRect (&rect, &client_rect, pRect);

            Toolbr_IDrawBtnBorder ( hDC, pItem->Btn.fsState, pthis->uStyles, &rect, &rect);
        }
        else
        {
            /*----------------------------------------------------------------
            **  The button rectangle we got may have been offset for a press
            **     but we want to undo that offset for the highlight border.
            **--------------------------------------------------------------*/
            if ( pItem->Btn.fsState & TBSTATE_PRESSED &&
                 pthis->uStyles & TBSTYLE_FLAT)
                OffsetRect(&rcIcon, -2, -2);

            Toolbr_IDrawBtnBorder ( hDC, pItem->Btn.fsState, pthis->uStyles, pRect, &rcIcon );
        }

    }

    /*cleanup*/
    DeleteObject ( hbPen );
    DeleteObject ( htxtPen );
}


static BOOL
Toolbr_SetState
(
	HWND				hwToolbar,
	TOOLBARDATA*			pthis, 
	int				idButton,
	int				State
)
{
	TLBITEM*			pToolBItmList	= ( TLBITEM* )NULL;
	int				ErrorCode	= 0;

	if ( ( idButton = Toolbr_CommandToIndex ( hwToolbar, pthis, idButton ) ) == -1 )
		ErrorCode = WTLB_ERR_OUTOFRANGE;
	else
	if ( WLLock ( &pthis->ItmList, ( LPVOID )&pToolBItmList ) )
		ErrorCode = WTLB_ERR_LISTLOCK;
	else {
		RECT rcBtn;
		( pToolBItmList + idButton )->Btn.fsState = State;

		CopyRect ( &rcBtn, &( ( pToolBItmList + idButton )->Rect) );
		InflateRect ( &rcBtn, 1, 1 );
		InvalidateRect ( hwToolbar, &rcBtn, TRUE );
	}

	if ( pToolBItmList )
		WLUnlock ( &pthis->ItmList, ( LPVOID )&pToolBItmList );

	return ( ErrorCode >= 0 );
}

/*
 * the toolbar would register itself with the tooltip. There's no need ( and there's no way )
 * to register individual toolbar buttons. 
 */
static void
Toolbr_SetTooltip 
( 
	HWND				hWnd, 
	HWND				hwTTip 
)
{
	
	TOOLINFO			ti;

	if ( hwTTip )
	{
		memset ( &ti, 0, sizeof ( TOOLINFO ) );
		ti.cbSize = sizeof(TOOLINFO); 
/*TTF_IDISHWND needed because ti.uId is supposed to be the id of a child*/
/*control of ti.hwnd and in this case ti.hwnd and ti.uId are both the same window*/
		ti.uFlags = 0; /*TTF_IDISHWND; */
		ti.hwnd = hWnd; 
		ti.uId = ( UINT )hWnd; 
        GetClientRect ( hWnd, &ti.rect );
		ti.lpszText = LPSTR_TEXTCALLBACK; 
		SendMessage ( hwTTip, TTM_ADDTOOL, 0, (LPARAM) (LPTOOLINFO) &ti ); 
		SendMessage ( hwTTip, TTM_ACTIVATE, ( WPARAM )TRUE, 0L );
	}

}


/*----------------------------------------------------------------------------------------------*
  Parameters:
  phbmImages : pointer to HBITMAP which is a collection of images
  pImgWidth  : width of each individual image.
  pImgHeight : height of each individual image.
  hBitmap    : a new collection of images to be added
  NoOfImages : number of new images being added 
 *----------------------------------------------------------------------------------------------*/
static BOOL 
W_IAddImage
( 
	HBITMAP*			phbmImages,
	int*				pImgWidth,
	int*				pImgHeight,
	int*				pImgCount, 
	HBITMAP				hBitmap,
	int				NoOfImages
)

{
	BITMAP				Bitmap;
	HDC				hMemoryDC	= ( HDC )NULL;
	HBITMAP				hTempBitmap	= ( HBITMAP )NULL;
	HBITMAP				hbmpOld		= ( HBITMAP )NULL;
	HBITMAP				hOld		= ( HBITMAP )NULL;
	HDC				hTempDC		= ( HDC )NULL;
	HDC				hDeskDC		= GetDC ( GetDesktopWindow () );
	int				NewWidth	= 0;
	int				ErrorCode	= 0;


	memset ( &Bitmap, 0, sizeof ( BITMAP ) );

	if ( ! ( GetObject ( hBitmap, sizeof ( BITMAP ), &Bitmap ) ) )
		ErrorCode = WIL_ERR_GETOBJECT;
	else
	if ( ! ( hMemoryDC = CreateCompatibleDC ( hDeskDC ) ) )
		ErrorCode = WIL_ERR_CREATECOMPATIBLEDC;
	else
	if ( ! ( hTempDC = CreateCompatibleDC ( hDeskDC ) ) )
		ErrorCode = WIL_ERR_CREATECOMPATIBLEDC;
	else
	{
		NewWidth = *pImgWidth;

		if ( ! ( hTempBitmap = CreateBitmap ( NewWidth * ( NoOfImages + *pImgCount ),
			 *pImgHeight, Bitmap.bmPlanes, Bitmap.bmBitsPixel, ( void* )NULL ) ) )
			ErrorCode = WIL_ERR_CREATEBITMAP;
		else
		{
			hbmpOld = SelectObject ( hTempDC, *phbmImages );
			hOld = SelectObject( hMemoryDC, hTempBitmap );
			BitBlt ( hMemoryDC, 0, 0, *pImgCount * NewWidth, *pImgHeight,
				 hTempDC, 0, 0, SRCCOPY );
			
			SelectObject ( hTempDC, hBitmap );
			BitBlt ( hMemoryDC, *pImgCount * NewWidth, 0, 
				 NewWidth * NoOfImages, *pImgHeight,
				 hTempDC, 0, 0, SRCCOPY );
			SelectObject ( hTempDC, hbmpOld );
			SelectObject( hMemoryDC, hOld );
			DeleteObject ( *phbmImages );
			*phbmImages = hTempBitmap;
		}
	}

	if ( hTempDC )
		DeleteDC ( hTempDC );

	if ( hMemoryDC )
		DeleteDC ( hMemoryDC );

	if ( hDeskDC )
		ReleaseDC ( GetDesktopWindow (), hDeskDC );

	return ( ErrorCode == 0 );

}

static BOOL
Toolbr_IBeginDrag 
(
	HWND				hWnd,
	TOOLBARDATA*			pthis,
	TLBITEM*			pItem,
	int				idBtn
)
{
	if ( pthis->enDrag == eBeginDrag || pthis->enDrag == eShftBeginDrag)
	{
		pthis->tbn.hdr.hwndFrom = hWnd;
		pthis->tbn.hdr.idFrom = idBtn;
		pthis->tbn.iItem = idBtn;
		pthis->tbn.tbButton = pItem->Btn;
		pthis->tbn.cchText = Toolbr_GetButtonText ( hWnd, pthis, idBtn, 
						pthis->cBtnText );
		pthis->tbn.pszText = pthis->cBtnText;
		SetCapture ( hWnd );
	}
	
	if ( pthis->enDrag == eLBtnDown )
		pthis->enDrag = eBeginDrag;
	else
	if ( pthis->enDrag == eShftLBtnDown )
		pthis->enDrag = eShftBeginDrag;
	else
	if ( pthis->enDrag == eBeginDrag )
	{
		pthis->tbn.hdr.code = TBN_BEGINDRAG;
		SendMessage ( GetParent ( hWnd ), WM_NOTIFY, 0, ( LPARAM )&pthis->tbn );
		pthis->enDrag = eDragging;
	}
	else
	if ( pthis->enDrag == eShftBeginDrag )
	{
		pthis->tbn.hdr.code = TBN_QUERYDELETE;
		if ( SendMessage ( GetParent ( hWnd ), WM_NOTIFY, 0, ( LPARAM )&pthis->tbn ) )
		{
			pthis->enDrag = eShftDragging;
			if ( !g_hDragCursor )
#ifdef _WINDOWS
				g_hDragCursor = LoadCursor ( ( HINSTANCE )GetWindowLong ( GetParent ( hWnd ),GWL_HINSTANCE ), 
						MAKEINTRESOURCE ( IDC_CURSOR_DRAG ) );
#else
				g_hDragCursor = LoadCursor ( hLibInstance, 
						MAKEINTRESOURCE ( IDC_CURSOR_DRAG ) );
#endif
			if ( g_hDragCursor )
				g_hDefCursor = SetCursor ( g_hDragCursor );
		}
		else
			pthis->enDrag = eCancelDrag;
	}

	return TRUE;
}

static BOOL
Toolbr_IEndDrag 
(
	HWND				hWnd,
	TOOLBARDATA*			pthis,
	BOOL*				pbInsert
)
{
	*pbInsert		= FALSE;
	if ( pthis->enDrag == eDragging )
	{
		ReleaseCapture ();
		pthis->enDrag = eEndDrag;
		pthis->tbn.hdr.code = TBN_ENDDRAG;
		SendMessage ( GetParent ( hWnd ), WM_NOTIFY, 0, ( LPARAM )&pthis->tbn );
	}
	else
	if ( pthis->enDrag == eShftDragging )
	{
		ReleaseCapture ();
		SetCursor ( g_hDefCursor );
		pthis->enDrag = eEndDrag;
		pthis->tbn.hdr.code = TBN_QUERYINSERT;
		if ( SendMessage ( GetParent ( hWnd ), WM_NOTIFY, 0, ( LPARAM )&pthis->tbn ) )
			*pbInsert = TRUE;
	}
	else
	if ( pthis->enDrag == eCancelDrag )
		ReleaseCapture ();

	return TRUE;
}

static void
Toolbr_IResetLayout
(
	HWND				hWnd,
	TOOLBARDATA*			pthis,
	TLBHITTEST			tbHitTest
)
{
    	NMHDR               		Notify;
	TBBUTTON			MovedBtn;
	TBBUTTON			SepBtn;
	int			    	iBtnAfter	= tbHitTest.HitBtn;
	int			    	iHitRow		= tbHitTest.HitRow;

	if (pthis->ItmCount == 0)
		return;

	if ( iBtnAfter == pthis->ItmSelected )
		return;
	memset ( &SepBtn, 0, sizeof ( TBBUTTON ) );
	SepBtn.fsStyle = TBSTYLE_SEP;
	SepBtn.idCommand = -1;
	SepBtn.iBitmap = -1;
	SepBtn.iString = -1;

	Toolbr_GetButton ( hWnd, pthis, pthis->ItmSelected, &MovedBtn );
	if ( iHitRow == eHitOutside )
	{
		Toolbr_HideBtn ( hWnd, pthis, MovedBtn.idCommand, TRUE );
		return;
	}
	if ( WLDelete ( &pthis->ItmList, pthis->ItmSelected ) == 0 )
	{
		pthis->ItmCount--;
		if ( iBtnAfter < 0 )
		{
			if ( pthis->ItmSelected == pthis->ItmCount ) 
				Toolbr_InsertBtn ( hWnd, pthis, pthis->ItmCount, &SepBtn );
			iBtnAfter = pthis->ItmCount;
		}
		else
		if ( iBtnAfter > pthis->ItmSelected )
			iBtnAfter--;
		Toolbr_InsertBtn ( hWnd, pthis, iBtnAfter, &MovedBtn );
		InvalidateRect ( hWnd, NULL, FALSE );

        Notify.hwndFrom = hWnd;
        Notify.idFrom = GetDlgCtrlID ( hWnd );
        Notify.code = TBN_TOOLBARCHANGE;
        SendMessage ( GetParent ( hWnd ), WM_NOTIFY, Notify.idFrom, ( LPARAM )&Notify );
	}
}

void
Toolbr_MoveBtn
(
    HWND                hWnd,
    int                 iBtnMove,
    int                 iBtnAfter
)
{
    TLBHITTEST          tbht;
    HGLOBAL				hToolbar	    = ( HGLOBAL )NULL;
	TOOLBARDATA*		pthis		    = ( TOOLBARDATA* )NULL;
    int                 ErrorCode       = 0;

	if ( ! ( hToolbar = GETTHISDATA ( hWnd ) ) )
		ErrorCode = WTLB_ERR_GETWINPROP;
	else
	if ( ! ( pthis = ( TOOLBARDATA *) GlobalLock ( hToolbar ) ) )
		ErrorCode = WTLB_ERR_GLOBALLOCK;
    else
    {
        pthis->ItmSelected = iBtnMove;
        tbht.HitBtn = iBtnAfter;
        Toolbr_IResetLayout ( hWnd, pthis, tbht );
    }

	if ( pthis )
		GlobalUnlock ( hToolbar );
}

/*reset a group of toolbar buttons so that they look undepressed*/
static void
Toolbr_IResetCheckGroup
(
	HWND				hWnd,
	TLBITEM*			pToolBItmList,
	int				nSelected,
	int				nBtnCount
)
{
	int				i;
	RECT				rcBtn;
	int				OldSelect	= -1;
	TLBITEM*			pItem		= ( TLBITEM* )NULL;
	BOOL				bDone		= FALSE;
	

	for ( i = nSelected - 1; i >= 0;i-- )
	{
		pItem = pToolBItmList + i;
		if ( !( pItem->Btn.fsStyle & TBSTYLE_GROUP ) )
			break;
		if ( pItem->Btn.fsState & TBSTATE_PRESSED )
		{
			bDone = TRUE;
			OldSelect = i;
			break;
		}
	}

	for ( i = nSelected + 1; !bDone && i < nBtnCount; i++ )
	{
		pItem = pToolBItmList + i;
		if ( !( pItem->Btn.fsStyle & TBSTYLE_GROUP ) )
			break;
		if ( pItem->Btn.fsState & TBSTATE_PRESSED )
		{
			bDone = TRUE;
			OldSelect = i;
			break;
		}
	}

	if ( OldSelect > -1 )
	{
		pItem = pToolBItmList + OldSelect;
		pItem->Btn.fsState ^= TBSTATE_PRESSED;
		CopyRect ( &rcBtn, &pItem->Rect );
		InflateRect ( &rcBtn, 1, 1 );
		InvalidateRect ( hWnd, &rcBtn, TRUE );
	}
}


/* need refinement */
static void
Toolbr_ISetItemRects 
( 
	HWND				hwToolbar,
	TOOLBARDATA*			pthis
)
{
	int				i;
	int				j;
	int				left;
	int				grpleft;
	int				top;
	int				Rows;
	int				Count		= 0;
	int				grCount		= 0;
	int				nitWidth	= 0;
	int				nitHeight	= 0;
        int                             maxtop          = 22;
        int                             maxleft         = 24;
	BOOL				bLineBreak	= FALSE;
	TLBITEM*			pItemList	= ( TLBITEM* )NULL;
	TLBITEM*			pItem		= ( TLBITEM* )NULL;
	TLBITEM*			pgrItem		= ( TLBITEM* )NULL;
        SIZE                            Size;
	TLBSTR*				pToolBItmText;
	TLBSTR*				pLabel;
        int                             len;

	Rows = 0;
	Count = pthis->ItmCount;

	if (Count == 0)
		return;

	if ( WLLock ( &pthis->ItmList, ( LPVOID )&pItemList ) )
		return;
	
	left = pthis->nXPadding;
	top = pthis->nYPadding;
	for ( i = 0; i < Count; i++ )
	{
		pItem = pItemList + i;

                /*------------------------------------------------------------
                ** Determine how big this item is; for strings,
                **  we compute it on the fly, for icons, we require
                **  them to be the same size, for separators,
                **  they're their own size.
                **----------------------------------------------------------*/
                nitHeight = pthis->ItmHeight;
		if ( pItem->Btn.fsStyle & TBSTYLE_SEP ) 
                    nitWidth = pItem->Btn.iBitmap;
                else if (pthis->StrCount > 0)
                {
                    HDC hDC = GetDC ( hwToolbar );
                    if ( WLLock ( &pthis->StrList, ( LPVOID )&pToolBItmText ) == 0)
                    {
                        pLabel = pToolBItmText + i;
                        len = lstrlen(pLabel->Label);
                        GetTextExtentPoint32 ( hDC, pLabel->Label, len, &Size);
                        nitWidth = Size.cx;
                        nitHeight = Size.cy;
                        WLUnlock ( &pthis->StrList, ( LPVOID )&pToolBItmText );
                    }
                    ReleaseDC ( hwToolbar, hDC );
                }
                else
                    nitWidth = pthis->ItmWidth;

		if ( pItem->Btn.fsState & TBSTATE_HIDDEN )
		{
			SetRectEmpty ( &pItem->Rect );
			continue;
		}
		if ( i >= grCount && pItem->Btn.fsStyle & TBSTYLE_GROUP )
		{
			for ( j = i, grpleft = left,grCount = 0; 
		      	      j < Count ; j++ )
			{
				pgrItem = pItemList + j;
				if ( pgrItem->Btn.fsState & TBSTATE_HIDDEN )
					continue;
				grpleft += nitWidth + 1;
				if ( ( pgrItem->Btn.fsStyle & TBSTYLE_SEP ) ||
			     	     !( pgrItem->Btn.fsStyle & TBSTYLE_GROUP ) ||
			     	     ( grpleft > pthis->Size.cx )			     
			   	   )
				{
					grCount = j;
					break;
				}
				else
				if ( grpleft + left > pthis->Size.cx )
				{
					bLineBreak = TRUE;
					grCount = j;
					break;
				}
			}
		}
		bLineBreak = ( bLineBreak || ( left + nitWidth > pthis->Size.cx ) ) &&
			     ( pthis->uStyles & TBSTYLE_WRAPABLE );
		if ( bLineBreak )
		{
			Rows++;
			top += nitHeight + VERT_PADDING;
			left = pthis->nXPadding;
			bLineBreak = FALSE;
		}
    		SetRect ( &pItem->Rect, left, top, left + nitWidth, top + nitHeight );

		pItem->Row = Rows ;
		left += nitWidth;
                left += HORZ_PADDING;
		if ( pItem->Btn.fsState & TBSTATE_WRAP )
		{
			Rows++;
			top += nitHeight + VERT_PADDING;
			left = pthis->nXPadding;
		}

                if (top > maxtop)
                    maxtop = top;
                if (left > maxleft)
                    maxleft = left;
	}

	pthis->Rows = Rows + 1;

        Toolbr_Resize_Toolbar(hwToolbar, pthis, maxleft, maxtop);

	pthis->bNeedSetRects = FALSE;

	if ( pItemList )
		WLUnlock ( &pthis->ItmList, ( LPVOID )&pItemList );
}

static int
Toolbr_IBtnFromPt 
( 
	HWND				hwToolbar,
	TOOLBARDATA*			pthis, 
	int				x,
	int				y,
	TLBHITTEST*			pHitTest
)
{
	POINT				pt;
	TLBHITTEST			tbHitTest;
	TLBITEM*			pItemList	= ( TLBITEM* )NULL;
	TLBITEM*			pItem		= ( TLBITEM* )NULL;
	int				i		= 1;
	int				Count		= 0;
	int				ErrorCode	= 0;
	
	tbHitTest.HitBtn = -1;
	tbHitTest.HitRow = eHitOutside;
	if ( WLLock ( &pthis->ItmList, ( LPVOID )&pItemList ) )
		ErrorCode = WTLB_ERR_LISTLOCK;
	else
	{
		Count = pthis->ItmCount;
		pt.x = x;
		pt.y = y;
		for ( i = 0; i < Count; i++ )
		{
			pItem = pItemList + i;
			if ( y >= pItem->Rect.top && y <= pItem->Rect.bottom )
				tbHitTest.HitRow = pItem->Row;
			if ( PtInRect (	&pItem->Rect, pt ) )
			{
				tbHitTest.HitBtn = i;
				break;
			}
		}
	}

	if ( pHitTest )
		*pHitTest = tbHitTest;
	if ( pItemList )
		WLUnlock ( &pthis->ItmList, ( LPVOID )&pItemList );

	return ( ( ErrorCode >= 0 && i < Count ) ? i : -1 );

}


static LRESULT WINAPI 
Toolbr_WndProc
(
	HWND				hWnd,
    UINT				uMessage,
    WPARAM				wParam,
    LPARAM				lParam
)
{
	LRESULT rc = 0;

	switch (uMessage)
	{
		case WM_CREATE:
			if ( ! Toolbr_OnCreate ( hWnd, ( LPCREATESTRUCT )lParam ) )
				return ( LRESULT ) -1L;
			break;
		case WM_DESTROY:
			Toolbr_OnDestroy ( hWnd );
			break;
		
		case WM_GETDLGCODE:
			return ( LRESULT )DLGC_WANTARROWS | DLGC_WANTCHARS;
		
		case WM_LBUTTONDBLCLK:
			Toolbr_OnLButtonDblClk ( hWnd,(int)(short)LOWORD(lParam), 
				( int )( short )HIWORD ( lParam ), ( UINT )( wParam ) );
			break;

		case WM_LBUTTONDOWN:
			Toolbr_OnLButtonDown ( hWnd,(int)(short)LOWORD(lParam), 
				( int )( short )HIWORD ( lParam ), ( UINT )( wParam ) );
			break;

		case WM_LBUTTONUP:
			Toolbr_OnLButtonUp ( hWnd,(int)(short)LOWORD(lParam), 
				( int )( short )HIWORD ( lParam ), ( UINT )( wParam ) );
			break;

		case WM_MOUSEMOVE:
			Toolbr_OnMouseMove ( hWnd, ( UINT )wParam, lParam );
			break;

                case WM_CAPTURECHANGED:
                        /*----------------------------------------------------
                        **  Special case:  on a capture changed, we use
                        **      the same logic on mousemove to turn
                        **      off highlights
                        **--------------------------------------------------*/
                        Toolbr_OnMouseMove (hWnd, 0, MAKELONG(-1, -1));
                        break;
#if 0
		case WM_NCCALCSIZE:
		case WM_NCACTIVATE:
			break;

		case WM_NCPAINT:
			Toolbr_OnNcPaint ( hWnd, ( UINT )wParam, lParam );
			break;
#endif
		case WM_NOTIFY:
			Toolbr_OnNotify ( hWnd, ( UINT )wParam, lParam );
			break;

		case WM_PAINT:
			Toolbr_OnPaint( hWnd, ( HDC ) wParam  );

			break;

		case WM_SIZE:
			Toolbr_OnSize( hWnd, ( DWORD )wParam, 
				( int )LOWORD ( lParam ), ( int )HIWORD ( lParam ) );
			break;
		case WM_WINDOWPOSCHANGING:
			rc = Toolbr_OnWndPosChanging ( hWnd, 
							( LPWINDOWPOS )lParam );
			return rc;
	
		case TB_ADDBITMAP:
		case TB_ADDBUTTONS:
		case TB_ADDBUTTONSW:
		case TB_ADDSTRING:
		case TB_ADDSTRINGW:
		case TB_AUTOSIZE:
		case TB_BUTTONCOUNT:
		case TB_CHANGEBITMAP:
		case TB_CHECKBUTTON:
		case TB_COMMANDTOINDEX:
		case TB_CUSTOMIZE:
		case TB_DELETEBUTTON:
		case TB_ENABLEBUTTON:
		case TB_GETBITMAP:
		case TB_GETBITMAPFLAGS:
		case TB_GETBUTTON:
		case TB_GETBUTTONTEXT:
		case TB_GETBUTTONTEXTW:
		case TB_GETITEMRECT:
		case TB_GETBUTTONSIZE:
		case TB_BUTTONSTRUCTSIZE:
		case TB_REPLACEBITMAP:
		
		case TB_SETINDENT:

		case TB_SETIMAGELIST:
		case TB_GETIMAGELIST:

		case TB_LOADIMAGES:
		case TB_GETRECT:

		case TB_SETHOTIMAGELIST:
		case TB_GETHOTIMAGELIST:

		case TB_GETDISABLEDIMAGELIST:
		case TB_SETDISABLEDIMAGELIST:

		case TB_GETROWS:
		case TB_GETSTATE:
		case TB_GETTOOLTIPS:
		case TB_HIDEBUTTON:
		case TB_INDETERMINATE:
		case TB_INSERTBUTTON:	
		case TB_ISBUTTONCHECKED:	
		case TB_ISBUTTONENABLED:	
		case TB_ISBUTTONHIDDEN:	
		case TB_ISBUTTONINDETERMINATE:	
		case TB_ISBUTTONPRESSED:	
		case TB_PRESSBUTTON:	

		case TB_SAVERESTORE:	
		case TB_SAVERESTOREW:
		case TB_SETBITMAPSIZE:	
		case TB_SETBUTTONSIZE:	
		case TB_SETCMDID:	
		case TB_SETPARENT:	
		case TB_SETROWS:	
		case TB_SETSTATE:	
		case TB_SETTOOLTIPS:
			case TB_MARKBUTTON:
			case TB_ISBUTTONHIGHLIGHTED:
			case TB_GETOBJECT: 
			case TB_GETHOTITEM:
			case TB_SETHOTITEM:
			case TB_SETANCHORHIGHLIGHT:
			case TB_GETANCHORHIGHLIGHT:
			case TB_MAPACCELLERATOR:
			case TB_MAPACCELLERATORW:
			case TB_GETINSERTMARK:	
			case TB_SETINSERTMARK:
			case TB_INSERTMARKHITTEST:
			case TB_MOVEBUTTON:
			case TB_GETMAXSIZE:
			case TB_SETEXTENDEDSTYLE:
			case TB_GETEXTENDEDSTYLE:	
			case TB_GETPADDING:	
			case TB_SETPADDING:
			case TB_SETINSERTMARKCOLOR:	
			case TB_GETINSERTMARKCOLOR:	
		
			case TB_GETBUTTONINFOW:
			case TB_SETBUTTONINFOW:
			case TB_GETBUTTONINFO:
			case TB_SETBUTTONINFO:

			case TB_INSERTBUTTONW:
			case TB_HITTEST:
			case TB_SETDRAWTEXTFLAGS:
			case TB_SETSTYLE:
			case TB_GETSTYLE:

			case TB_SETMAXTEXTROWS:
	
			case TB_GETTEXTROWS:
			
			rc = Toolbar_WndProc(hWnd, uMessage, wParam, lParam);
			break;

        case DL_BEGINDRAG:
            return TRUE;

        default:
			return DefWindowProc(hWnd, uMessage, wParam, lParam);
	}
    
	return rc;
}
static LRESULT
UnhandledMsg(UINT uMessage)
{
	UINT rc = 0xdeadbeef;
	APISTR((LF_CONSOLE,"Unhandle Toolbar message(%x)\n",uMessage));
	debuggerbreak();
	return rc;
}

static LRESULT
Toolbr_GetInfo(HWND hWnd,TOOLBARDATA *pthis, WPARAM wParam,LPARAM lParam)
{
	TBBUTTONINFO *lp = (TBBUTTONINFO *) lParam;
	APISTR((LF_CONSOLE,"GetButtonInfo  %x\n",lp->dwMask));
	debuggerbreak();
	return 0;
}
static LRESULT
Toolbr_SetInfo(HWND hWnd,TOOLBARDATA *pthis, WPARAM wParam,LPARAM lParam)
{
	TBBUTTONINFO *lp = (TBBUTTONINFO *) lParam;
	int	      btnindex;
	TLBITEM*      pToolBItmList	= ( TLBITEM* )NULL;
	int	      ErrorCode	= 0;

	if(lp == 0)
		return 0;

	/* get an index */
        if((btnindex = Toolbr_CommandToIndex (hWnd,pthis,wParam))==-1)
		return 0;	

	/* lock down the button list */
	if ( WLLock ( &pthis->ItmList, ( LPVOID )&pToolBItmList ) )
		ErrorCode = WTLB_ERR_LISTLOCK;

	if(pToolBItmList && lp->dwMask & TBIF_COMMAND)  {

		( pToolBItmList + btnindex )->Btn.idCommand = lp->idCommand;
	}
	if(lp->dwMask & TBIF_IMAGE) {
		/* TBD */
	 	LOGSTR((LF_WARNING,"SetButtonInfo image \n"));
	}
	if(pToolBItmList && lp->dwMask & TBIF_LPARAM) {
		( pToolBItmList + btnindex )->Btn.dwData = lp->lParam;
	}

	if(lp->dwMask & TBIF_SIZE) {
		/* TBD */
		LOGSTR((LF_WARNING,"SetButtonInfo size \n"));
	}
	if(lp->dwMask & TBIF_STYLE) {
		( pToolBItmList + btnindex )->Btn.fsStyle = lp->fsStyle;
	}
	if(lp->dwMask & TBIF_STATE) {
		( pToolBItmList + btnindex )->Btn.fsState = lp->fsState;
	}
	if(lp->dwMask & TBIF_TEXT) {
		/* TBD */
		LOGSTR((LF_WARNING,"SetButtonInfo text \n"));
	}

	if ( pToolBItmList )
		WLUnlock ( &pthis->ItmList, ( LPVOID )&pToolBItmList );

	return ErrorCode?1:0;
}

static LRESULT  
Toolbar_WndProc
(
	HWND				hWnd,
    UINT				uMessage,
    WPARAM				wParam,
    LPARAM				lParam
)
{
	HGLOBAL				hToolbar	= ( HGLOBAL )NULL;
	TOOLBARDATA*			pthis		= ( TOOLBARDATA* )NULL;
	LRESULT				lResult		= 0L;
	
	int				ErrorCode	= 0;

	if ( ! ( hToolbar = GETTHISDATA ( hWnd ) ) )
		ErrorCode = WTLB_ERR_GETWINPROP;
	else
	if ( ! ( pthis = ( TOOLBARDATA *) GlobalLock ( hToolbar ) ) )
		ErrorCode = WTLB_ERR_GLOBALLOCK;
	else
	{
			switch ( uMessage )
			{
				case TB_ADDBITMAP:
					lResult = ( LRESULT )Toolbr_AddBitmap ( hWnd, pthis, ( UINT )wParam,
						( LPTBADDBITMAP ) lParam );
					break;
				case TB_ADDBUTTONS:
					lResult = ( LRESULT )Toolbr_AddButtons ( hWnd, pthis, 
						( UINT )wParam, ( LPTBBUTTON )lParam );
					break;
				case TB_ADDSTRING:
					lResult = ( LRESULT )Toolbr_AddStrings ( hWnd, pthis, 
								( HINSTANCE )wParam, ( LPSTR )lParam );
					break;
				case TB_AUTOSIZE:
					Toolbr_AutoSize ( hWnd, pthis );
					break;
				case TB_BUTTONCOUNT:
					lResult = ( LRESULT )pthis->ItmCount;
					break;
				case TB_CHANGEBITMAP:
					lResult = Toolbr_ChangeBitmap ( hWnd, pthis, ( int )wParam, 
							( int )LOWORD ( lParam ) );
					break;
				case TB_CHECKBUTTON:
					lResult = Toolbr_CheckBtn ( hWnd, pthis, ( int )wParam, 
							( BOOL )LOWORD ( lParam ) );
					break;
				case TB_COMMANDTOINDEX:
					lResult = ( LRESULT )Toolbr_CommandToIndex ( hWnd, pthis, ( int )wParam );
					break;
				case TB_DELETEBUTTON:
					lResult = ( LRESULT )Toolbr_DeleteBtn ( hWnd, pthis, ( int )wParam );
					break;
				case TB_ENABLEBUTTON:
					lResult = ( LRESULT )Toolbr_EnableBtn ( hWnd, pthis, ( int )wParam, 
								( BOOL )LOWORD ( lParam ) );
					break;
				case TB_GETBITMAP:
					lResult = ( LRESULT )Toolbr_GetBitmap ( hWnd, pthis, ( int )wParam );
					break;
				case TB_GETBITMAPFLAGS:
					lResult = ( LRESULT )Toolbr_GetBitmapFlag ( hWnd );
					break;
				case TB_GETBUTTON:
					lResult = ( LRESULT )Toolbr_GetButton ( hWnd, pthis, ( int )wParam, 
								( LPTBBUTTON )lParam );
					break;
				case TB_GETBUTTONTEXT:
					lResult = ( LRESULT )Toolbr_GetButtonText ( hWnd, pthis, ( int )wParam, 
								( LPSTR )lParam );
					break;
				case TB_GETITEMRECT:
					lResult = ( LRESULT )Toolbr_GetItemRect ( hWnd, pthis, ( int )wParam, 
								( LPRECT )lParam );
					break;
				case TB_GETROWS:
					lResult = ( LRESULT )pthis->Rows;
					break;
				case TB_GETSTATE:
					lResult = ( LRESULT )Toolbr_GetState ( hWnd, pthis, ( int )wParam );
					break;
				case TB_GETTOOLTIPS:
					lResult = ( LRESULT )pthis->hwTTip;
					break;
				case TB_HIDEBUTTON:
					lResult = ( LRESULT )Toolbr_HideBtn ( hWnd, pthis, ( int )wParam, 
								( BOOL )LOWORD ( lParam ) );
					break;
				case TB_INDETERMINATE:
					lResult = ( LRESULT )Toolbr_InDeterminate ( hWnd, pthis, ( int )wParam, 
								( BOOL )LOWORD ( lParam ) );
					break;
				case TB_INSERTBUTTON:	
					lResult = ( LRESULT )Toolbr_InsertBtn ( hWnd, pthis, ( int )wParam, 
								( LPTBBUTTON )lParam );
					Toolbr_EnableBtn(hWnd,pthis,
						(int)wParam,TRUE);
			
					break;
				case TB_ISBUTTONCHECKED:	
				case TB_ISBUTTONENABLED:	
				case TB_ISBUTTONHIDDEN:	
				case TB_ISBUTTONINDETERMINATE:	
				case TB_ISBUTTONPRESSED:	
				case TB_ISBUTTONHIGHLIGHTED:
					lResult = ( LRESULT )Toolbr_QueryStat ( hWnd, pthis, uMessage,
								( int )wParam );
				break;
			case TB_PRESSBUTTON:	
				lResult = ( LRESULT )Toolbr_PressBtn ( hWnd, pthis, ( int )wParam,
							( BOOL )LOWORD ( lParam ) );
				break;
			case TB_SETBITMAPSIZE:	
				lResult = ( LRESULT )Toolbr_SetBMPSize ( hWnd, pthis, 
						( int )LOWORD ( lParam ), ( int )HIWORD ( lParam ) );
				break;
			case TB_SETBUTTONSIZE:	
				lResult = ( LRESULT )Toolbr_SetBtnSize ( hWnd, pthis, 
						( int )LOWORD ( lParam ), ( int )HIWORD ( lParam ) );
				break;
			case TB_SETCMDID:	
				lResult = ( LRESULT )Toolbr_SetCmdID ( hWnd, pthis, 
						( UINT )wParam, ( UINT )lParam );
				break;
			case TB_SETPARENT:	
				SetParent ( hWnd, ( HWND )wParam );
				break;
			case TB_SETROWS:	
				Toolbr_SetRows ( hWnd, pthis, ( int )LOWORD ( wParam ), 
					( BOOL )HIWORD ( wParam ), ( LPRECT )lParam );
				break;
			case TB_SETSTATE:	
				lResult = ( LRESULT )Toolbr_SetState ( hWnd, pthis, ( int )wParam,
					( int )LOWORD ( lParam ) );
				break;
			case TB_SETTOOLTIPS:
				pthis->hwTTip = ( HWND )wParam;
				Toolbr_SetTooltip ( hWnd, pthis->hwTTip );
				break;

			case TB_GETBUTTONSIZE:
				lResult = MAKELONG(pthis->ItmWidth,
						pthis->ItmHeight);
				break;
			case TB_MARKBUTTON:
				lResult = UnhandledMsg(uMessage);
				break;
			case TB_SAVERESTORE:	
			case TB_SAVERESTOREW:
				break;
				lResult = UnhandledMsg(uMessage);
				break;
			case TB_GETOBJECT: 
				lResult = UnhandledMsg(uMessage);
				break;
			case TB_BUTTONSTRUCTSIZE:
				pthis->buttonstructsize = wParam;
				break;
			case TB_GETHOTITEM:
				lResult = pthis->ItmHot;
				break;
			case TB_SETHOTITEM:
				pthis->ItmHot = lParam;
				break;
			case TB_SETANCHORHIGHLIGHT:
				lResult = UnhandledMsg(uMessage);
				break;
			case TB_GETANCHORHIGHLIGHT:
				lResult = UnhandledMsg(uMessage);
				break;
			case TB_MAPACCELLERATOR:
			case TB_MAPACCELLERATORW:
				lResult = UnhandledMsg(uMessage);
				break;
			case TB_GETINSERTMARK:	
				lResult = UnhandledMsg(uMessage);
				break;
			case TB_SETINSERTMARK:
				lResult = UnhandledMsg(uMessage);
				break;
			case TB_ADDSTRINGW:
				lResult = UnhandledMsg(uMessage);
				break;
			case TB_INSERTMARKHITTEST:
				lResult = UnhandledMsg(uMessage);
				break;
			case TB_MOVEBUTTON:
				lResult = UnhandledMsg(uMessage);
				break;
			case TB_CUSTOMIZE:
				lResult = UnhandledMsg(uMessage);
				break;
			case TB_GETMAXSIZE:
				lResult = UnhandledMsg(uMessage);
				break;
			case TB_SETEXTENDEDSTYLE:
				pthis->uExtStyles = lParam;
				lResult = 0;
				break;
			case TB_GETEXTENDEDSTYLE:	
				lResult = pthis->uExtStyles;
				break;
			case TB_GETPADDING:	
				lResult = UnhandledMsg(uMessage);
				break;
			case TB_SETPADDING:
				lResult = UnhandledMsg(uMessage);
				break;
			case TB_SETINSERTMARKCOLOR:	
				lResult = UnhandledMsg(uMessage);
				break;
			case TB_GETINSERTMARKCOLOR:	
				lResult = UnhandledMsg(uMessage);
				break;
		
			case TB_GETBUTTONINFOW:
				lResult = UnhandledMsg(uMessage);
				break;
		
			case TB_GETBUTTONINFO:
				lResult = (LRESULT)Toolbr_GetInfo(hWnd,pthis, 
					wParam, lParam);
				break;
			case TB_SETBUTTONINFO:
				lResult = (LRESULT)Toolbr_SetInfo(hWnd,pthis, 
					wParam, lParam);
				break;
			case TB_SETBUTTONINFOW:
				lResult = UnhandledMsg(uMessage);
				break;
			case TB_INSERTBUTTONW:
				lResult = UnhandledMsg(uMessage);
				break;
			case TB_ADDBUTTONSW:
				lResult = UnhandledMsg(uMessage);
				break;
			case TB_HITTEST:
				lResult = UnhandledMsg(uMessage);
				break;
			case TB_SETDRAWTEXTFLAGS:
				lResult = UnhandledMsg(uMessage);
				break;
			case TB_GETBUTTONTEXTW:
				lResult = UnhandledMsg(uMessage);
				break;
			case TB_REPLACEBITMAP:
#if 0
				{
				TBREPLACEBITMAP *lp = (TBREPLACEBITMAP *)lParam;
				printf("replacebitmap %x:%d %x:%d #=%d\n",
					lp->hInstOld,lp->nIDOld,
					lp->hInstNew,lp->nIDNew, lp->nButtons);

				}	
#endif
				lResult = 1;
				break;
			case TB_SETINDENT:
				pthis->nXPadding = wParam;
				lResult = 1;
				break;

			case TB_SETIMAGELIST:
				lResult = (LRESULT)pthis->himl;
				pthis->himl = (HIMAGELIST) lParam;

                                /* FIXME:  Implement wParam == iImageID */
                                /*         From version 5.80 of common  */
                                /*         controls                     */

				break;

			case TB_GETIMAGELIST:
				lResult = (LRESULT) pthis->himl;
				break;

			case TB_LOADIMAGES:
				lResult = UnhandledMsg(uMessage);
				break;
			case TB_GETRECT:
				lResult = UnhandledMsg(uMessage);
				break;

			case TB_SETHOTIMAGELIST:
				lResult = (LRESULT)pthis->himlHot;
				pthis->himlHot = (HIMAGELIST) lParam;

                                /* FIXME:  Implement wParam == iImageID */
                                /*         From version 5.80 of common  */
                                /*         controls                     */

				break;

			case TB_GETHOTIMAGELIST:
				lResult = (LRESULT) pthis->himlHot;
				break;

			case TB_GETDISABLEDIMAGELIST:
				lResult = UnhandledMsg(uMessage);
				break;
			case TB_SETDISABLEDIMAGELIST:
				lResult = UnhandledMsg(uMessage);
				break;
			case TB_SETSTYLE:
				lResult = UnhandledMsg(uMessage);
				break;
			case TB_GETSTYLE:
				lResult = UnhandledMsg(uMessage);
				break;
			case TB_SETMAXTEXTROWS:
				pthis->maxrows = wParam;
				break;
			case TB_GETTEXTROWS:
				lResult = UnhandledMsg(uMessage);
				break;
			default:
				lResult = UnhandledMsg(uMessage);
				break;
		}
	}

	if ( pthis )
		GlobalUnlock ( hToolbar );
	return lResult;
}


int WINAPI 
Toolbr_Initialize 
(	
	HINSTANCE		hinst
)
{

	WNDCLASS		wc;
	int			ErrorCode	= 0;

	memset ( &wc, 0, sizeof ( WNDCLASS ) );
	wc.style         = CS_HREDRAW | CS_VREDRAW | CS_GLOBALCLASS | CS_DBLCLKS;
	wc.lpfnWndProc   = Toolbr_WndProc;
	wc.cbWndExtra    = 0; 
	wc.hInstance     = hinst;
	wc.hCursor       = LoadCursor ( ( HINSTANCE )NULL, IDC_ARROW );
	wc.hbrBackground = GetStockObject ( LTGRAY_BRUSH );
	wc.lpszClassName = WTOOLBARCLASSNAME;

	if ( ! RegisterClass ( &wc ) )
		ErrorCode = WTLB_ERR_REGISTERCLASS;

	return ( ErrorCode );
}


void WINAPI 
Toolbr_Terminate 
(
	HINSTANCE			hInstance
) 
{
	WNDCLASS			WndClass;


	if ( GetClassInfo ( hInstance, WTOOLBARCLASSNAME, &WndClass ) )
		UnregisterClass ( WTOOLBARCLASSNAME, hInstance );

}

/*------------------------- < Full Revision History > ----------------------
** $Log: WToolbar.c,v $
** Revision 1.18  2000/01/21 06:33:50  jwhite
** Fixed menu strings locations and separator positions.
**
** Revision 1.17  2000/01/20 22:43:27  jwhite
** Revised to tweak button location and highlighting.
**
** Revision 1.16  2000/01/19 04:36:11  jwhite
** Revised to add button highlighting and improve general behavior.
**
** Revision 1.15  2000/01/19 03:59:34  robf
** make unhandled cases return bad value
**
** Revision 1.14  2000/01/17 20:05:51  jwhite
** Made disabled functions actually be disabled.
**
** Revision 1.13  2000/01/14 04:59:44  jwhite
** First pass at implementing image list draw for buttons.
**
** Revision 1.12  2000/01/13 17:47:11  jwhite
** Revised toolbar processing to enable highlighting of buttons,
** and to correctly remove the button press if you move away from the
** window.
**
** Revision 1.11  2000/01/07 09:40:20  robf
** toolbar and font usage
**
** Revision 1.10  2000/01/05 21:59:09  robf
** take 1 on different fonts
**
** Revision 1.9  2000/01/03 19:03:49  robf
** no borders
**
** Revision 1.8  1999/12/27 09:27:44  robf
** updates for visual fixes
**
** Revision 1.7  1999/12/08 20:19:25  robf
** updated comments
**
** Revision 1.6  1999/11/30 17:22:07  robf
** code cleanup
**
** Revision 1.5  1999/11/30 01:10:43  robf
** fixes to run win32 binaries
**
** Revision 1.4  1999/07/12 15:38:23  mwalsh
** Updated comments to standard header and footer style
**
**-------------------------------------------------------------------------*/
