/*--------------------------------------------------------------------------
**      GdiPrinter                Twin           From:  Twin/graphics
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      Copyright 1997 Willows Software, Inc.
**------------------------ < File Content Description > --------------------
**
**  Module:	graphics/GdiPrinter.c
**
**  Description:
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/graphics/GdiPrinter.c,v 1.4 1999/07/08 18:53:09 mwalsh Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#include "windows.h"
#include "windowsx.h"
#include "print.h"
#include "Log.h"
#include "GdiDC.h"
#include "DeviceData.h"

int	WINAPI
Escape(HDC hDC, int nEscape, int cbInput, LPCSTR lpszInData, void *lpvOutData)
{
    HDC32 hDC32;
    int result;
    LSDS_PARAMS argptr;

    APISTR((LF_API,"Escape: %x, hDC %x\n",nEscape,hDC));

    if (!((hDC32) = GETDCINFO((hDC)))) { 
	    ERRSTR((LF_ERROR,"***ERROR*** bad DC %x\n",(hDC)));
	    return (FALSE);
    }

    argptr.lsde_validate.dwInvalidMask = 0;

    argptr.lsde.escape.cbInput = cbInput;
    argptr.lsde.escape.lpszInData = (LPCSTR)lpszInData;
    argptr.lsde.escape.lpvOutData = lpvOutData;

    result = (int)DC_OUTPUT(LSD_ESCAPE,hDC32,nEscape,&argptr);
    RELEASEDCINFO(hDC32);
    return result;
}

BOOL WINAPI
QueryAbort(HDC hDC, int reserved)
{
    APISTR((LF_APISTUB,"QueryAbort(HDC=%x,int=%d)\n",hDC,reserved));
    return FALSE;
}

int WINAPI
SetAbortProc(HDC hDC, ABORTPROC lpfnAbortProc)
{
    HDC32 hDC32;
    int result;
    LSDS_PARAMS argptr;

    APISTR((LF_API,"SetAbortProc: hDC %x\n",hDC));

    if (!((hDC32) = GETDCINFO((hDC)))) { 
	    ERRSTR((LF_ERROR,"***ERROR*** bad DC %x\n",(hDC)));
	    return (FALSE);
    }

    argptr.lsde_validate.dwInvalidMask = 0;

    argptr.lsde.lpfnabort = lpfnAbortProc;

    result = (int)DC_OUTPUT(LSD_SETABORTPROC,hDC32,0,&argptr);
    RELEASEDCINFO(hDC32);
    return result;
}

int WINAPI
AbortDoc(HDC hDC)
{
    HDC32 hDC32;
    int result;

    APISTR((LF_API,"AbortDoc: hDC %x\n",hDC));

    if (!((hDC32) = GETDCINFO((hDC)))) { 
	    ERRSTR((LF_ERROR,"***ERROR*** bad DC %x\n",(hDC)));
	    return (FALSE);
    }

    result = (int)DC_OUTPUT(LSD_ABORTDOC,hDC32,0,0);
    RELEASEDCINFO(hDC32);
    return result;
}

int WINAPI
StartDoc(HDC hDC, DOCINFO *lpdi)
{
    HDC32 hDC32;
    int result;
    LSDS_PARAMS argptr;

    APISTR((LF_API,"StartDoc: hDC %x\n",hDC));

    if (!((hDC32) = GETDCINFO((hDC)))) { 
	    ERRSTR((LF_ERROR,"***ERROR*** bad DC %x\n",(hDC)));
	    return (FALSE);
    }

    argptr.lsde_validate.dwInvalidMask = 0;

    argptr.lsde.lpdocinfo = lpdi;

    result = (int)DC_OUTPUT(LSD_STARTDOC,hDC32,0,&argptr);
    RELEASEDCINFO(hDC32);
    return result;
}

int WINAPI
EndDoc(HDC hDC)
{
    HDC32 hDC32;
    int result;

    APISTR((LF_API,"EndDoc: hDC %x\n",hDC));

    if (!((hDC32) = GETDCINFO((hDC)))) { 
	    ERRSTR((LF_ERROR,"***ERROR*** bad DC %x\n",(hDC)));
	    return (FALSE);
    }

    result = (int)DC_OUTPUT(LSD_ENDDOC,hDC32,0,0);
    RELEASEDCINFO(hDC32);    
    return result;
}

int WINAPI
StartPage(HDC hDC)
{
    HDC32 hDC32;
    int result;

    APISTR((LF_API,"StartPage: hDC %x\n",hDC));

    if (!((hDC32) = GETDCINFO((hDC)))) { 
	    ERRSTR((LF_ERROR,"***ERROR*** bad DC %x\n",(hDC)));
	    return (FALSE);
    }

    result = (int)DC_OUTPUT(LSD_STARTPAGE,hDC32,0,0);
    RELEASEDCINFO(hDC32);
    return result;
}

int WINAPI
EndPage(HDC hDC)
{
    HDC32 hDC32;
    int result;

    APISTR((LF_API,"EndPage: hDC %x\n",hDC));

    if (!((hDC32) = GETDCINFO((hDC)))) { 
	    ERRSTR((LF_ERROR,"***ERROR*** bad DC %x\n",(hDC)));
	    return (FALSE);
    }

    result = (int)DC_OUTPUT(LSD_ENDPAGE,hDC32,0,0);
    RELEASEDCINFO(hDC32);
    return result;
}

/*------------------------- < Full Revision History > ----------------------
** $Log: GdiPrinter.c,v $
** Revision 1.4  1999/07/08 18:53:09  mwalsh
** Updated Comments
**
**-------------------------------------------------------------------------*/