/*--------------------------------------------------------------------------
**      DrvDP.h               Twin           From:  Twin/graphics/x11drv
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      Copyright 1997 Willows Software, Inc.
**------------------------ < File Content Description > --------------------
**
**  Module:	graphics/x11drv/DrvDP.h
**
**  Description:
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/graphics/x11drv/DrvDP.h,v 1.3 1999/07/08 18:53:32 mwalsh Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/

#ifndef DrvDP__h
#define DrvDP__h

	/* enable new palette code */
#define TWIN_PALETTE

/* display specific information */
typedef struct {
	Display	*display;
	int     screen;
	BOOL	bConnected;

	BOOL 	 HardwareEnabled;
	BOOL 	 SwappedButtons;
	int	 CodePage;

	DWORD	 LastClickTime;
	DWORD 	 DoubleClickTime;
	DWORD	 ServerTime;

	RECT	 rcCursorClip;
	BOOL	 fCursorClipped;

	char 	*DisplayName;
	char    *VendorName;
	int     bSyncronous;

#ifdef TWIN_PALETTE
	/* uSystemPaletteUse
	 *	 - SYSPAL_STATIC (20 static colors in system palette)
	 *	 - SYSPAL_NOSTATIC (2 static colors in system palette)
	 * uSystemPaletteUseCount
	 *	 - use count for each system palette entry
	 *	   (decremented to zero during palette initialization)
	 *	   (incremented by one during palette realization)
	 * hSystemPalette
	 *	 - Windows system palette
	 * hOldSystemPalette
	 *	 - previous Windows system palette (used by UpdateColors())
	 * SystemPalette
	 *	 - X system palette
	 */
	UINT     uSystemPaletteUse;
	LPUINT	 uSystemPaletteUseCount;
	HPALETTE hSystemPalette;
	HPALETTE hOldSystemPalette;
	Colormap SystemPalette;
#else
	Colormap color_map;
#endif

	DWORD	 pixel_base;		/* start of pixel space */
	DWORD	 pixel_mask;		/* pixel bit mask to use */
	int	 nNumPixels;		/* number of X pixels used by us */

	int	 BitsPixel;		/* bits per pixel in ZPixmap image */
	int	 Depth;			/* DefaultDepth of the screen */
	int	 BitOrder;		/* MSBFirst/LSBFirst */
	int	 ByteOrder;		/* image byte order */

	UINT	 ErrorCode;
	LONG	 KeyBoardType;
	LONG	 KeyBoardSubType;
	LONG	 KeyBoardFuncKeys;

	Atom	wm_protocols;		
	Atom	wm_delete_window;

	Atom	ipc_protocols;	

	Atom	XFormatAtom;
	Atom	XDataAtom;
	Atom	XCutBuffer;

	Atom	atmColormapProperty;

	XContext client_hwnd;
	XContext window_style;
	XContext window_ex_style;
} PRIVATEDISPLAY;

PRIVATEDISPLAY *GETDP();

#define	ASSERT_CONNECTION(r)	{ PRIVATEDISPLAY *dp = GETDP(); \
				  if (!dp->bConnected) return (r) ;}

Region TSXCreateRegion();
void   TSXDestroyRegion(Region);
GC     TSXCreateGC(Display *, Drawable, unsigned long , XGCValues *);
void   TSXFreeGC(Display *, GC );

void  * TSXAllocDDC(int);
void   TSXFreeDDC(void *);

#endif /* DrvDP__h */

/*------------------------- < Full Revision History > ----------------------
** $Log: DrvDP.h,v $
** Revision 1.3  1999/07/08 18:53:32  mwalsh
** Updated to Standard Header and Footer comments
**
**-------------------------------------------------------------------------*/