/*--------------------------------------------------------------------------
**     io.h                Twin           From:  Twin/include
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**
**------------------------ < File Content Description > --------------------
**
**  Module:	include/io.h
**
**  Description:
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/include/io.h,v 1.2 1999/07/12 21:13:44 mwalsh Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/



#ifndef IO_H
#define	IO_H


/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/
#include <time.h>
#include <unistd.h>
#include <fcntl.h>

/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/


/*
 *  ftime structure used by getftime()/setftime().
 */
struct ftime
{
    unsigned ft_tsec  : 5;   /* 2-second intervals */
    unsigned ft_min   : 6;   /* minutes */
    unsigned ft_hour  : 5;   /* hours */
    unsigned ft_day   : 5;   /* days */
    unsigned ft_month : 4;   /* months */
    unsigned ft_year  : 7;   /* year */
};

/*
 * _fsize_t is used by some MSVC items, including _finddata_t struct.
 */
typedef unsigned long _fsize_t;

/*
 *  _finddata_t structure is used by MSVC's _findfirst()/_findnext()
 *  functions.
 */
struct _finddata_t
{
    unsigned attrib;
    time_t time_create;
    time_t time_access;
    time_t time_write;
    _fsize_t size;
    char name[260];
};

/*
 *  Define some prototypes and synonyms for various Microsoft and
 *  Borland library calls we support.
 */

#ifdef __cplusplus
extern "C" {
#endif

long _filelength(int handle);
long filelength(int handle);
int getftime(int handle, struct ftime *ftimep);
int setftime(int handle, struct ftime *ftimep);
long _findfirst(const char *filespec, struct _finddata_t *fileinfo);
int _findnext(long handle, struct _finddata_t *fileinfo);


#define _O_RDONLY	O_RDONLY
#define _O_TEXT		0

#ifdef __cplusplus
}
#endif

#endif /* IO_H */

/*------------------------- < Full Revision History > ----------------------
** $Log: io.h,v $
** Revision 1.2  1999/07/12 21:13:44  mwalsh
** Updated comments to utilize standard header and footer
**
**-------------------------------------------------------------------------*/
