
#ifndef __shellapi__
#define __shellapi__

/*****************************************************************************\
*                                                                             *
* shellapi.h -  SHELL.DLL functions, types, and definitions                   *
*                                                                             *
*               Copyright (c) 1992, Microsoft Corp.  All rights reserved      *
*                                                                             *
\*****************************************************************************/

#ifndef _INC_SHELLAPI
#define _INC_SHELLAPI


#ifndef WINSHELLAPI
#if !defined(_SHELL32_)
#define WINSHELLAPI DECLSPEC_IMPORT
#else
#define WINSHELLAPI
#endif
#endif // WINSHELLAPI


#ifdef	MSC
#ifndef RC_INVOKED
#pragma pack(1)         /* Assume byte packing throughout */
#endif /* RC_INVOKED */
#endif

#ifdef __cplusplus
extern "C" {            /* Assume C declarations for C++ */
#endif	/* __cplusplus */

struct HKEY__;
typedef struct HKEY__ *HKEY;
typedef HKEY FAR* PHKEY;


#ifndef HKEY_IS_DEFINED
#define HKEY_IS_DEFINED
#endif /* HKEY_IS_DEFINED */

#if !defined(TWIN32)
/* return codes from Registration functions */
#define ERROR_SUCCESS           0L
#define ERROR_BADDB             1L
#define ERROR_BADKEY            2L
#define ERROR_CANTOPEN          3L
#define ERROR_CANTREAD          4L
#define ERROR_CANTWRITE         5L
#define ERROR_OUTOFMEMORY       6L
#define ERROR_INVALID_PARAMETER 7L
#define ERROR_ACCESS_DENIED     8L

#define HKEY_CLASSES_ROOT	1

# else

#define HKEY_CLASSES_ROOT           ((HKEY)0x80000000UL)
#define HKEY_CURRENT_USER           ((HKEY)0x80000001UL)
#define HKEY_LOCAL_MACHINE          ((HKEY)0x80000002UL)
#define HKEY_USERS                  ((HKEY)0x80000003UL)
#define HKEY_PERFORMANCE_DATA       ((HKEY)0x80000004UL)
#define HKEY_CURRENT_CONFIG         ((HKEY)0x80000005UL)
#define HKEY_DYN_DATA               ((HKEY)0x80000006UL)

#endif /* TWIN32 */

#define REG_SZ			(1)

/*
** Requested Key access mask type.
*/
typedef ACCESS_MASK REGSAM;

#ifndef NOAPIPROTO

LONG WINAPI RegOpenKey(HKEY, LPCSTR, HKEY FAR*);
LONG WINAPI RegCreateKey(HKEY, LPCSTR, HKEY FAR*);
LONG WINAPI RegCloseKey(HKEY);
LONG WINAPI RegDeleteKey(HKEY, LPCSTR);
LONG WINAPI RegSetValue(HKEY, LPCSTR, DWORD, LPCSTR, DWORD);
LONG WINAPI RegQueryValue(HKEY, LPCSTR, LPSTR, LONG FAR*);
LONG WINAPI RegEnumKey(HKEY, DWORD, LPSTR, DWORD);
LONG WINAPI RegOpenKeyEx ( HKEY, LPCSTR, DWORD, REGSAM, PHKEY);
LONG WINAPI RegQueryValueEx ( HKEY, LPCSTR, LPDWORD, LPDWORD, LPBYTE, LPDWORD);
LONG RegCreateKeyEx(
    HKEY hKey,	// handle of an open key 
    LPCTSTR lpSubKey,	// address of subkey name 
    DWORD Reserved,	// reserved 
    LPTSTR lpClass,	// address of class string 
    DWORD dwOptions,	// special options flag 
    REGSAM samDesired,	// desired security access 
    LPSECURITY_ATTRIBUTES lpSecurityAttributes,	// address of key security structure 
    PHKEY phkResult,	// address of buffer for opened handle  
    LPDWORD lpdwDisposition 	// address of disposition value buffer 
   );

LONG RegSetValueEx(
    HKEY hKey,		// handle of key to set value for  
    LPCTSTR lpValueName,// address of value to set 
    DWORD Reserved,	// reserved 
    DWORD dwType,	// flag for value type 
    CONST BYTE *lpData,	// address of value data 
    DWORD cbData 	// size of value data 
   );	
 
LONG RegDeleteValue(
    HKEY hKey,	// handle of key 
    LPCTSTR lpValueName 	// address of value name 
   );	
 

UINT WINAPI DragQueryFile(HDROP, UINT, LPSTR, UINT);
BOOL WINAPI DragQueryPoint(HDROP, POINT FAR*);
void WINAPI DragFinish(HDROP);
void WINAPI DragAcceptFiles(HWND, BOOL);

HICON WINAPI ExtractIcon(HINSTANCE hInst, LPCSTR lpszExeFileName, UINT nIconIndex);

#endif	/* NOAPIPROTO */

/* error values for ShellExecute() beyond the regular WinExec() codes */
#define SE_ERR_SHARE            26
#define SE_ERR_ASSOCINCOMPLETE  27
#define SE_ERR_DDETIMEOUT       28
#define SE_ERR_DDEFAIL          29
#define SE_ERR_DDEBUSY          30
#define SE_ERR_NOASSOC          31

#ifndef NOAPIPROTO

HINSTANCE WINAPI ShellExecute(HWND hwnd, LPCSTR lpOperation, LPCSTR lpFile, LPCSTR lpParameters, LPCSTR lpDirectory, int iShowCmd);
HINSTANCE WINAPI FindExecutable(LPCSTR lpFile, LPCSTR lpDirectory, LPSTR lpResult);

#ifndef UNICODE_ONLY
typedef struct _SHFILEINFOA
{
	HICON	hIcon;
	int	iIcon;
	DWORD	dwAttributes;
	CHAR	szDisplayName[MAX_PATH];
	CHAR	szTypeName[80];
} SHFILEINFOA;
#endif	/* UNICODE_ONLY */

#ifndef ANSI_ONLY
typedef struct _SHFILEINFOW
{
	HICON	hIcon;
	int	iIcon;
	DWORD	dwAttributes;
	WCHAR	szDisplayName[MAX_PATH];
	WCHAR	szTypeName[80];
} SHFILEINFOW;
#endif	/* ANSI_ONLY */

#ifdef UNICODE
typedef SHFILEINFOW	SHFILEINFO;
#else
typedef SHFILEINFOA	SHFILEINFO;
#endif	/* UNICODE */

DWORD WINAPI SHGetFileInfo(
    LPCTSTR pszPath, 	
    DWORD dwFileAttributes, 	
    SHFILEINFO *psfi, 	
    UINT cbFileInfo, 	
    UINT uFlags	
   );	

#endif	/* NOAPIPROTO */

/* SHFILEINFO */



typedef struct _NOTIFYICONDATAA {
        DWORD cbSize;
        HWND hWnd;
        UINT uID;
        UINT uFlags;
        UINT uCallbackMessage;
        HICON hIcon;
        CHAR   szTip[64];
} NOTIFYICONDATAA, *PNOTIFYICONDATAA;
typedef struct _NOTIFYICONDATAW {
        DWORD cbSize;
        HWND hWnd;
        UINT uID;
        UINT uFlags;
        UINT uCallbackMessage;
        HICON hIcon;
        WCHAR  szTip[64];
} NOTIFYICONDATAW, *PNOTIFYICONDATAW;
#ifdef UNICODE
typedef NOTIFYICONDATAW NOTIFYICONDATA;
typedef PNOTIFYICONDATAW PNOTIFYICONDATA;
#else
typedef NOTIFYICONDATAA NOTIFYICONDATA;
typedef PNOTIFYICONDATAA PNOTIFYICONDATA;
#endif // UNICODE


#define NIM_ADD         0x00000000
#define NIM_MODIFY      0x00000001
#define NIM_DELETE      0x00000002

#define NIF_MESSAGE     0x00000001
#define NIF_ICON        0x00000002
#define NIF_TIP         0x00000004

WINSHELLAPI BOOL WINAPI Shell_NotifyIconA(DWORD dwMessage, PNOTIFYICONDATAA lpData);
WINSHELLAPI BOOL WINAPI Shell_NotifyIconW(DWORD dwMessage, PNOTIFYICONDATAW lpData);
#ifdef UNICODE
#define Shell_NotifyIcon  Shell_NotifyIconW
#else
#define Shell_NotifyIcon  Shell_NotifyIconA
#endif // !UNICODE


////
//// Shell File Operations
////

#ifndef FO_MOVE //these need to be kept in sync with the ones in shlobj.h

#define FO_MOVE           0x0001
#define FO_COPY           0x0002
#define FO_DELETE         0x0003
#define FO_RENAME         0x0004

#define FOF_MULTIDESTFILES         0x0001
#define FOF_CONFIRMMOUSE           0x0002
#define FOF_SILENT                 0x0004  // don't create progress/report
#define FOF_RENAMEONCOLLISION      0x0008
#define FOF_NOCONFIRMATION         0x0010  // Don't prompt the user.
#define FOF_WANTMAPPINGHANDLE      0x0020  // Fill in SHFILEOPSTRUCT.hNameMapp
                                       // Must be freed using SHFreeNameMapping
#define FOF_ALLOWUNDO              0x0040
#define FOF_FILESONLY              0x0080  // on *.*, do only files
#define FOF_SIMPLEPROGRESS         0x0100  // means don't show names of files
#define FOF_NOCONFIRMMKDIR         0x0200  // don't confirm making any needed

typedef WORD FILEOP_FLAGS;

typedef struct _SHFILEOPSTRUCT
{
        HWND            hwnd;
        UINT            wFunc;
        LPCSTR          pFrom;
        LPCSTR          pTo;
        FILEOP_FLAGS    fFlags;
        BOOL            fAnyOperationsAborted;
        LPVOID          hNameMappings;
        LPCSTR           lpszProgressTitle; 
} SHFILEOPSTRUCT, FAR *LPSHFILEOPSTRUCT;

#endif

#define SHGFI_LARGEICON		0x000000000
#define SHGFI_SMALLICON		0x000000001
#define SHGFI_OPENICON		0x000000002
#define SHGFI_SHELLICONSIZE	0x000000004
#define SHGFI_PIDL		0x000000008
#define SHGFI_USEFILEATTRIBUTES	0x000000010
#define SHGFI_ICON		0x000000100
#define SHGFI_DISPLAYNAME	0x000000200
#define SHGFI_TYPENAME		0x000000400
#define SHGFI_ATTRIBUTES	0x000000800
#define SHGFI_ICONLOCATION	0x000001000
#define SHGFI_EXETYPE		0x000002000
#define SHGFI_SYSICONINDEX	0x000004000
#define SHGFI_LINKOVERLAY	0x000008000
#define SHGFI_SELECTED		0x000010000

#define SFGAO_CANCOPY		DROPEFFECT_COPY
#define SFGAO_CANMOVE		DROPEFFECT_MOVE
#define SFGAO_CANLINK		DROPEFFECT_LINK
#define SFGAO_CANRENAME		0x00000010L
#define SFGAO_CANDELETE		0x00000020L
#define SFGAO_HASPROPSHEET	0x00000040L
#define SFGAO_DROPTARGET	0x00000100L
#define SFGAO_CAPABILITYMASK	0x00000177L
#define SFGAO_LINK		0x00010000L
#define SFGAO_SHARE		0x00020000L
#define SFGAO_READONLY		0x00040000L
#define SFGAO_GHOSTED		0x00080000L
#define SFGAO_DISPLAYATTRMASK	0x000F0000L
#define SFGAO_FILESYSANCESTOR	0x10000000L
#define SFGAO_FOLDER		0x20000000L
#define SFGAO_FILESYSTEM	0x40000000L
#define SFGAO_HASSUBFOLDER	0x80000000L
#define SFGAO_CONTENTSMASK	0x80000000L
#define SFGAO_VALIDATE		0x01000000L
#define SFGAO_REMOVABLE		0x02000000L

/*************************************************/

typedef struct _AppBarData
{
    DWORD cbSize;
    HWND hWnd;
    UINT uCallbackMessage;
    UINT uEdge;
    RECT rc;
    LPARAM lParam;
} APPBARDATA, *PAPPBARDATA;

#define ABM_NEW           0x00000000
#define ABM_REMOVE        0x00000001
#define ABM_QUERYPOS      0x00000002
#define ABM_SETPOS        0x00000003
#define ABM_GETSTATE      0x00000004
#define ABM_GETTASKBARPOS 0x00000005
#define ABM_ACTIVATE      0x00000006
#define ABM_GETAUTOHIDEBAR 0x00000007
#define ABM_SETAUTOHIDEBAR 0x00000008
#define ABM_WINDOWPOSCHANGED 0x0000009

#define ABN_STATECHANGE 	0
#define ABN_POSCHANGED 		1
#define ABN_FULLSCREENAPP	2
#define ABN_WINDOWARRANGE	3

#define ABE_LEFT        0
#define ABE_TOP         1
#define ABE_RIGHT       2
#define ABE_BOTTOM      3

#define ABS_AUTOHIDE	1
#define ABS_ALWAYSONTOP 2

#ifdef __cplusplus
}
#endif	/* __cplusplus */

#ifdef	MSC
#ifndef RC_INVOKED
#pragma pack()
#endif  /* RC_INVOKED */
#endif

#endif  /* _INC_SHELLAPI */
#endif /* __shellapi__ */
