/*--------------------------------------------------------------------------
**      shlobj.h                 Twine           From:  Wine/include
**
**              
**------------------------- < License Information > ------------------------
**
**      This file is a direct copy of the equivalent file in the
**  Wine project (www.winehq.com).  The Wine license permits
**  copying of source code, so long as all copyright and credit
**  information is preserved.  Any and all copyrights originally
**  in the Wine source code file are preserved exactly as they were
**  when this copy was made from the Wine source tree.
**  (see the 'Original Copyright(s)', below).
**  An exacty copy of the Wine license is contained in WINE-LICENSE.
**  
**      For a more general description of the Wine license, and
**  for a full list of the authors who deserve credit for this
**  code, please see www.winehq.com, or WINE-AUTHORS.
**  
**      However, because this file is released with Twine, we
**  are obliged to also release it under the LGPL license.
**  In short, you may make a copy or republish this code so long as
**  all of your changes to source code are again made publicly available.
**  Further, absolutely no warranty of any kind is given or implied.
**  Refer to the LGPL description at www.fsf.org or the file LICENSE 
**  for further details.
**  
**  
**      ==========================================================
**      IF YOU MAKE CHANGES TO THIS CODE, PLEASE REFLECT
**      THOSE CHANGES IN THE ORIGINAL WINE CODE, AT WWW.WINEHQ.COM
**      ==========================================================
**
**      
**------------------------ < Original Copyright(s) > -----------------------
**      
**------------------------ < File Content Description > --------------------
**
**  Module:	include/shlobj.h
**
**  Description:
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/include/shlobj.h,v 1.4 1999/07/31 02:29:16 jwhite Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/

#ifndef _SHLOBJ_H_
#define _SHLOBJ_H_

/*stub*/
/* used by filecore.cpp */

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#include "objbase.h"

#ifdef __cplusplus
extern "C" {            /* Assume C declarations for C++ */

#endif  /* __cplusplus */

typedef struct {
	USHORT cb;
	BYTE   abID[1];
} SHITEMID;

typedef struct {
	SHITEMID mkid;
} ITEMIDLIST;
typedef ITEMIDLIST *LPITEMIDLIST;
typedef const ITEMIDLIST *LPCITEMIDLIST;

/*  Removed STDMETHODCALLTYPE empty definition, as it conflicts with
    new code .  JPW 07/29/99  */

//
// IShellLink Interface
//

#ifdef UNICODE
#define IShellLink      IShellLinkW
#define IShellLinkVtbl  IShellLinkWVtbl
#else
#define IShellLink      IShellLinkA
#define IShellLinkVtbl  IShellLinkAVtbl
#endif

// IShellLink::Resolve fFlags
typedef enum {
    SLR_NO_UI           = 0x0001,
    SLR_ANY_MATCH       = 0x0002,
    SLR_UPDATE          = 0x0004,
    SLR_NOUPDATE        = 0x0008,
} SLR_FLAGS;

// IShellLink::GetPath fFlags
typedef enum {
    SLGP_SHORTPATH      = 0x0001,
    SLGP_UNCPRIORITY    = 0x0002,
    SLGP_RAWPATH        = 0x0004,
} SLGP_FLAGS;

#undef  INTERFACE
#define INTERFACE   IShellLinkA

DECLARE_INTERFACE_(IShellLinkA, IUnknown)
{
    // *** IUnknown methods ***
    STDMETHOD(QueryInterface) (THIS_ REFIID riid, LPVOID * ppvObj) PURE;
    STDMETHOD_(ULONG,AddRef) (THIS)  PURE;
    STDMETHOD_(ULONG,Release) (THIS) PURE;

    // *** IShellLink methods ***
    STDMETHOD(GetPath)(THIS_ LPSTR pszFile, int cchMaxPath, WIN32_FIND_DATA *pfd, DWORD fFlags) PURE;

    STDMETHOD(GetIDList)(THIS_ LPITEMIDLIST * ppidl) PURE;
    STDMETHOD(SetIDList)(THIS_ LPCITEMIDLIST pidl) PURE;

    STDMETHOD(GetDescription)(THIS_ LPSTR pszName, int cchMaxName) PURE;
    STDMETHOD(SetDescription)(THIS_ LPCSTR pszName) PURE;

    STDMETHOD(GetWorkingDirectory)(THIS_ LPSTR pszDir, int cchMaxPath) PURE;
    STDMETHOD(SetWorkingDirectory)(THIS_ LPCSTR pszDir) PURE;

    STDMETHOD(GetArguments)(THIS_ LPSTR pszArgs, int cchMaxPath) PURE;
    STDMETHOD(SetArguments)(THIS_ LPCSTR pszArgs) PURE;

    STDMETHOD(GetHotkey)(THIS_ WORD *pwHotkey) PURE;
    STDMETHOD(SetHotkey)(THIS_ WORD wHotkey) PURE;

    STDMETHOD(GetShowCmd)(THIS_ int *piShowCmd) PURE;
    STDMETHOD(SetShowCmd)(THIS_ int iShowCmd) PURE;

    STDMETHOD(GetIconLocation)(THIS_ LPSTR pszIconPath, int cchIconPath, int *piIcon) PURE;
    STDMETHOD(SetIconLocation)(THIS_ LPCSTR pszIconPath, int iIcon) PURE;

    STDMETHOD(SetRelativePath)(THIS_ LPCSTR pszPathRel, DWORD dwReserved) PURE;

    STDMETHOD(Resolve)(THIS_ HWND hwnd, DWORD fFlags) PURE;

    STDMETHOD(SetPath)(THIS_ LPCSTR pszFile) PURE;
};

#ifdef __cplusplus
}

#endif  /* __cplusplus */

#ifndef RC_INVOKED
#pragma pack()
#endif  /* !RC_INVOKED */


#if ! defined(USES_CONVERSION)
#define USES_CONVERSION int _convert = 0
#endif

#if ! defined(EXTERN_C)
#ifdef __cplusplus__
#define EXTERN_C extern "C"
#else
#define EXTERN_C extern
#endif
#endif

#pragma option pop /*P_O_Pop*/
#endif // _SHLOBJ_H_

/*------------------------- < Full Revision History > ----------------------
** $Log: shlobj.h,v $
** Revision 1.4  1999/07/31 02:29:16  jwhite
** Removed a newly conflicting #define, better protected EXTERN_C
**
** Revision 1.3  1999/07/15 17:39:39  jwhite
** Protected #define on USES_CONVERSION
**
** Revision 1.2  1999/07/13 20:17:06  mwalsh
** Updated comments to use standard header and footer.
**
**-------------------------------------------------------------------------*/
