/*--------------------------------------------------------------------------
**      Module.h                 Twin           From:  Twin/include
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      Copyright 1997 Willows Software, Inc. 
**------------------------ < File Content Description > --------------------
**
**  Module:	include/Module.h
**
**  Description:
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/include/twine/Module.h,v 1.1 1999/07/15 17:56:55 jwhite Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/



 
#ifndef Module__h
#define Module__h

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#include "ModTable.h"
#include "ObjEngine.h"
#include "setjmp.h"

/* **** InternalLoadLibrary flags **** */

#define ILL_APPL	0x0001
#define ILL_DSCR	0x0002
#define ILL_BINARY	0x0004
#define ILL_FILENAME	0x0008
#define ILL_NOERROR	0x0010
#define ILL_NOLOAD	0x0020
#define ILL_EXEC	0x0040
#define ILL_RESOURCE	0x0080
#define ILL_INTERMED	0x0100
#define ILL_FINAL	0x0200
#define ILL_NOFORK	0x0400
#define ILL_NEWINST	0x0800
#define ILL_SHELL  	0x1000

#define BUFFER 0x200

typedef struct tagUSERDLL
  {
	HFILE	hfExeFile;
	WORD	wSelectorBase;
	LPBYTE	lpHeaderImage;
	LPBYTE	lpEntryTable;
	LPBYTE	lpImportTable;
	LPBYTE	lpSegmentTable;
	LPWORD	lpModuleTable;
	LPBYTE	lpNonResTable;
	DWORD	dwNEOffset;
	DWORD	dwResPart;
	WORD	wProgramFlags;
	BYTE	bAdditFlags;
	WORD	wFLOffset;
	WORD	wFLLength;
	WORD	wAutoData;
	WORD	wInitHeap;
	WORD	wInitStack;
	WORD	wInitCS;
	WORD	wInitIP;
	WORD	wInitSS;
	WORD	wInitSP;
	WORD	wStackBottom;
	WORD	wSegCount;
	WORD	wSegOffset;
	WORD	wModCount;
	WORD	wModOffset;
	WORD	wEntryTblSize;
	WORD	wEntryTblOffset;
	WORD	wResourceTblOffset;
	WORD	wNResTblSize;
	DWORD	dwNResTblOffset;
	WORD	wResTblOffset;
	WORD	wImpTblOffset;
	WORD    wMovCount;
	WORD	wSectorShift;
	WORD	wTargetOS;
	WORD	wWinVer;
	LPHANDLE lpModIndex;
	WORD	wNumEntries;
	LPSTR	lpDLLName;
	LPSTR	lpDLLDescription;
  } USERDLL;

typedef	USERDLL *LPUSERDLL;

#define START_PARA      0x0010
#define	__ROMBIOS	0x00ad
#define __WINFLAGS      0x00b2
#define __C000H		0x00c3

#define	__C000H_BY_NAME	"__C000H"

#define LEXE_exit(x) {LEXE_perror(x);_lclose(lpDLL->hfExeFile); return(-x);}

/* Module information structure */
typedef struct tagMODULEINFO
  {
    OBJHEAD	ObjHead;		/* generic object header */
    LPSTR	lpModuleName;		/* ptr to the program name */
    LPSTR	lpFileName;
    LPSTR	lpCmdline;		/* lpCmdline: getcommandline() */
    LPSTR	lpModuleDescr;		/* lpDLL: module description string */
    TYPEINFO	*ResourceTable;
    ENTRYTAB	*lpEntryTable;
    SEGTAB	*lpSegTable;		/* segment table for LOADONCALL */
    LPHANDLE	lpModIndex;
    USERDLL	*lpDLL;
    ATOM	atmModuleName;
    WORD	wDGROUP;		/* selector assigned to DGROUP */
    HANDLE	hInst;			/* global handle of DGROUP */
    WORD	wSegCount;		/* number of segments in module */
    WORD	wSelBase;		/* first sel assigned to module */
    WORD	wProgramFlags;
    WORD	wSectorShift;
    WORD	bModIndex;		/* index into module table */
    BOOL	fMakeInstance;		/* when "reloading", make a new inst */
    HGLOBAL     InitTHDB;		/* tdb for initial thread */
    WORD	TIBSelector;

    int	        nRefCnt;		/* how many times LoadLibraried */
    void       *hSharedObject;		/* native tag for shared object */

    int   exit_code;
    jmp_buf	env;

    DWORD	dwWin32flag;		/* win16 or win32 application */
    void       *lpNTHeader;
    void       *lpNTBaseAddress;
    TWINRT     *lptwinrt;		/* pointer to resource table... */
    MAPTABLE   *map_table;		/* win32 interface table */

    void * (*start)(int,int,int); 	/* the start address */
  } MODULEINFO;
 
typedef MODULEINFO *LPMODULEINFO;

typedef struct tagPARAMBLOCK {
    WORD wSegEnv;
    LPSTR lpszCmdLine;
    UINT *lpShow;
    UINT *lpReserved;
 } PARAMBLOCK;

/* SEGMENTINFO struct for utility function GetSegmentInfo() */
typedef struct tagSEGMENTINFO {
    LPSTR lpModuleName;
    LPSTR lpFileName;
    int nSegmentOrd;
    WORD wSegType;
} SEGMENTINFO;
    
#endif /* Module__h */

/*------------------------- < Full Revision History > ----------------------
** $Log: Module.h,v $
** Revision 1.1  1999/07/15 17:56:55  jwhite
** Moved files that are not clearly a part of the Win32 API specification
** out of the main include directory into a 'twine' subdirectory.
** This is going to allow us to more readily integrate Wine include files.
**
** Revision 1.3  1999/07/12 21:13:43  mwalsh
** Updated comments to utilize standard header and footer
**
**-------------------------------------------------------------------------*/
