/*--------------------------------------------------------------------------
**      Resources.h                Twin           From:  Twin/include
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      Copyright 1997 Willows Software, Inc. 
**------------------------ < File Content Description > --------------------
**
**  Module:	include/Resources.h
**
**  Description:
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/include/twine/Resources.h,v 1.3 2000/01/26 19:55:28 jwhite Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/



#ifndef Resources__h
#define Resources__h

#ifndef RESOURCE_H
#define RESOURCE_H

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/


#define ALIGN_16BIT(x)		((long)((long)(x)+1) & ~1)
#define ALIGN_32BIT(x)		((long)((long)(x)+3) & ~3)


typedef struct tagNAMEENTRY {
	WORD	wTypeOrdinal;
	WORD	wIdOrdinal;
	LPSTR	szType;
	LPSTR	szId;
} NAMEENTRY;	

typedef struct tagCURSORDIRENTRY {
	WORD	wWidth;
	WORD	wHeight;
	WORD	wPlanes;
	WORD	wBitsPerPel;
	DWORD	dwBytesInRes;
	WORD	wOrdinalNumber;
} CURSORDIRENTRY;

typedef struct tagICONDIRENTRY {
	char	bWidth;
	char	bHeight;
	char	bColorCount;
	WORD	wPlanes;
	WORD	wBitsPerPel;
	DWORD	dwBytesInRes;
	WORD	wOrdinalNumber;
} ICONDIRENTRY;

typedef struct iconitem {
	/* the first three elements are equivalent to MENUITEMTEMPLATE,
         * we convert them to this form for internal use...
	 */ 
	WORD  iconflag;
	WORD  iconid;
	LPSTR iconname;

	/* this element points to a popup if menuflag & MF_POPUP */
	struct iconitem *icondata;
} ICONDATA;

typedef struct menuitem {
	/* the first three elements are equivalent to MENUITEMTEMPLATE,
         * we convert them to this form for internal use...
	 */ 
	WORD  menuflag;
	WORD  menuid;
	LPSTR menuname;

	/* this element points to a popup if menuflag & MF_POPUP */
	struct menuitem *menupopup;
} MENUDATA;

typedef struct {
	WORD 	x,y;
	WORD	cx,cy;
	DWORD	dwID;
	DWORD   lStyle;
	DWORD   dwExtStyle;
	LPSTR   szClass;
	LPSTR   szText;	
        DWORD   dwHelpId;
        LPBYTE  bData;
} CONTROLDATA;

typedef struct {
	DWORD	lStyle;
	DWORD	dwExtStyle;
        BOOL    dialogEx;      /* TRUE if DIALOGEX */
        DWORD   dwHelpId;
	WORD    bNumberOfItems;
	WORD	x,y;
	WORD    cx,cy;
	LPSTR   szMenuName;
	LPSTR   szClassName;
	LPSTR 	szCaption;
	WORD	wPointSize;
	LPSTR   szFaceName;
        WORD    wWeight;
        WORD    wItalic;
	CONTROLDATA *controlinfo;
} DIALOGDATA;

typedef struct {		/* MFC 4.x internal resource */
	WORD	wDataSize;
	BYTE	bData[1024];
} DLGINITDATA;

typedef struct {		/* MFC 4.x internal resource */
	WORD	wVersion;
	WORD	wWidth;
	WORD	wHeight;
	WORD	wItemCount;
	WORD	aItems[32];
} TOOLBARDATA;

typedef struct {
	LPBYTE	rcsdata;	/* INTEL data */
	WORD	wType;		/* index into conversion routines table	*/
	HGLOBAL	hGlobal;	/* memory handle of resource	   */
	DWORD	rcsoffset;	/* location in resource file       */
	DWORD	rcslength;	/* length of data in resource file */
	WORD	rcsflags;	/* pre-loaded, discardable...      */
	HRSRC	hRsrc;		/* handle to 'found' resource      */
	HGDIOBJ	hObject;	/* GDI object associated with resource */
	LPSTR	rcsitemname;	/* name/ordinal of resource        */
} NAMEINFO;
typedef NAMEINFO *LPNAMEINFO;

typedef char	RCDATAENTRY;

typedef struct	tagCURSORIMAGE {
	short	wHotSpotX;
	short	wHotSpotY;
	BITMAPINFOHEADER *bmp_hdr;
	RGBQUAD	*rgb_quad;
	LPBYTE	xor_mask;
	LPBYTE	and_mask;
} CURSORIMAGE;

typedef struct	tagBITMAPIMAGE {
	BITMAPINFOHEADER *bmp_hdr;
	RGBQUAD	*rgb_quad;
	LPBYTE	bitmap_bits;
} BITMAPIMAGE;
typedef BITMAPIMAGE *LPBITMAPIMAGE;

typedef struct	tagICONIMAGE {
	BITMAPINFOHEADER *icon_bmp_hdr;
	RGBQUAD		 *icon_rgb_quad;
	LPBYTE		 icon_xor_mask;
	LPBYTE		 icon_and_mask;
} ICONIMAGE;
typedef ICONIMAGE *LPICONIMAGE;

typedef struct {
	LPSTR     rcstypename;
	WORD  rcscount;
	NAMEINFO *rcsinfo;
} TYPEINFO;

typedef struct	tagACCELENTRY {
	BYTE	fFlags;
	WORD	wEvent;
	WORD	wId;
	WORD	wPadding;
} ACCELENTRY;

typedef LPCSTR STRINGENTRY;

#define CONTROLITEM(x,y,cx,cy,id,style,exstyle,class,text,helpid,data) { \
	x,y, \
	cx,cy, \
	id, \
	style, \
	exstyle, \
	class, \
	text,  \
	helpid,  \
	data,  \
}
#define ACCELITEM(f,event,wid) { f,event,wid }

#define MENUITEM(type,id,text,submenu) { \
	type,	id,	text,	submenu }

#define RT_USER_DEF	MAKEINTRESOURCE(11)
#define RT_NAME_TABLE	MAKEINTRESOURCE(15)

#define TEMPL_SIZE 0x14
#define MAXSYSTYPE (WORD)(DWORD)RT_NAME_TABLE

LPBYTE hswr_dialog_nat_to_bin(LPBYTE);

typedef struct {
        char *owner;
        char *company;
        char *keyword;
	char *pgpkey;
} REGISTRATION;

typedef struct {
	HMENU   (*lpfmenu)(HINSTANCE , LPCSTR ,int);
	HICON   (*lpficon)(HINSTANCE , LPCSTR ,int);
	int     (*lpfstring)(HINSTANCE , UINT, LPSTR, int, int );
	HCURSOR (*lpfcursor)(HINSTANCE , LPCSTR ,int);
	HBITMAP (*lpfbitmap)(HINSTANCE , LPCSTR ,int);
	HACCEL  (*lpfaccel)(HINSTANCE , LPCSTR ,int);
	HRSRC   (*lpffind)(HINSTANCE ,LPCSTR ,LPCSTR, int);
	HGLOBAL (*lpfload)(HINSTANCE , HRSRC, int);
	DWORD   (*lpfsizeof)(HINSTANCE , HRSRC, int);
	HWND    (*lpfcreatedlg)(HINSTANCE ,LPCSTR ,HWND ,DLGPROC ,LPARAM ,int);
	int     (*lpfdialogbox)(HINSTANCE ,LPCSTR ,HWND ,DLGPROC ,LPARAM ,int);
	FARPROC (*lpfgetprocaddr)(HINSTANCE,LPCSTR,int);
} TWINRT;
	
#endif
#endif /* Resources__h */

/*------------------------- < Full Revision History > ----------------------
** $Log: Resources.h,v $
** Revision 1.3  2000/01/26 19:55:28  jwhite
** This includes a wide range of changes required to get Twine to work
** with MFC on the Alpha platform.
**
** Revision 1.2  1999/12/29 04:18:41  jwhite
** Revised to support reading of DIALOGEX binary file.
** Added code for DrawTextW and CreateEventW.
**
** Revision 1.1  1999/07/15 17:56:55  jwhite
** Moved files that are not clearly a part of the Win32 API specification
** out of the main include directory into a 'twine' subdirectory.
** This is going to allow us to more readily integrate Wine include files.
**
** Revision 1.2  1999/07/12 21:13:43  mwalsh
** Updated comments to utilize standard header and footer
**
**-------------------------------------------------------------------------*/
