
/*--------------------------------------------------------------------------
**      LocalMem.c                   Twine           
**              
**------------------------- < License Information > ------------------------
**
**  
**      This file is released under terms of an LGPL.  Readers of 
**  this file may modify or use this source code so long as they make all
**  modifications publically available.  Please refer to TWINE-LICENSE 
**  for further details.
**  
**
**      The original Copyright is in 1999, by CodeWeavers, Inc.
**
**      
**------------------------ < File Content Description > --------------------
**
**  Module:	memory/LocalMem.c
**	
**  Description:
**		provides implementation of Local Memory Functions
**
**  Functions defined:
**
**		LocalAlloc, LocalRealloc, LocalFree
**		LocalLock, LocalUnlock
**		LocalSize, LocalHandle
**
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/memory/LocalMem.c,v 1.11 1999/12/08 19:23:18 robf Exp $
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/

#include "windows.h"
#include "Log.h"

void
LocalInit(int bExit)
{
	return;
}

HLOCAL
LocalAlloc(UINT flags,UINT bytes)
{
	HANDLE hMem;
	HLOCAL  dwWord;
	APISTR((LF_APICALL,"LocalAlloc(%x,%x)\n", flags,bytes));
	
	hMem = GlobalAlloc(GHND,bytes);
	if(hMem && !(flags  & LMEM_MOVEABLE)) {
		dwWord = (HLOCAL) GlobalLock(hMem);
		if(flags & LMEM_ZEROINIT)
			memset((void *)dwWord,0,bytes);
		APISTR((LF_APIRET,"LocalAlloc returns DWORD (ptr) %x\n", 
			dwWord));
		return dwWord;
	}

	APISTR((LF_APIRET,"LocalAlloc returns DWORD (handle) %x\n", hMem));
	return (HLOCAL) hMem;
}

HLOCAL
LocalReAlloc(HLOCAL hLocal,UINT flags,UINT bytes)
{
	HANDLE hMem = (HANDLE) hLocal;
	DWORD  dwWord;
	
	APISTR((LF_APICALL,"LocalReAlloc(%x,%x,%x)\n", hLocal,flags,bytes));
	if(HIWORD(hLocal))
		hMem = GlobalHandle32((LPCVOID)hLocal);
		
	hMem =  GlobalReAlloc(hMem,flags,bytes);
	if(HIWORD(hLocal)) {
		dwWord = (DWORD) GlobalLock(hMem);
		APISTR((LF_APIRET,"LocalReAlloc returns DWORD (ptr) %x\n", 
			dwWord));
		return dwWord;
	}	
	APISTR((LF_APIRET,"LocalReAlloc returns DWORD (handle) %x\n", hMem));
	return (HLOCAL) hMem;
}

HLOCAL
LocalFree(HLOCAL hLocal)
{
	HANDLE hMem = hLocal;
	HGLOBAL hFree;
	APISTR((LF_APICALL,"LocalFree32(%x)\n", hMem));
	if(HIWORD(hLocal))
		hMem = GlobalHandle32((LPCVOID)hLocal);
		
	hFree =  GlobalFree(hMem);
	APISTR((LF_APIRET,"LocalFree32 returns HLOCAL\n", hFree));
	return (HLOCAL) hFree;
}

LPVOID
LocalLock(HLOCAL hLocal)
{
	HANDLE hMem = hLocal;
	LPVOID  lpVoid;
	
	APISTR((LF_APICALL,"LocalLock(%x)\n", hLocal));
	if(HIWORD(hLocal)) {
		APISTR((LF_APIRET,"LocalLock returns DWORD (ptr) %p\n", 
			(void *)hLocal));
		return (LPVOID) hLocal;
	}
	lpVoid = (LPVOID) GlobalLock(hMem);
	APISTR((LF_APIRET,"LocalLock returns DWORD (ptr) %p\n", lpVoid));
	return lpVoid;
}

HLOCAL 
LocalHandle(
    LPCVOID pMem 	// address of local memory object  
   )
{
	return GlobalHandle32(pMem);
}
 
BOOL LocalUnlock(
    HLOCAL hLocal 	// handle of local memory object 
   )
{
	HANDLE hMem = hLocal;
	BOOL bret;	
	APISTR((LF_APICALL,"LocalUnlock(%x)\n", hMem));
	if(HIWORD(hLocal))
		hMem = GlobalHandle32((LPCVOID)hLocal);
		
	bret = GlobalUnlock(hMem);
	APISTR((LF_APIRET,"LocalUnlock returns BOOL %d\n",bret));
	return bret;	
}

UINT LocalSize(
    HLOCAL  hLocal 	// handle of local memory object 
   )
{
	if(HIWORD(hLocal))
		hLocal = GlobalHandle32((LPCVOID)hLocal);
	return GlobalSize(hLocal);
}
 
/*------------------------- < Full Revision History > ----------------------
** $Log: LocalMem.c,v $
** Revision 1.11  1999/12/08 19:23:18  robf
** zeroinit if we get a LPTR type and flag set
**
** Revision 1.10  1999/11/30 04:11:01  robf
** localalloc can lock the handle as well
**
** Revision 1.8  1999/07/13 20:17:08  mwalsh
** Updated comments to use standard header and footer.
**
**-------------------------------------------------------------------------*/
