/*--------------------------------------------------------------------------
**      memory.c                   Twine           
**              
**------------------------- < License Information > ------------------------
**
**  
**      This file is released under terms of an LGPL.  Readers of 
**  this file may modify or use this source code so long as they make all
**  modifications publically available.  Please refer to TWINE-LICENSE 
**  for further details.
**  
**
**      The original Copyright is in 1999, by CodeWeavers, Inc.
**
**      
**------------------------ < File Content Description > --------------------
**
**  Module:	memory/memory.c
**	
**  Description:
**		
**
**  Functions defined:
**
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/memory/memory.c,v 1.9 2000/01/28 07:56:37 robf Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#include "windows.h"
#include "WinDefs.h"
#include "kerndef.h"
#include "Resources.h"
#include "Module.h"
#include "Kernel.h"
#include "Driver.h"
#include "Log.h"
#include "WinMalloc.h"
#include "Strings32.h"
#include "virtual.h"
#include "mtlock.h"

extern VMINFO TWINVM;

static char *savedstrings[512];

LPSTR
WinSaveString(LPCSTR lpstr)
{
	int i;
	char *lp;

	if(lpstr == (LPCSTR) 0xffffffff) {
		for(i=0;i<512;i++) {
		   if(savedstrings[i]) {
		     WinFree(savedstrings[i]);
		   }
		}
		return 0;
	}
	for(i=0;i<512;i++) {
		if (savedstrings[i] == 0) {
		   break;
		}

		if (strcmp(savedstrings[i],lpstr) == 0) {
		   return savedstrings[i];
		}
	}
	if(i == 512) {
		lp = WinStrdup(lpstr);
		return lp;
	}
	savedstrings[i] = WinMalloc(strlen(lpstr) + 1);
	strcpy(savedstrings[i],lpstr);
	return savedstrings[i];
}

LPSTR
WinStrdup(LPCSTR szStringIn)
{
	char * szStringOut;
	short int Length;
	static int dwtotal;
	static int dwcount;

	if(szStringIn == (LPCSTR) 0xffffffff) {
		logstr(LF_MEM,"total allocated by WinStrdup: %dBytes %d calls\n",
			dwtotal,dwcount);
		return 0;
	}

	/* get length of string */
	Length = strlen( szStringIn );
	Length++;
	dwtotal += Length;
	dwcount++;

	/* alloc space to hold dup */
	szStringOut = (char *) WINMALLOC( Length,1,0 );

	/* copy string */
	strcpy( szStringOut, szStringIn );

	/* return pointer to dup string */
	return ( szStringOut );
}

void WINAPI
RtlFillMemory(LPVOID ptr,UINT len, UINT fill)
{
	memset(ptr,fill,len);
}

void WINAPI
RtlMoveMemory(LPVOID ptr,LPVOID src,UINT len)
{
	memmove(ptr,src,len);
}

void WINAPI
RtlZeroMemory(LPVOID ptr,UINT len)
{
	memset(ptr,0,len);
}

void 
CopyMemory(PVOID t,PVOID s,DWORD l)
{
	memcpy(t,s,l);
}

void 
ZeroMemory(PVOID p,DWORD l)
{
	memset(p,0,l);
}

LPVOID WinCalloc( unsigned int units, unsigned int size )
{
	LPVOID *ptr;
	ptr = WinMalloc(units * size);
	if (*ptr)
		memset(ptr, 0, size);
	return ptr;
}

typedef struct {
	LPVOID 	(*malloc)(size_t);
	void    (*free)(LPVOID);
	LPVOID  (*realloc)(LPVOID,size_t);
} MEMALLOC;

static	MEMALLOC basic = {
	(LPVOID) malloc,
	free,
	realloc
};

MEMALLOC *TWIN_Memory = &basic;

LPVOID
xmalloc(unsigned int size)
{
	return WinMalloc(size);
}
void
xfree(LPVOID lpvoid)
{
	WinFree(lpvoid);
}
LPVOID
xrealloc(LPVOID ptr, unsigned int size)
{
	return WinRealloc(ptr,size);
}

static  long winmalloclock;

#ifndef WINMALLOC_CHECK

LPVOID
WinMalloc(unsigned int size)
{
    size_t dwSize = ( (size_t) size+3 ) & ~( (size_t) 3);
    LPVOID lpCore;

    dwSize += 32;
    acquire(&winmalloclock); 
    lpCore = TWIN_Memory->malloc(dwSize);  
    release(&winmalloclock);

    if (TWINVM.vm_map)
	logstr(LF_API,"WinMalloc(%ld) %p-%p\n",
		size,lpCore,(ULONG)lpCore + dwSize);


    return lpCore;
}

void 
WinFree(LPVOID ptr)
{
    LPVOID lpCore = ptr;
    acquire(&winmalloclock);
    TWIN_Memory->free(lpCore);
    release(&winmalloclock);
    if (TWINVM.vm_map)
	logstr(LF_API,"WinFree(%p)\n",ptr);
}

LPVOID
WinRealloc(LPVOID ptr, unsigned int size)
{
   LPVOID lpCore;
   DWORD dwSize = (size+3) & ~3;

   if (!ptr) {
      acquire(&winmalloclock);
      lpCore = TWIN_Memory->malloc(size);
      release(&winmalloclock);
      return lpCore;
   }

    acquire(&winmalloclock);
    lpCore = TWIN_Memory->realloc(ptr,dwSize);

    release(&winmalloclock);

    if (TWINVM.vm_map)
	logstr(LF_API,"WinRealloc(%p,%ld) %d %p\n",
		ptr,size,dwSize,lpCore);
    return lpCore;
}
#endif

/*****************************************************************************/

#ifdef WINMALLOC_CHECK

typedef struct _malloc_tag {
	struct _malloc_tag *next;
	char	*data;
	int     size;
	int	call;
	char 	*lparam;
	int	 wparam;
	int	 flag;
	int 	 handle;
} MALLOCINFO;

MALLOCINFO	*lpMallocInfo;

static int	totalalloc;
static int	totalfree;
static int	totalacalls;
static int	totalfcalls;

LPVOID
WinMallocCheck(unsigned int size, char *lparam,int wparam,int flag, int handle)
{
	LPVOID   lpMemory;
	MALLOCINFO *p;

        acquire(&winmalloclock);
        lpMemory =  TWIN_Memory->malloc(size);
        release(&winmalloclock);

        logstr(LF_MEM,"%s:%d: WinMallocCheck data=%p size=%d %x %x\n",
		lparam,wparam,
                lpMemory,size,
		flag,handle);

        acquire(&winmalloclock);
	p = (MALLOCINFO *) TWIN_Memory->malloc(sizeof(MALLOCINFO));
        release(&winmalloclock);

	memset(p,0,sizeof(MALLOCINFO));

	p->next = lpMallocInfo;
	p->data = lpMemory;
	p->size = size;

	p->lparam = lparam;
	p->wparam = wparam;
	p->flag   = flag;
	p->handle = handle; 
	p->call   = totalacalls;
	
	lpMallocInfo = p;
	totalacalls++;

	totalalloc += size;

        return lpMemory;
}

void
WinFreeCheck(LPVOID ptr,char *lparam, int wparam)
{
	MALLOCINFO *p,*l;

	for(p=lpMallocInfo; p; p = p->next) {
		if(p->data == ptr) {
			if(lpMallocInfo == p) {
				lpMallocInfo = p->next;
			} else {
				for(l=lpMallocInfo; l; l = l->next) {
					if(l->next == p) {
						l->next = p->next;
					}
				}
			}
        		logstr(LF_MEM,"%s:%d: WinFreeCheck: data=%x from=%s:%d %x %x\n",
			    lparam,wparam,
                	    ptr,
			    p->lparam,p->wparam,
			    p->flag,p->handle);

			totalfree += p->size;
			totalfcalls++;

        		acquire(&winmalloclock);
			TWIN_Memory->free(p);
    			release(&winmalloclock);
			break;
		}
	}
	if(p == 0) {
		logstr(LF_MEM,"*** ERROR *** WinFreeCheck: freeing %x %s:%d\n",
			ptr, lparam,wparam);
	}
        acquire(&winmalloclock);
	TWIN_Memory->free(ptr);
        release(&winmalloclock);
}

LPVOID
WinReallocCheck(LPVOID ptr, unsigned int size, char *lparam, int wparam,int flag,int handle)
{
	LPVOID	lp;
	MALLOCINFO *p;

	for(p=lpMallocInfo; p; p = p->next) {
		if(p->data == ptr) {
			totalalloc -= p->size;
			break;
		}
	}

        	acquire(&winmalloclock);
		lp = TWIN_Memory->realloc(ptr,size);
        	release(&winmalloclock);

        logstr(LF_MEM,"%s:%d: WinReallocCheck: old data=%x new data=%x size=%x %x %x\n",
		lparam,wparam,
                ptr,lp,size,
		flag,handle);

	if(ptr == 0)
		return 0;

	if(lp) {
		p->size  = size;
	} else {
		p->size  = 0;
	}

	p->data  = lp;
	p->lparam = lparam;
	p->wparam = wparam;

	totalalloc += size;	
	return lp;
}

long
WinMallocInfo(int opcode, char *ptr,int flag, int handle)
{
	MALLOCINFO *p;
	int	total,n;

	if(lpMallocInfo == 0)
		return 0;

	switch(opcode) {
	case WMI_DUMP:
		logstr(LF_MEM,"WinMallocInfo Malloc Chain\n");

		total = 0;
		n     = 0;
		for(p=lpMallocInfo; p; p = p->next) {
			if (p->flag == 7)
			   continue;
	logstr(LF_MEM,"%d: %s:%d: %p #%d size=%d type=%x handle=%x\n",
				n++,
				p->lparam?p->lparam:"(void)",p->wparam,
				p->data,
				p->call,p->size,p->flag,p->handle);
			total += p->size;
			
		}

		logstr(LF_MEM,"WinMallocInfo Statistics\n");
		logstr(LF_MEM,"   calls alloc = %d size=%d\n",
			totalacalls,totalalloc);
		logstr(LF_MEM,"   calls free = %d size=%d\n",
			totalfcalls,totalfree);
		logstr(LF_MEM,"   remaining = %d delta=%d\n",
			total, totalalloc-totalfree);
		break;

	case WMI_TAG:
		for(p=lpMallocInfo; p; p = p->next) {
			if(p->data == ptr) {
				p->flag   = flag;	
				p->handle = handle;
				break;
			}
		}
		return 0;

	case WMI_CHECK:
		for(p=lpMallocInfo; p; p = p->next) {
			if(p->handle == handle) {
				logstr(LF_MEM,"WinMallocInfo: %s:%d %p %d %d\n",
					p->lparam, p->wparam,p->data,
					p->flag, p->handle);
				return 1;
			}
		}
		return 0;
		
	default:
		break;
	}
	return 0;
}


void TWIN_FreeAllMemory()	/* free all remaining memory as we exit windows .. */
{
	MALLOCINFO *p, *q;
	for(p=lpMallocInfo; p; p = q) {
		q	=	p->next;
        	acquire(&winmalloclock);
		TWIN_Memory->free(p);
        	release(&winmalloclock);
	}
}

#endif

/*------------------------- < Full Revision History > ----------------------
** $Log: memory.c,v $
** Revision 1.9  2000/01/28 07:56:37  robf
** RtlSetMemory wrong
**
** Revision 1.8  2000/01/26 19:55:30  jwhite
** This includes a wide range of changes required to get Twine to work
** with MFC on the Alpha platform.
**
** Revision 1.7  2000/01/21 20:44:30  robf
** added rtl memory routines
**
** Revision 1.6  1999/07/13 20:17:09  mwalsh
** Updated comments to use standard header and footer.
**
**-------------------------------------------------------------------------*/
