/*--------------------------------------------------------------------------
**      dos_timer.c                 Twin           From:  Twin/msdos
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**    Copyright 1997 Willows Software, Inc.  
**------------------------ < File Content Description > --------------------
**
**  Module:	msdos/dos_timer.c
**
**  Description:
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/msdos/dos_timer.c,v 1.2 1999/07/08 21:30:38 mwalsh Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#include "platform.h"

#define		_POSIX_SOURCE	1
#include	<time.h>


#define	dysize(y)	\
	(((y) % 4) ? 365 : (((y) % 100) ? 366 : (((y) % 400) ? 365 : 366)))

static int  dmsize[] = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };


/* Perform converse of gmtime */
static long
time_gm(
	register int sec,		/* 0 - 59 */
	register int min,		/* 0 - 59 */
	register int hour,		/* 0 - 23 */
	register int mday,		/* 1 - 31 */
	register int mon,		/* 0 - 11 */
	register int year		/* year - 1900 */
       )
{
	register long   result;
	int		i;

	if (sec < 0 || sec > 59
	    || min < 0 || min > 59
	    || hour < 0 || hour > 23
	    || mday < 1 || mday > 31
	    || mon < 0 || mon > 11)
		return -1L;

	result = 0L;
	year += 1900;
	for (i = 1970; i < year; i++)
		result += dysize(i);
	if (dysize(year) == 366 && mon >= 3)
		result++;
	while (mon--)
		result += dmsize[mon];
	result += mday - 1;
	result = 24 * result + hour;
	result = 60 * result + min;
	result = 60 * result + sec;

	return result;
}


/* Performs converse of localtime(3) */
static long
time_local(
	   register int sec,		/* 0 - 59 */
	   register int min,		/* 0 - 59 */
	   register int hour,		/* 0 - 23 */
	   register int mday,		/* 1 - 31 */
	   register int mon,		/* 0 - 11 */
	   register int year		/* year - 1900 */
          )
{
	long		result = 0;
	struct tm	*tm, *localtime();
	long		zonefix;

	tm = localtime(&result);	/* Epoch (1/1/70) adjusted for GMT */

	result = time_gm(sec, min, hour, mday, mon, year);
	zonefix = (((long)(tm->tm_hour * 60) + tm->tm_min) * 60) + tm->tm_sec;
	if (tm->tm_year < 70) {		/* In Western hemisphere? */
		result += (24 * 60 * 60) - zonefix;
	}
	else {
		result -= zonefix;
	}
	tm = localtime(&result);	/* Break down current time ... */
	if (tm->tm_isdst)		/* To check if it is during DST */
		result -= 60 * 60;
	return result;
}


unsigned long
seconds( unsigned int access_date, unsigned int access_time )
{
	unsigned int	year;
	unsigned int	month;
	unsigned int	day;
	unsigned int	hour;
	unsigned int	minute;
	unsigned int	second;

	/* unformat dos date and time */

	year   = ((access_date >> 9) & 0x7f) + 1980;
	month  = ((access_date >> 5) & 0x0f) - 1;
	day    =   access_date & 0x1f;
	hour   =  (access_time >> 11) & 0x1f;
	minute =  (access_time >> 5) & 0x3f;
	second =   access_time & 0x1f;

	/* compute number of seconds since jan 1, 1970 */

	return ( time_local(second, minute, hour, day, month, year - 1900) );
}

/*------------------------- < Full Revision History > ----------------------
** $Log: dos_timer.c,v $
** Revision 1.2  1999/07/08 21:30:38  mwalsh
** Updated to new standard comment header and footer
**
**-------------------------------------------------------------------------*/