/*--------------------------------------------------------------------------
**      fn_36.c                 Twin           From:  Twin/msdos
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**    Copyright 1997 Willows Software, Inc.  
**------------------------ < File Content Description > --------------------
**
**  Module:	msdos/fn_36.c
**
**  Description:Return disk free space (int 21 Fn 36h)
**              get free disk space...
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/msdos/fn_36.c,v 1.8 1999/12/12 20:06:34 jwhite Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/



/**	(needs to be moved into mfs code, to handle floppy driver...
**	can be put into mfs_disk.c for generic handling...
*/

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/
#include "config.h"
#include <sys/types.h>
#if defined(HAVE_SYS_PARAM_H)
#include <sys/param.h>
#endif
#if defined(HAVE_SYS_STATVFS_H)
#include <sys/statvfs.h>
#elif defined(HAVE_STATFS_H)
#ifdef STATFS_DEFINED_BY_SYS_MOUNT
    #include <sys/mount.h>
#else
    #ifdef STATFS_DEFINED_BY_SYS_VFS
        #include <sys/vfs.h>
    #else
        #ifdef STATFS_DEFINED_BY_SYS_STATFS
            #include <sys/statfs.h>
        #endif
    #endif
#endif

#else
/* #include <sys/vfs.h> */
#endif


/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#include "windows.h"
#include "Log.h"
#include "fn_36.h"
#include "platform.h"







extern int get_path(int, char *, int);

#define		DOSMAXVALUE	0x7fff
#define		DOSMINBLKSIZE	1024
#define		DOSCLUSTER	(8 * 512)

unsigned int
disk_free(
		int		drive,
		unsigned int	*clusters,	/* Number of free clusters */
		unsigned int	*driveclusters,	/* Number of total lcusters on disk */
		unsigned int	*sectorsize,	/* Bytes per sector */
		unsigned int	*sectorsper	/* Sectors per cluster */
	 )
{
	register unsigned int	blkpcluster;
	register unsigned long	nclusters;
	unsigned int		bsize;
	unsigned long		nfree;
	unsigned long		ntotal;
	char 			file[256];

	/* this gets us the mapped drive... */
	if(get_path(drive,file,256)) 
	{
		return -1;
	}

#if defined(NETWARE)
	{
		char server[49];
		char volume[17];
		char dirs[256];
		int volumeNumber;
		VOLUME_INFO vi;
		GetAvailableUserDiskSpace(file, (LONG*)clusters);
		ParsePath(file, server, volume, dirs);
		GetVolumeNumber(volume, &volumeNumber);
		GetVolumeStatistics(GetDefaultFileServerID(),
			volumeNumber,
			sizeof(VOLUME_INFO),
			&vi);
		*sectorsize = 512;
		*sectorsper = vi.sectorsPerBlock;
		*driveclusters = vi.totalBlocks;
	}
#else
	{
	#if defined(HAVE_VFS_H)
		struct statvfs sbuf;
	#else
		struct statfs sbuf;
	#endif

	#if defined(HAVE_VFS_H)
		if (statvfs(file, &sbuf) < 0)
			return -1;
        #elif defined(STATFS_HAS_FOUR_PARMS)
		if(statfs(file, &sbuf, sizeof(struct statfs), 0) < 0)
			return -1;
	#else
		if(statfs(file, &sbuf) < 0 )
			return -1;
	#endif

	bsize = (unsigned long)sbuf.f_bsize;
	nfree =  (unsigned long)sbuf.f_bfree;
	ntotal = (unsigned long)sbuf.f_blocks;

	if (((blkpcluster = (DOSCLUSTER / bsize)) * bsize) != DOSCLUSTER)
		blkpcluster = 1;		/* If not a multiple of 512 */

	nclusters = ntotal / blkpcluster;

	while ((nclusters > DOSMAXVALUE)
		&& ((blkpcluster * bsize) < (DOSMAXVALUE / 2))) 
	{
		blkpcluster *= 2;
		nclusters /= 2;
	}

	*driveclusters = (nclusters > DOSMAXVALUE) ? DOSMAXVALUE : nclusters;

	nclusters = nfree / blkpcluster;

	*clusters = (nclusters > DOSMAXVALUE) ? DOSMAXVALUE : nclusters;
	*sectorsize = bsize;
	*sectorsper = blkpcluster;
	}
#endif	/* NETWARE */

	return 0;

}

/*------------------------- < Full Revision History > ----------------------
** $Log: fn_36.c,v $
** Revision 1.8  1999/12/12 20:06:34  jwhite
** Include config.h prior to using #defines from it.
**
** Revision 1.7  1999/07/08 21:30:38  mwalsh
** Updated to new standard comment header and footer
**
**-------------------------------------------------------------------------*/
