/*--------------------------------------------------------------------------
**      MultiMedia.c                 Twin           From:  Twin/multimedia
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      Copyright 1997 Willows Software, Inc.
**------------------------ < File Content Description > --------------------
**
**  Module:	multimedia/MultiMedia.c
**
**  Description:
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/multimedia/MultiMedia.c,v 1.2 1999/07/13 20:17:11 mwalsh Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/
#include <string.h>

/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#include "windows.h"
#include "Log.h"

UINT WINAPI
timeBeginPeriod(UINT uPeriod)
{
    APISTR((LF_APISTUB,"timeBeginPeriod(UINT=%x)\n",uPeriod));
    return 0;
}


UINT WINAPI
timeEndPeriod(UINT uPeriod)
{
    APISTR((LF_APISTUB,"timeEndPeriod(UINT=%x)\n",uPeriod));
    return 0;
}

UINT WINAPI
timeGetDevCaps(LPTIMECAPS lpTimeCaps, UINT uSize)
{
    APISTR((LF_APISTUB,"timeGetDevCaps(LPTIMECAPS=%p,UINT=%d)\n",uSize));
    lpTimeCaps->wPeriodMin = 1000;	/* milliseconds */
    lpTimeCaps->wPeriodMax = 65535;	/* milliseconds */
    return 0;
}

UINT WINAPI
timeGetSystemTime(LPMMTIME lpTime, UINT uSize)
{
    APISTR((LF_APISTUB,"timeGetSystemTime(LPMMTIME=%p,UINT=%x)\n",uSize));
    lpTime->wType = TIME_MS;
    lpTime->u.ms = GetCurrentTime();
    return 0;
}

DWORD WINAPI
timeGetTime()
{
    DWORD	ctime;
    APISTR((LF_APICALL,"timeGetTime()\n"));
    ctime =  GetCurrentTime();
    APISTR((LF_APIRET,"timeGetTime: returns DWORD %lx\n",ctime));
    return ctime;
}

typedef struct tagMMTIMER
{
	LPTIMECALLBACK	lpCallBack;
	DWORD		dwUser;
	UINT		uFlags;
} MMTIMER;
static MMTIMER mmTimers[16];

static void CALLBACK
mmTimerCallback(HWND hWnd, UINT msg, UINT idTimer, DWORD dwTime)
{
    if (mmTimers[idTimer-1].lpCallBack == 0)
	return;
    mmTimers[idTimer-1].lpCallBack(idTimer,msg,mmTimers[idTimer-1].dwUser,
					0L,0L);
    if (mmTimers[idTimer-1].uFlags == TIME_ONESHOT)
	timeKillEvent(idTimer);
}

UINT WINAPI
timeSetEvent(UINT uDelay, UINT uResolution, LPTIMECALLBACK lpFunction,
		DWORD dwUser, UINT uFlags)
{
    UINT uIDTimer;

    APISTR((LF_APICALL,
	"timeSetEvent(UINT=%x,UINT=%x,LPPROC=%p,DWORD=%x,UINT=%x\n",
	uDelay,uResolution,lpFunction,dwUser,uFlags));

    uIDTimer = SetTimer((HWND)0,0,uDelay,(TIMERPROC)mmTimerCallback);

    if (uIDTimer == 0) {
    	APISTR((LF_APIFAIL,"timeSetEvent: returns UINT %x\n",0));
	return 0;
    }

    mmTimers[uIDTimer-1].lpCallBack = lpFunction;
    mmTimers[uIDTimer-1].dwUser = dwUser;
    mmTimers[uIDTimer-1].uFlags = uFlags; 

    APISTR((LF_APIRET,"timeSetEvent: returns UINT %x\n",uIDTimer));
    return uIDTimer;
}

UINT WINAPI
timeKillEvent(UINT uTimerID)
{
    APISTR((LF_APICALL,"timeKillEvent(UINT=%x)\n",uTimerID));

    if (uTimerID == 0 || uTimerID >= 16) {
    	APISTR((LF_APIFAIL,"timeKillEvent: returns UINT %x\n",TIMERR_NOCANDO));
	return TIMERR_NOCANDO;
    }

    KillTimer((HWND)0,uTimerID);
    memset((LPSTR)&mmTimers[uTimerID-1],'\0',sizeof(MMTIMER));
    APISTR((LF_APIRET,"timeKillEvent: returns UINT %x\n",0));
    return 0;
}


BOOL WINAPI
sndPlaySound(LPCSTR lpSoundName, UINT Options)
{
    APISTR((LF_APISTUB,"sndPlaySound(LPCSTR=%s,UINT=%x)\n",
	lpSoundName?lpSoundName:"NULL", Options));
    return TRUE;
}

/*------------------------- < Full Revision History > ----------------------
** $Log: MultiMedia.c,v $
** Revision 1.2  1999/07/13 20:17:11  mwalsh
** Updated comments to use standard header and footer.
**
**-------------------------------------------------------------------------*/