/*--------------------------------------------------------------------------
**      wndpos.c                 Twin           From:  Twin/programs/twinview/twinhelp
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      Copyright 1997 Willows Software, Inc.
**------------------------ < File Content Description > --------------------
**
**  Module:	programs/twinview/twinhelp/wndpos.c
**
**  Description:Window Position Functions
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/programs/twinview/twinhelp/wndpos.c,v 1.2 1999/07/13 16:39:45 mwalsh Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#include "wndpos.h"



/*************************************

*

*  Private Defines

*

**************************************/





/*************************************

*

*  Function Prototypes

*

**************************************/



static void __far __pascal GetWindowSize( HWND hWnd, RECT __far * RectPtr );





/*************************************

*

*  Get a window's position and

*  convert it into the virtual

*  coordinate space we use.

*

**************************************/

void __far __pascal MakeWndVirtualPos( HWND hWnd, WINDOWPLACEMENT __far * WndPlacePtr )

{

  HDC hDC;

  short int OldMappingMode;

  DWORD     OldWindowExt;

  DWORD     OldViewportExt;

  

  

  /* Get the position of the window. */  

  WndPlacePtr->length = sizeof(WINDOWPLACEMENT);

  GetWindowPlacement( hWnd, WndPlacePtr );

  

  if( WndPlacePtr->showCmd != SW_SHOWMAXIMIZED )

  {  

    /* Get DC for desktop. */

    hDC = GetDC( hWnd );

      

    /* Set the window's mapping mode. */

    OldMappingMode = SetMapMode( hDC, MM_ANISOTROPIC );

    OldWindowExt = SetWindowExt( hDC, 1024, 1024 );

    OldViewportExt = SetViewportExt( hDC, GetDeviceCaps( hDC, HORZRES ),

                                     GetDeviceCaps( hDC, VERTRES ) );

      

    /* Convert to virtual coordinates. */

    DPtoLP( hDC, (POINT __far *) &(WndPlacePtr)->rcNormalPosition, 2 );

    

    /* Restore the window's mapping mode. */

    SetMapMode( hDC, OldMappingMode );

    SetWindowExt( hDC, LOWORD(OldWindowExt), HIWORD(OldWindowExt) );

    SetViewportExt( hDC, LOWORD(OldViewportExt), HIWORD(OldViewportExt) );

    

    /* Release the DC. */

    ReleaseDC( hWnd, hDC );

  

    /* Change rect's right and bottom to width and height. */

    WndPlacePtr->rcNormalPosition.right = WndPlacePtr->rcNormalPosition.right -

                                          WndPlacePtr->rcNormalPosition.left;

    WndPlacePtr->rcNormalPosition.bottom = WndPlacePtr->rcNormalPosition.bottom -

                                           WndPlacePtr->rcNormalPosition.top;

  }

}





/*************************************

*

*  Get a window's virtual

*  coordinate position into physcial

*  screen coordinates.

*

**************************************/

void __far __pascal VirtualPosToScrnPos( HWND hWnd, RECT __far * RectPtr )

{

  HDC hDC;

  short int OldMappingMode;

  DWORD     OldWindowExt;

  DWORD     OldViewportExt;

  

  /* Get DC for desktop. */

  hDC = GetDC( hWnd );

  

  /* Set the window's mapping mode. */

  OldMappingMode = SetMapMode( hDC, MM_ANISOTROPIC );

  OldWindowExt = SetWindowExt( hDC, 1024, 1024 );

  OldViewportExt = SetViewportExt( hDC, GetDeviceCaps( hDC, HORZRES ),

                                        GetDeviceCaps( hDC, VERTRES ) );

  

  /* Convert to virtual coordinates. */

  LPtoDP( hDC, (POINT __far *) RectPtr, 2 );



  /* Restore the window's mapping mode. */

  SetMapMode( hDC, OldMappingMode );

  SetWindowExt( hDC, LOWORD(OldWindowExt), HIWORD(OldWindowExt) );

  SetViewportExt( hDC, LOWORD(OldViewportExt), HIWORD(OldViewportExt) );



  /* Release the DC. */

  ReleaseDC( hWnd, hDC );

}



/*------------------------- < Full Revision History > ----------------------
** $Log: wndpos.c,v $
** Revision 1.2  1999/07/13 16:39:45  mwalsh
** Updated comments to standard header and footer
**
**-------------------------------------------------------------------------*/

