/*--------------------------------------------------------------------------
**      windemo.c                 Twin           From:  Twin/samples/windemo
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      Copyright (c) 1995-1996, Willows Software Inc.  All rights reserved.
**------------------------ < File Content Description > --------------------
**
**  Module:	samples/windemo/windemo.c
**
**  Description:Simple Windows Demonstration Application
**	Demonstrates the following:
**             - use of resource and module definition compilers.
**             - registering classes, and creating windows.
**             - standard message loop and message dispatch logic.
**             - use of resources, menues, bitmaps, stringtables and accelerators.
**             - calling the ShellAbout API.
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/samples/windemo/windemo.c,v 1.3 1999/07/14 18:14:28 mwalsh Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#include "windows.h"
#include "windemo.h"

int FAR PASCAL ShellAbout(HWND, LPCSTR, LPCSTR, HICON);

HANDLE	hInstance = (HANDLE)NULL;
char    Program[] = "WinDemo";
char    Title[256] = "";

long FAR PASCAL __export 
WinWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    HICON hIcon;

    switch (message) {

    case WM_COMMAND:
           switch (wParam) {
           case IDM_ABOUT:
               hIcon = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_ICON1));
			   ShellAbout(hWnd, Program, Title, hIcon);
               DestroyIcon(hIcon);
               break;
	    case IDM_EXIT:
	       DestroyWindow(hWnd);
	       break;
           }
	   break;

    case WM_DESTROY:
           PostQuitMessage(0);
           break;

    case WM_LBUTTONDOWN:
	   ShowScrollBar(hWnd, SB_VERT, TRUE);
	   break;

    default:
           return (DefWindowProc(hWnd, message, wParam, lParam));
    }
    return (0);
}


BOOL PASCAL 
WinMain(HANDLE hInst, HANDLE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
    HACCEL hAccelTable = 0;
    HWND   hWnd;
    MSG    Msg;
    
    if (!hPrevInstance) {
        WNDCLASS wc;

        wc.style         = 0;
        wc.lpfnWndProc   = WinWndProc;
        wc.cbClsExtra    = 0;
        wc.cbWndExtra    = 0;
        wc.hInstance     = hInst;
        wc.hIcon         = LoadIcon(hInst, MAKEINTRESOURCE(IDI_ICON1));
        wc.hCursor       = LoadCursor((HINSTANCE)NULL, IDC_ARROW);
        wc.hbrBackground = CreateSolidBrush(RGB(0,0x80,0));
        wc.lpszMenuName  = (LPSTR) "WinMenu";
        wc.lpszClassName = (LPSTR) Program;

        if ( !RegisterClass(&wc) )
            return FALSE;
    }

    hInstance = hInst;

    hAccelTable = LoadAccelerators( hInstance, (LPSTR)"RESOURCE" );

    hWnd = CreateWindow(Program,
			Title,
			WS_OVERLAPPEDWINDOW,
			CW_USEDEFAULT,
			CW_USEDEFAULT,
			640,
			480,
			(HWND)NULL,
			(HMENU)NULL,
			hInst,
			NULL);

    if (!hWnd)
       return FALSE;

    ShowWindow(hWnd, nCmdShow);
    UpdateWindow(hWnd);

    while (GetMessage(&Msg, (HWND)NULL, 0, 0)) {
      if (TranslateAccelerator( hWnd, hAccelTable, (LPMSG)&Msg ) == 0) {
      	TranslateMessage(&Msg);
      	DispatchMessage(&Msg);
      }	
    }
    return Msg.wParam;
}

/*------------------------- < Full Revision History > ----------------------
** $Log: windemo.c,v $
** Revision 1.3  1999/07/14 18:14:28  mwalsh
** Updated comments to standard header and footer
**
**-------------------------------------------------------------------------*/