#--------------------------------------------------------------------------
#      move_wine.sh                   Twine           
#              
#------------------------- < License Information > ------------------------
#
#  
#      This file is released under terms of an LGPL.  Readers of 
#  this file may modify or use this source code so long as they make all
#  modifications publically available.  Please refer to TWINE-LICENSE 
#  for further details.
#  
#
#      The original Copyright is in 1999, by CodeWeavers, Inc.
#
#      
#------------------------ < File Content Description > --------------------
#
#  Module:	templates/move_wine.sh
#	
#  Description:
#     This script will take a wine file from a wine snapshot release
#  and put it into Twine, adding the appropriate wine header.
#
#  Usage:
#      move_wine.sh wine-release filename
#
#  (e.g.   move_wine.sh wine-990523 include/objbase.h)
#
#  Requirements:
#      The wine release has to be right off your home directory.
#      TWINDIR has to be set to point to your Twine distribution.
#    
#------------------------- < Revision Information > -----------------------
# $Header: /twinecvs/src/templates/move_wine.sh,v 1.1 1999/07/15 18:17:16 jwhite Exp $
#
#--------------------------------------------------------------------------


# TODO:  Better comments

if [ ! -f $TWINDIR/Makefile.config ]; then
    echo "Error:  Could not find TWINDIR/Makefile.config"
    exit
fi

wine_release_name=$1
source_file_name=$2
target_dir=$TWINDIR

fname=${source_file_name##*/}
fdir=${source_file_name%/$fname}

if [ ! -f ~/$wine_release_name/$fdir/$fname ]; then
    echo "Error:  Could not find ~/$wine_release_name/$fdir/$fname"
    exit 1
fi

if [ -f $target_dir/$fdir/$fname ]; then
    echo "Error: $target_dir/$fdir/$fname  already exists"
    exit 1
fi


awk -- "{ gsub(\"<dir>\", \"$fdir\"); gsub(\"<fname>\", \"$fname\"); gsub(\"<wine-dir>\", \"$fdir\");  gsub(\"<wine-rel>\", \"$wine_release_name\"); print \$0; }" $TWINDIR/templates/template.wine-begin > $target_dir/$fdir/$fname

if [ ! -f $target_dir/$fdir/$fname ]; then
    echo "Error: $target_dir/$fdir/$fname couldn't be created"
    exit 1
fi

cat ~/$wine_release_name/$fdir/$fname >>$target_dir/$fdir/$fname 
cat $TWINDIR/templates/template.end >>$target_dir/$fdir/$fname 

/*------------------------- < Full Revision History > ----------------------
** $Log: move_wine.sh,v $
** Revision 1.1  1999/07/15 18:17:16  jwhite
** Various templates and a utility script for bringing wine files in.
**
**
**-------------------------------------------------------------------------*/


