/*--------------------------------------------------------------------------
**      mklog.c                   Twine           
**              
**------------------------- < License Information > ------------------------
**
**  
**      This file is released under terms of an LGPL.  Readers of 
**  this file may modify or use this source code so long as they make all
**  modifications publically available.  Please refer to TWINE-LICENSE 
**  for further details.
**  
**
**      The original Copyright is in 1999, by CodeWeavers, Inc.
**
**      
**------------------------ < File Content Description > --------------------
**
**  Module:	tools/mklog.c
**	
**  Description:
**		
**
**  Functions defined:
**
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/tools/mklog.c,v 1.2 1999/07/13 20:17:39 mwalsh Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>

/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/



#define LINKSZ 1024

#define MODULE(x)	n##x

#define nKERNEL32	1
#define nUSER32		2
#define nGDI32		3
#define nSHELL32	4

char *modulenames[] = {
	"",
	"KERNEL32",
	"USER32",
	"GDI32",
	"SHELL32",
	0
};
	
typedef struct link {
	char  *method;		// name
	int   enabled;		// if used
	int   dll;		// which dll
	int   type;		// stdcall/cdecl
	int   nargs;		// number args   
	int  *pargs;		// pointer to args
	char *alias;
	int   ord;
} LINK;

LINK rets[2048];
LINK funcs[2048];
LINK pargs[2048];

int
addmethod(LINK *tbl,char *method, int enabled,int dll,int ord,int type,int args,int *pargs)
{
	int  i;
	
	for(i=0;i<2048;i++) {
		if(tbl[i].method && strcasecmp(method,tbl[i].method) == 0) {
			if(tbl[i].nargs && tbl[i].nargs != args) {
				printf("WARN %s doesn't match %d %d\n",
					method,tbl[i].nargs,args);
			}
			if(pargs)
				tbl[i].pargs = pargs;
			if(args)
				tbl[i].nargs = args;
			return i + 1;
		}
	}

	if(enabled == 0)
		return 0;

	for(i=0;i<2048;i++)
		if(tbl[i].method == 0) break;

	tbl[i].method = strdup(method);
	tbl[i].dll	= dll;
	tbl[i].type	= type;
	tbl[i].nargs	= args;
	tbl[i].pargs	= pargs;
	tbl[i].ord	= ord;
	tbl[i].enabled	= enabled;
	return i + 1;
}

char *
mktoken(char *p)
{
	char *t;

	/* find end of p */
	for(t = p;t && *t;t++) {
		if(*t == ' ') {
			break;
		}
		if(*t == '\t') {
			break;
		}
	}
	/* terminate this one */
	if(t) *t++ = 0;
	
	/* find next one... */
	for(;t && *t;t++) {
		if(*t == ' ')
			continue;
		if(*t == '\t')
			continue;
		break;
	}
	if(t && *t == 0)
		return 0;
	return t;
}

void
parseargs(char *func,char *plist)
{
	char *p;
	char *list,*next;
	int   nargs = 2;
	int   *pargsl,*ptr;
	int   x,n;

	//printf("parse args %s %s\n",func,plist);

	p = strchr(plist,'(');
	if(p == 0)
		return;
	list = p + 1;
	p = strchr(list,')');
	if(p == 0)
		return;
	*p = 0;
	for(p=list;*p;p++)
		if(*p == ',')
			nargs++;

	ptr = pargsl = (int *) malloc(nargs * 4);
	n = 0;
	for(p=list;p && *p;p = next) {
		char *br;
		next = strchr(p,',');
		if(next) *next++ = 0;
		while(*p) {
			if(*p == ' ') {
				p++;
				continue;
			}
			if(*p == '\t') {
				p++;
				continue;
			}
			break;
		}
		br = strchr(p,'*');
		if(br) { 
			br++;
			*br = 0;
		} else {
			br = strchr(p,' ');
			if(br) *br = 0;
		}
		if(strlen(p)) {
			x = addmethod(pargs,p,1,0,0,0,0,0);
			*ptr++ = x-1;
			if(x == 1) nargs--;
			n++;
		}
	}
	*ptr = 0;
	nargs--;

	//printf("%s has %d %d args ( ",func,nargs,n);
	//for(ptr = pargsl;*ptr;ptr++) 
	//	printf(" %d ",*ptr);
	//printf(" )\n");
	addmethod(funcs,func,1,0,0,0,nargs,pargsl);
}

int
main(int argc,char **argv)
{
	char input[512];
	char method[32];
	char *p,*next;
	char *line;
	LINK *lp;
	FILE *fp = 0;
	int  add = 1;
	
	addmethod(rets,"VOID",1,0,0,0,0,0);
	addmethod(pargs,"VOID",1,0,0,0,0,0);
	addmethod(pargs,"VOID *",1,0,0,0,0,0);
	if(argc > 1) {
		fp = fopen(argv[1],"r");
		if(fp == 0)
			printf("failed to open %s\n",argv[1]);
		if(fp)
			add = 0;
	}

	if(fp) {
		int  n;
		char filename[1024];
		while (fgets(filename,1024,fp)) {
			char *spec,*args,*api,*ord,*alias,*prefix;
			int  offset;
			int  type = 0;

			api = strchr(filename,'\n');
			if(api) *api = 0;

			if(strlen(filename) == 0)
				continue;
			if(strncmp(filename,"stub",4) == 0)
				continue;
			if(filename[0] == '#')
				continue;
		
			spec  = filename;
			args  = mktoken(filename);
			api   = mktoken(args);
			ord   = mktoken(api);
			alias   = mktoken(ord);
			
			if(spec && strcmp(spec,"cdecl") == 0)
				type = 1;

			n = atoi(ord);

			funcs[n].method = strdup(api);
			funcs[n].dll	= MODULE(USER32);
			funcs[n].type	= type;
			funcs[n].nargs	= atoi(args);
			funcs[n].pargs	= 0;
			funcs[n].ord	= n;
			funcs[n].enabled = 1;
			funcs[n].alias   = alias?strdup(alias):0;
		}
	}
#if 0
#endif
	while(fgets(input,512,stdin)) {
		line = input;
		p = strchr(line,'\n');
		if(p) *p = 0;
	
		p = strpbrk(line," \t;,");	
		if(p) {
			next = p;
			while(*next) {
				if(*next == ' ') {
					next++;
					continue;
				}
				if(*next == '\t') {
					next++;
					continue;
				}
				break;
			}
			if(next && *next == '*') p = next+1;
			*p++ = 0;
		}
		// next character afterwords
		next = p;
		
		if(strlen(line))
			addmethod(rets,line,1,0,0,0,0,0);
		
		line = next;
		if(line == 0)
			continue;
	
		p = strpbrk(line," \t");	
		if(p) {
			*p++ = 0;			
		}
		next = p;
		parseargs(line,next);
	}

	{
		int i;
		printf("char *RETURN_METHODS[] = {\n");
		for(i = 0;rets[i].method;i++) {
			printf("\t%s\n",rets[i].method);	
		}
		printf("};\n");
	}
	{
		int i;
		printf("char *ARGLIST_METHODS[] = {\n");
		for(i = 0;pargs[i].method;i++) {
			printf("\t%s /* %d */\n",pargs[i].method,i);
		}
		printf("};\n");
	}

#if 0
#endif
	{
		int i;
		printf("FUNCTIONS\n");
		for(i = 0;i<2048;i++) {
			int *ptr;
			if(fp && funcs[i].enabled == 0) {
				//printf("not enabled %d\n",i);
				continue;
			}
			if(fp && funcs[i].method == 0) {
				//printf("no method %d\n",i);
				continue;
			}
			if(funcs[i].ord == 0) {
				//printf("no ordinal %d\n",i);
				continue;
			}


			printf("%s %d %s %d %s",
				funcs[i].type?"cdecl":"stdcall",
				funcs[i].nargs,
				funcs[i].method,
				funcs[i].ord,
				funcs[i].alias?funcs[i].alias:""
				);	
//#if 0
			printf("# ( ");
			if(funcs[i].pargs)
				for(ptr = funcs[i].pargs;*ptr;ptr++) 
					printf(" %d ",*ptr);
			else
				printf("WARN no declaration");

			printf(" )");
//#endif
			printf("\n");
		}
	}
}

/*------------------------- < Full Revision History > ----------------------
** $Log: mklog.c,v $
** Revision 1.2  1999/07/13 20:17:39  mwalsh
** Updated comments to use standard header and footer.
**
**-------------------------------------------------------------------------*/