/*--------------------------------------------------------------------------
**      define.c                 Twin           From:  Twin/tools/rc
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      Copyright 1997 Willows Software, Inc. 
**------------------------ < File Content Description > --------------------
**
**  Module:	tools/rc/define.c
**
**  Description:rc support for #define macros
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/tools/rc/define.c,v 1.2 1999/07/13 20:17:41 mwalsh Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#include "define.h"
#include "rc.h"
#include "buffer.h"

DEFINESTRUCT *dTop = NULL;
DEFINESTRUCT **defineTop = &dTop;

DEFINESTRUCT **internal_find_define(char *key, DEFINESTRUCT **start);

DEFINESTRUCT **internal_find_define(char *key, DEFINESTRUCT **start)
{
  register int i;

  if (!*start)
    return start;

  if (!(i = strcmp((*start)->key, key)))
    return start;

  if (i > 0)
    return internal_find_define(key, &((*start)->less));
  else
    return internal_find_define(key, &((*start)->more));
}

DEFINESTRUCT *create_define(char *key)
{
  DEFINESTRUCT *pNew;
  DEFINESTRUCT **pInsert = internal_find_define(key, defineTop);

  if (*pInsert)
    return *pInsert;

  pNew = (DEFINESTRUCT *)get_mem(sizeof(DEFINESTRUCT));
  pNew->key = str_dup(key);
  pNew->less = NULL;
  pNew->more = NULL;
  pNew->value = NULL;
  *pInsert = pNew;
  return pNew;
}

char *match_define(char *key)
{
  DEFINESTRUCT **pMatch = internal_find_define(key, defineTop);
  if (*pMatch)
    return (*pMatch)->value;
  return NULL;
}

void define_define(char *key, char *value)
{
  DEFINESTRUCT *pDefine = create_define(key);
  if (value)
    pDefine->value = str_dup(value);
  else
    pDefine->value = str_dup("");
}

void undefine_define(char *key)
{
  DEFINESTRUCT **pMatch = internal_find_define(key, defineTop);

  if (!*pMatch)
    return;

  str_free((*pMatch)->value);
  (*pMatch)->value = NULL;
}

void delete_define(DEFINESTRUCT *pDefine)
{
  if (!pDefine)
    return;

  delete_define(pDefine->less);
  delete_define(pDefine->more);
  str_free(pDefine->value);
  str_free(pDefine->key);
  free_mem(pDefine);
}

void clean_defines(void)
{
  delete_define(dTop);
  dTop = NULL;
}

/*------------------------- < Full Revision History > ----------------------
** $Log: define.c,v $
** Revision 1.2  1999/07/13 20:17:41  mwalsh
** Updated comments to use standard header and footer.
**
**-------------------------------------------------------------------------*/