/*--------------------------------------------------------------------------
**      ExecSysV.c                 Twin           From:  Twin/windows
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      Copyright 1997 Willows Software, Inc.
**------------------------ < File Content Description > --------------------
**
**  Module:	windows/ExecSysV.c
**
**  Description:
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/windows/ExecSysV.c,v 1.2 1999/07/09 21:55:40 mwalsh Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/
#include <stdio.h>
#include <signal.h>
#include <siginfo.h>
#include <sys/regset.h>
#include <sys/ucontext.h>
#include <setjmp.h>
#include <errno.h>
#include <memory.h>
#include <sys/sysi86.h>

/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/

#include "Exec.h"

#ifndef UCONTEXT_REGS
#define UCONTEXT_REGS regs
#endif

#define	STACK_DWORDS	0x4000

static struct sigaction sact;
static unsigned long TWIN_Stack[STACK_DWORDS];
static int altstackdefined = 0;

static void
TWIN_GPFSigHandler(int sig, siginfo_t *si, ucontext_t *uc)
{
    struct TWIN_i386_context_s  *sc;
    
    sc = (struct TWIN_i386_context_s *) uc->uc_mcontext.UCONTEXT_REGS;
    TWIN_GPFHandler(sig, sc);
    TWIN_ValidateSegmentRegisters(sc);
}

static void
TWIN_DebugSigHandler (int sig, siginfo_t *si, ucontext_t *uc)
{
    struct TWIN_i386_context_s  *sc;

    sc = (struct TWIN_i386_context_s *) uc->uc_mcontext.UCONTEXT_REGS;

    if (sig == SIGTRAP)
	sc->eip--;
    
    wine_debug(sig, sc);
    TWIN_ValidateSegmentRegisters(sc);
}

int
TWIN_RegisterGPFSignal(int sig)
{
    if (!altstackdefined)
    {
	stack_t ss;
	
	altstackdefined = 1;
	ss.ss_sp = TWIN_Stack;
	ss.ss_size = sizeof(TWIN_Stack);
	ss.ss_flags = 0;
	if (sigaltstack(&ss, NULL) < 0)
	    perror("sigaltstack");
    }
    
    sact.sa_handler = TWIN_GPFSigHandler;
    sact.sa_flags = SA_SIGINFO | SA_ONSTACK;
    return sigaction(sig, &sact, NULL);
}

int
TWIN_RegisterDebugSignal(int sig)
{
    if (!altstackdefined)
    {
	stack_t ss;
	
	altstackdefined = 1;
	ss.ss_sp = TWIN_Stack;
	ss.ss_size = sizeof(TWIN_Stack);
	ss.ss_flags = 0;
	if (sigaltstack(&ss, NULL) < 0)
	    perror("sigaltstack");
    }
    
    sact.sa_handler = TWIN_DebugSigHandler;
    sact.sa_flags = SA_SIGINFO | SA_ONSTACK;
    return sigaction(sig, &sact, NULL);
}

/*------------------------- < Full Revision History > ----------------------
** $Log: ExecSysV.c,v $
** Revision 1.2  1999/07/09 21:55:40  mwalsh
** Updated comments to standard header and footer
**
**-------------------------------------------------------------------------*/
