/*--------------------------------------------------------------------------
**      SysMisc.c                 Twin           From:  Twin/windows
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      Copyright 1997 Willows Software, Inc.
**------------------------ < File Content Description > --------------------
**
**  Module:	windows/SysMisc.c
**
**  Description:
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/windows/SysMisc.c,v 1.2 1999/07/09 21:55:44 mwalsh Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/
#include <string.h>

/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#include "windows.h"
#include "Log.h"
#include "kerndef.h"
#include "Kernel.h"
#include "Driver.h"

/* exported stuff */
BOOL TWIN_GetTimerMsg(HWND, LPMSG, UINT);
int TWIN_GetTimeout(int);

/* internal routines */
static UINT SetTimer2(HWND, UINT, UINT, TIMERPROC, BOOL);
static BOOL KillTimer2(HWND, UINT, BOOL);

UINT WINAPI
GetDoubleClickTime(void)
{
    UINT uiRet;

    APISTR((LF_APICALL,"GetDoubleClickTime()\n"));

    uiRet = (UINT)DRVCALL_EVENTS(PEH_DBLCLKTIME,0,0,0);

    APISTR((LF_APIRET,"GetDoubleClickTime: return UINT %x\n",uiRet));

    return uiRet;
}

void WINAPI
SetDoubleClickTime(UINT uInterval)
{

    APISTR((LF_APICALL,"SetDoubleClickTime(UINT=%x)\n",uInterval));
    (void)DRVCALL_EVENTS(PEH_DBLCLKTIME,1,uInterval,0);
    APISTR((LF_APIRET,"SetDoubleClickTime: returns void\n"));
}

BOOL WINAPI
SwapMouseButton(BOOL fSwap)
{
    BOOL rc;
    LOGSTR((LF_APICALL,"SwapMouseButton(BOOL=%x)\n",fSwap));
    rc =  (BOOL)DRVCALL_EVENTS(PEH_MOUSEBUTTON,fSwap,0,0);
    LOGSTR((LF_APIRET,"SwapMouseButton: returns BOOL %x\n",rc));
    return rc;
}

typedef struct tagSYNCTIMER
{
    DWORD	tid;
    HWND	hWnd;
    UINT	uID;
    UINT	uTimeout;
    UINT	uFlags;
    int		nWaitTime;
    TIMERPROC	lpfnTimerProc;
} SYNCTIMER;
typedef SYNCTIMER *LPSYNCTIMER;

#define	STF_SYSTEM	0x0001

#define	SYNC_TIMERS	48
static SYNCTIMER SyncTimers[SYNC_TIMERS];

UINT WINAPI
SetTimer(HWND hWnd, UINT idTimer, UINT uTimeout, TIMERPROC tmprc)
{
    UINT rc;
    APISTR((LF_APICALL,"SetTimer(HWND=%x,UINT=%x,UINT=%x,TIMERPROC=%x)\n",
	hWnd,idTimer,uTimeout,tmprc));
    rc = SetTimer2(hWnd,idTimer,uTimeout,tmprc,FALSE);
    APISTR((LF_APIRET,"SetTimer: returns UINT %x\n",rc));
    return rc;
}

UINT WINAPI
SetSystemTimer(HWND hWnd, UINT idTimer, UINT uTimeout, TIMERPROC tmprc)
{
    UINT rc;
    APISTR((LF_APICALL,"SetSystemTimer(HWND=%x,UINT=%x,UINT=%x,TIMERPROC=%x)\n",
	hWnd,idTimer,uTimeout,tmprc));
    rc = SetTimer2(hWnd,idTimer,uTimeout,tmprc,TRUE);
    APISTR((LF_APIRET,"SetSystemTimer: returns UINT %x\n",rc));
    return rc;
}

static int nMaxTimer = 0;

static UINT
SetTimer2(HWND hWnd, UINT idTimer, UINT uTime,
		TIMERPROC tmprc, BOOL bSystem)
{
    LPSYNCTIMER lpSyncTimer;
    DWORD tid;
    int n;
    UINT uTimeout = (uTime)?uTime:20;

    if (hWnd != 0 && IsWindow(hWnd))
	tid = GetWindowThreadProcessId(hWnd,0);
    else
	tid = GetCurrentThreadId();

    for (n = 0; n < SYNC_TIMERS; n++)
	if (SyncTimers[n].tid == 0) {
	    lpSyncTimer = &SyncTimers[n];
	    lpSyncTimer->tid = tid;
	    lpSyncTimer->hWnd = hWnd;
	    lpSyncTimer->uID = idTimer;
	    lpSyncTimer->uTimeout = uTimeout;
	    lpSyncTimer->uFlags = (bSystem)?STF_SYSTEM:0;
	    lpSyncTimer->nWaitTime = (int)uTimeout;
	    lpSyncTimer->lpfnTimerProc = tmprc;

	    if (hWnd == 0)
		lpSyncTimer->uID = n + 1;
	    if (n > nMaxTimer)
		nMaxTimer = n;
	    return (lpSyncTimer->uID) ? lpSyncTimer->uID : 1;
	}

    return 0;
}

BOOL WINAPI
KillTimer(HWND hWnd, UINT uID)
{
    BOOL rc;
    APISTR((LF_APICALL,"KillTimer(HWND=%x,UINT=%x)\n",hWnd,uID));
    rc = KillTimer2(hWnd,uID,FALSE);
    APISTR((LF_APIRET,"KillTimer: returns BOOL %x\n",rc));
    return rc;
}

BOOL WINAPI
UserKillSystemTimer(HWND hWnd, UINT uID)
{
    BOOL rc;
    APISTR((LF_APICALL,"UserKillSystemTimer(HWND=%x,UINT=%x)\n",hWnd,uID));
    rc = KillTimer2(hWnd,uID,TRUE);
    APISTR((LF_APIRET,"UserKillSystemTimer: returns BOOL %x\n",rc));
    return rc;
}

static BOOL
KillTimer2(HWND hWnd, UINT uID, BOOL bSystem)
{
    DWORD tid;
    int n;

    if (hWnd != 0 && IsWindow(hWnd))
	tid = GetWindowThreadProcessId(hWnd,0);
    else
	tid = GetCurrentThreadId();

    for (n = 0; n < SYNC_TIMERS; n++)
	if (SyncTimers[n].tid == tid) {
	    if (SyncTimers[n].hWnd == hWnd && SyncTimers[n].uID == uID) {
		SyncTimers[n].tid = 0;
		if (nMaxTimer && n == nMaxTimer) {
		    nMaxTimer--;
		    while (nMaxTimer && SyncTimers[nMaxTimer].tid == 0)
			nMaxTimer--;
		}
		return TRUE;
	    }
	}

    return FALSE;
}

BOOL
TWIN_GetTimerMsg(HWND hWnd, LPMSG lpMsg, UINT uFlg)
{
    static int nOffset = 0;
    int n,nIndex;
    LPSYNCTIMER lpSyncTimer;
    BOOL bSystem;
    DWORD tid;

    if (hWnd != 0 && IsWindow(hWnd))
	tid = GetWindowThreadProcessId(hWnd,0);
    else
	tid = GetCurrentThreadId();

    for (n = 0; n <= nMaxTimer; n++) {
	nIndex = (n+nOffset) % (nMaxTimer+1);
	if (SyncTimers[nIndex].tid == 0)
	    continue;

	lpSyncTimer = &SyncTimers[nIndex];
	if (lpSyncTimer->tid == tid && lpSyncTimer->nWaitTime <= 0) {

	    /* kill the timer of a window, if it is no longer valid */
            if (lpSyncTimer->hWnd && !(IsWindow(lpSyncTimer->hWnd))) {
		lpSyncTimer->tid = 0;
		continue;
	    }

	    if (hWnd == 0 || lpSyncTimer->hWnd == hWnd) {
		bSystem = (lpSyncTimer->uFlags & STF_SYSTEM)?TRUE:FALSE;
		memset((LPSTR)lpMsg,'\0',sizeof(MSG));
		lpMsg->hwnd = lpSyncTimer->hWnd;
		lpMsg->message = (bSystem)?WM_SYSTIMER:WM_TIMER;
		lpMsg->wParam = (WPARAM)lpSyncTimer->uID;
		lpMsg->lParam = (LPARAM)lpSyncTimer->lpfnTimerProc;
		if (uFlg & PM_REMOVE)
		    lpSyncTimer->nWaitTime = (int)lpSyncTimer->uTimeout;
		nOffset = ++nOffset % (nMaxTimer+1);
		return TRUE;
	    }
	}
    }

    QueueClearFlags(tid, QFTIMER);
    nOffset = ++nOffset % (nMaxTimer+1);
    return FALSE;
}

int
TWIN_GetTimeout(int nTimeout)
{
    int n;
    int nMinTimeout = 0;
    LPSYNCTIMER lpSyncTimer;

    for (n = 0; n < SYNC_TIMERS; n++) {
	lpSyncTimer = &SyncTimers[n];
	if (lpSyncTimer->tid == 0)
	    continue;
#ifdef	LATER
	/* sol.exe timers get killed on menu select if we test for 0 here */
	if (lpSyncTimer->nWaitTime == 0)
	    continue;
#endif

	lpSyncTimer->nWaitTime -= nTimeout;

	if (lpSyncTimer->nWaitTime <= 0) {
	    QueueSetFlags(lpSyncTimer->tid,QFTIMER);
	    lpSyncTimer->nWaitTime = 0;
	    nMinTimeout = -1;
	    continue;
	}

	if (nMinTimeout == 0 || lpSyncTimer->nWaitTime < nMinTimeout)
	    nMinTimeout = lpSyncTimer->nWaitTime;

    }

    return nMinTimeout;
}

DWORD WINAPI
GetCurrentTime()
{
    DWORD rc;
    APISTR((LF_APICALL,"GetCurrentTime()\n"));
    rc = DRVCALL_EVENTS(PEH_SYSTIME,0,0,0);
    APISTR((LF_APIRET,"GetCurrentTime: returns DWORD %x\n",rc));
    return rc;
}

DWORD WINAPI
GetTickCount()
{
    DWORD rc;
    APISTR((LF_APICALL,"GetTickCount()\n"));
    rc =  DRVCALL_EVENTS(PEH_SYSTIME,0,0,0);
    APISTR((LF_APIRET,"GetTickCount: returns DWORD %x\n",rc));
    return rc;
}

/*------------------------- < Full Revision History > ----------------------
** $Log: SysMisc.c,v $
** Revision 1.2  1999/07/09 21:55:44  mwalsh
** Updated comments to standard header and footer
**
**-------------------------------------------------------------------------*/
