
/*--------------------------------------------------------------------------
**      lib32.c                   Twine           
**              
**------------------------- < License Information > ------------------------
**
**  
**      This file is released under terms of an LGPL.  Readers of 
**  this file may modify or use this source code so long as they make all
**  modifications publically available.  Please refer to TWINE-LICENSE 
**  for further details.
**  
**
**      The original Copyright is from Willows TWIN.  
**
**      
**------------------------ < File Content Description > --------------------
**
**  Module:	windows/lib32.c
**	
**  Description:
**      This file contains certain functions that are specific
**  to the Win32 API as opposed to the 16 bit windows api.
**		
**
**  Functions defined:
**      FIXME Do this.
**
**  Notes:
**      1.  This function should be revised.  It should not define
**          any xxxA style functions, and the W functions should
**          be #ifdefed on TWINE_WIDEAPI
**                FIXME - Remove xxxA, and add #ifdef TWINE_WIDEAPI
**
**
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/windows/lib32.c,v 1.12 1999/12/24 08:32:23 robf Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/
#include <stdio.h>

/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#include "windows.h"
#include "Log.h"
#include "winnt.h"
#include "heap.h"
#include "RscData.h"
#include "stdarg.h"
#include "Resources.h"
#include "Module.h"
#include "WinConfig.h"
#include "Strings32.h"
#include "Log.h"

LPWSTR lstrcpyAtoW( LPWSTR , LPCSTR );
char *GetTwinMsgCode(HWND, UINT);
static short wcmdln[256];

/* based on current running application */
int
GetStartupInfo(LPSTARTUPINFOA lp)
{
	APISTR((LF_APICALL,"GetStartupInfo(%p) sizeof=%x\n",
		lp,sizeof(STARTUPINFOA)));
	memset(lp,0,sizeof(STARTUPINFOA));
	lp->dwFlags = 1;
	lp->wShowWindow = 1;	
	APISTR((LF_APIRET,"GetStartupInfo returns %d\n",0));
	return 0;
}

BOOL
CharToOemA(LPCSTR instr,LPSTR outstr)
{
	APISTR((LF_API,"CharToOemA(%p,%p)\n",instr,outstr));
	strcpy(outstr,instr);	
	return TRUE;
}
DWORD
GetFileType32(HANDLE hFile)
{
	APISTR((LF_API,"GetFileType32(%x)\n",hFile));
	return 0;	
}

HANDLE
GetStdHandle(DWORD nStdHandle)
{
	APISTR((LF_API,"GetStdHandle(%x)\n",nStdHandle));
	if(nStdHandle == STD_INPUT_HANDLE)
		return INVALID_HANDLE_VALUE;
	if(nStdHandle == STD_OUTPUT_HANDLE)
		return INVALID_HANDLE_VALUE;
	if(nStdHandle == STD_ERROR_HANDLE)
		return INVALID_HANDLE_VALUE;

	return INVALID_HANDLE_VALUE;
}

UINT WINAPI
GetOEMCP(void)
{
	/* MS-DOS United States */
    	return 437;    
}

UINT WINAPI
GetACP(void)
{
	/* return ansi code page 1252 */
	
	APISTR((LF_APICALL,"GetACP()\n"));
	APISTR((LF_APIRET,"GetACP returns %x\n",0x4e4));
	return 0x4e4;  
}

typedef struct {
	UINT   MaxCharSize;
	BYTE   DefaultChar[2];
	BYTE   LeadByte[8];
} CPINFO;

typedef CPINFO *LPCPINFO;

UINT
GetCPInfo(UINT codepage,LPCPINFO lpcpinfo)
{
	APISTR((LF_APICALL,"GetCPInfo(%x,%p)\n",codepage,lpcpinfo));

	lpcpinfo->DefaultChar[0] = '?';
	lpcpinfo->DefaultChar[1] = ' ';

	switch (codepage)
	{
	case 932 : /* Shift JIS (japan) */
		lpcpinfo->MaxCharSize = 2;
		lpcpinfo->LeadByte[0]= 0x81; lpcpinfo->LeadByte[1] = 0x9F;
		lpcpinfo->LeadByte[2]= 0xE0; lpcpinfo->LeadByte[3] = 0xFC;
		lpcpinfo->LeadByte[4]= 0x00; lpcpinfo->LeadByte[5] = 0x00;
		break;
	case 936 : /* GB2312 (Chinese) */
	case 949 : /* KSC5601-1987 (Korean) */
	case 950 : /* BIG5 (Chinese) */
		lpcpinfo->MaxCharSize = 2;
		lpcpinfo->LeadByte[0]= 0x81; lpcpinfo->LeadByte[1] = 0xFE;
		lpcpinfo->LeadByte[2]= 0x00; lpcpinfo->LeadByte[3] = 0x00;
		break;
	case 1361 : /* Johab (Korean) */
		lpcpinfo->MaxCharSize = 2;
		lpcpinfo->LeadByte[0]= 0x84; lpcpinfo->LeadByte[1] = 0xD3;
		lpcpinfo->LeadByte[2]= 0xD8; lpcpinfo->LeadByte[3] = 0xDE;
		lpcpinfo->LeadByte[4]= 0xE0; lpcpinfo->LeadByte[5] = 0xF9;
		lpcpinfo->LeadByte[6]= 0x00; lpcpinfo->LeadByte[7] = 0x00;
	break;
	default :
		lpcpinfo->MaxCharSize = 1;
		lpcpinfo->LeadByte[0]= 0x00; lpcpinfo->LeadByte[1] = 0x00;
	break;
	}

	APISTR((LF_APIRET,"GetCPInfo returns TRUE\n"));

	return 1;  
}

HTASK GetCurrentTask();
HINSTANCE TWIN_GetInstanceFromTask( HTASK );
HMODULE GetModuleFromInstance(HINSTANCE);	

LPTSTR
GetCommandLine(VOID)
{
	LPMODULEINFO modinfo;
	HMODULE     hModule;
	HINSTANCE	hInst;
	char *lpstr;

	APISTR((LF_APICALL,"GetCommandLine\n"));

	hInst = TWIN_GetInstanceFromTask( GetCurrentTask());
	hModule = GetModuleFromInstance(hInst);	
	modinfo = GETMODULEINFO(hModule);

#if 0
	lpstr = strchr(modinfo->lpFileName,' ');
	if(lpstr) lpstr++;
	else      lpstr = modinfo->lpFileName;
#else
	lpstr = modinfo->lpCmdline;
#endif
	if(lpstr == 0)
		lpstr = "";

	RELEASEMODULEINFO(modinfo);
	APISTR((LF_APIRET,"GetCommandLine returns [%s]\n",lpstr));
	return lpstr;
}

LPWSTR
GetCommandLineW(void)
{
	APISTR((LF_APICALL,"GetCommandLineW()\n"));
	lstrcpyAtoW((LPWSTR) wcmdln, GetCommandLine());
	APISTR((LF_APIRET,"GetCommandLineW: returns LPWSTR %p\n",wcmdln));
	return (LPWSTR) wcmdln;
}

HWND
CreateWindowExW(DWORD dwExStyle, LPCSTR lpClass, 
		LPCSTR lpWindowName, DWORD dwStyle,
		int X, int Y, int nWidth, int nHeight, 
		HWND hWndParent, HMENU hMenu, HINSTANCE hInstance, 
		LPVOID lpParam)
{
	char lpclassname[80];
	char lpwinname[80];

	lpclassname[0] = 0;
	lpwinname[0] = 0;
	WideCharToMultiByte(0,0,(LPCWSTR)lpClass,0,lpclassname,80,0,0);
	WideCharToMultiByte(0,0,(LPCWSTR)lpWindowName,0,lpwinname,80,0,0);


	APISTR((LF_API,"CreateWindowExW(...)\n"));
	return CreateWindowEx(dwExStyle,lpclassname,lpwinname,dwStyle,
		X,Y,nWidth,nHeight,hWndParent,hMenu,hInstance,lpParam);
}

BOOL    
GetMessageW(MSG *lpmsg, HWND hWnd, UINT idlo, UINT idhi)
{
	APISTR((LF_API,"GetMessageW(%p,%x,%x,%x)\n",
		lpmsg,hWnd,idlo,idhi));

	return GetMessage(lpmsg,hWnd,idlo,idhi);
}

int
TranslateAcceleratorW(HWND hWnd, HACCEL hAccel, MSG *lpmsg)
{
	APISTR((LF_API,"TranslateAcceleratorW(%x,%x,%p)\n",
		hWnd,hAccel,lpmsg));

	return TranslateAccelerator(hWnd,hAccel,lpmsg);
}

LONG    
DispatchMessageW(MSG *lpmsg)
{
	APISTR((LF_API,"DispatchMessageW(%p)\n",lpmsg));
	return DispatchMessage(lpmsg);

}

DWORD
DefWindowProcW(HWND hWnd,UINT msg,DWORD wparam,DWORD lparam)
{
	APISTR((LF_API,"DefWindowProcW(%x,%x:%s,%x,%x)\n",
		hWnd,msg,
		GetTwinMsgCode(hWnd, msg),
		wparam,lparam));
	return DefWindowProc(hWnd,msg,wparam,lparam);
}

DWORD
DefDlgProcW(HWND hWnd,UINT msg,DWORD wparam,DWORD lparam)
{
	APISTR((LF_API,"DefDlgProcW(%x,%x:%s,%x,%x)\n",
		hWnd,msg,
		GetTwinMsgCode(hWnd, msg),
		wparam,lparam));
	return DefDlgProc(hWnd,msg,wparam,lparam);
}

static void
FixClassEx(WNDCLASSEX *wc,char *MenuName,char *ClassName)
{
	if(HIWORD(wc->lpszMenuName)) {	
	WideCharToMultiByte(0,0,(LPCWSTR)wc->lpszMenuName,0,MenuName,32,0,0);
	wc->lpszMenuName = MenuName;
	}

	if(HIWORD(wc->lpszClassName)) {	
	WideCharToMultiByte(0,0,(LPCWSTR)wc->lpszClassName,0,ClassName,32,0,0);
	wc->lpszClassName = ClassName;
	}
#if 0
	APISTR((LF_CONSOLE,"WNDCLASSEX(%p)\n",wc));
	APISTR((LF_CONSOLE,"sz=%d Style=%x xtra=%d,%d\n",
		wc->cbSize,wc->style,wc->cbClsExtra,
		wc->cbWndExtra));
	APISTR((LF_CONSOLE,"inst=%x icon=%x cursor=%x\n",
		wc->hInstance,wc->hIcon,wc->hCursor));
	APISTR((LF_CONSOLE,"bg=%x iconsm=%x\n",
		wc->hbrBackground,wc->hIconSm));
	DumpMemory("MenuName",wc->lpszMenuName,8);
	DumpMemory("ClassName",wc->lpszClassName,8);
#endif
}

int
RegisterClassW(WNDCLASS *wc)
{
	char MenuName[32];
	char ClassName[32];

	APISTR((LF_API,"RegisterClassW(%p)\n",wc));
	if(HIWORD(wc->lpszMenuName)) {
		WideCharToMultiByte(0,0,(LPCWSTR)wc->lpszMenuName,0,MenuName,32,0,0);
		wc->lpszMenuName = MenuName;
	} 

	if(HIWORD(wc->lpszClassName)) {
		WideCharToMultiByte(0,0,(LPCWSTR)wc->lpszClassName,0,ClassName,32,0,0);
		wc->lpszClassName = ClassName;
	}

	APISTR((LF_API,"RegisterClassW(%s)(%s)\n",
		MenuName, ClassName));
	return RegisterClass(wc);	
}

int
RegisterClassExW(WNDCLASSEX *wc)
{
	char MenuName[32];
	char ClassName[32];
	int  rc;
	APISTR((LF_APICALL,"RegisterClassExW(%p)\n",wc));
	FixClassEx(wc,MenuName,ClassName);
	rc = RegisterClassEx(wc);	
	APISTR((LF_APIRET,"RegisterClassExW returns int %d\n",rc));
	return rc;
}

BOOL
GetTextMetricsW(HDC hDC, LPTEXTMETRIC lpMetrics)
{
    BOOL rc;
    APISTR((LF_APICALL,"GetTextMetricsW(HDC=%x,LPTEXTMETRIC=%p)\n",hDC,lpMetrics));
    rc = GetTextMetrics(hDC,lpMetrics);
    APISTR((LF_APIRET,"GetTextMetricsW returns BOOL %d\n",rc));
    return rc;
}

BOOL GetTextExtentPoint32W(HDC hDC,LPCWSTR lpcstr,int cbstring,LPSIZE lpSize)
{
	APISTR((LF_API,"GetTextExtentPoint32W(HDC=%x,%s,UINT=%x)\n",
		hDC,lpcstr,cbstring));
	return GetTextExtentPoint(hDC,(LPSTR)lpcstr,cbstring,lpSize);
}

BOOL GetTextExtentPointW(HDC hDC,LPCWSTR lpcstr,int cbstring,LPSIZE lpSize)
{
	char *lpstr;
	BOOL  rc;
	lpstr = WinStrdupW(lpcstr);
	APISTR((LF_API,"GetTextExtentPointW(HDC=%x,%s,UINT=%x)\n",
		hDC,lpstr,cbstring));
	rc = GetTextExtentPoint(hDC,lpstr,cbstring,lpSize);
	WinFree(lpstr);
	return rc;
}

void
SetDlgItemTextW(HWND hDlg,int nID,LPCWSTR p)
{
	LPSTR lpstr;
	
	lpstr = WinStrdupW(p);
	APISTR((LF_API,"SetDlgItemTextW(%x,%d,%s)\n",
		hDlg,nID,lpstr));
	SetDlgItemText(hDlg,nID,lpstr);
	WinFree(lpstr);
}

BOOL
TextOutW(HDC hDC,int x,int y,LPCWSTR lpwstr,int cbstring)
{	
	char *lpstr;
	BOOL  rc;

	lpstr = WinStrdupW(lpwstr);
	APISTR((LF_API,"TextOutW(HDC=%x,%d,%d,%p[%s],UINT=%x)\n",
		hDC,x,y,lpwstr,lpstr,cbstring));
	rc = TextOut(hDC,x,y,lpstr,cbstring);
	WinFree(lpstr);
	return rc;
}

BOOL
ExtTextOutW(HDC hDC, int x, int y, UINT uiFlags,
	   const RECT *lpRect, LPCWSTR lpStr, UINT nCnt, LPINT lpDX)
{
	char *tstr;
	BOOL  rc;
	tstr = WinStrdupW(lpStr);
	APISTR((LF_API,"ExtTextOutW(HDC=%x,%d,%d,%d,%p,%s[%s],UINT=%x,%p)\n",
		hDC,x,y,uiFlags,lpRect,lpStr,tstr,nCnt,lpDX));
	rc = ExtTextOut(hDC,x,y,uiFlags,lpRect,tstr,nCnt,lpDX);
	WinFree(tstr);
	return rc;
}

LPARAM
SendMessageW(HWND hWnd,UINT msg,WPARAM wparam,LPARAM lparam)
{
	return SendMessage(hWnd,msg,wparam,lparam);
}

BOOL
IsDialogMessageW(HWND hWnd,LPMSG lpmsg)
{
	return IsDialogMessage(hWnd,lpmsg);
}

HFONT 
CreateFontW(
    int nHeight,	// logical height of font 
    int nWidth,	// logical average character width 
    int nEscapement,	// angle of escapement 
    int nOrientation,	// base-line orientation angle 
    int fnWeight,	// font weight 
    DWORD fdwItalic,	// italic attribute flag 
    DWORD fdwUnderline,	// underline attribute flag 
    DWORD fdwStrikeOut,	// strikeout attribute flag 
    DWORD fdwCharSet,	// character set identifier 
    DWORD fdwOutputPrecision,	// output precision 
    DWORD fdwClipPrecision,	// clipping precision 
    DWORD fdwQuality,	// output quality 
    DWORD fdwPitchAndFamily,	// pitch and family 
    LPCWSTR lpszFace 	// pointer to typeface name string 
   )
{
 	HFONT hFont;
	LPSTR lpstr;

	lpstr = WinStrdupW(lpszFace);
  	hFont = CreateFont(
	    nHeight,
	    nWidth,	
	    nEscapement,	
	    nOrientation,
	    fnWeight,
	    fdwItalic,	
	    fdwUnderline,
	    fdwStrikeOut,	
	    fdwCharSet,	
	    fdwOutputPrecision,	
	    fdwClipPrecision,	
	    fdwQuality,	
	    fdwPitchAndFamily,
	    lpstr);
	WinFree(lpstr);
	return hFont;
}

BOOL
CreateFontIndirectW(LPLOGFONT lpfont)
{
	return CreateFontIndirect(lpfont);
}

BOOL
WinHelpW( HWND hWnd, LPCWSTR lpszHelpFile, UINT fuCommand, DWORD dwData )
{
	BOOL  rc;
	char *lpstr;
	lpstr = WinStrdupW(lpszHelpFile);
	APISTR((LF_API,"WinHelpW(%x,%s:%s,%d,%d)\n",
		hWnd,lpszHelpFile,lpstr,fuCommand,dwData));
	rc = WinHelp( hWnd, lpstr, fuCommand, dwData );
	WinFree(lpstr);
	return rc;
}

BOOL 
GetTextExtentExPointW(HDC hDC,LPCTSTR lpsz,int cch,int nMax,LPINT lpfit,LPINT lpdx, LPSIZE lpsize)
{
	char *lpstr;
	BOOL rc;
	lpstr = WinStrdupW((LPWSTR)lpsz);
	APISTR((LF_API,"GetTextExtentExPointW(.%s..)\n",lpstr));
	rc = GetTextExtentExPoint(hDC,lpstr,cch,nMax,lpfit,lpdx,lpsize);
	WinFree(lpstr);
	return rc;
}

int
GetWindowLongW(HWND hWnd,int offset)
{
	return GetWindowLong(hWnd,offset);
}

int
SetWindowLongW(HWND hWnd,int offset,int value)
{
	return SetWindowLong(hWnd,offset,value);
}

BOOL
SetWindowTextW(HWND hWnd,LPCWSTR lpstr)
{
	LPSTR tstr;
	BOOL  rc;
	tstr = WinStrdupW(lpstr);
	rc = SetWindowText(hWnd,tstr);
	WinFree(tstr);
	return rc;
}

BOOL
AppendMenuW(HMENU hMenu,int flag,int id,LPWSTR *item)
{
// TODO - is item WIDE???
	APISTR((LF_APICALL,"AppendMenuW(%x,%x,%x,%p)\n",
		hMenu,flag,id,item));
	return 1;
}

LONG
SendDlgItemMessageW(HWND h,int id,UINT msg,WPARAM wparam,LPARAM lparam)
{
	return SendDlgItemMessage(h,id,msg,wparam,lparam);
}

HWND WINAPI
FindWindowW(LPCSTR lpszClass, LPCSTR lpszWindow)
{
	HWND hWnd;
	APISTR((LF_APICALL,"FindWindowW()\n"));
	hWnd = FindWindowEx((HWND)NULL, (HWND)NULL, lpszClass, lpszWindow);
	APISTR((LF_APIRET,"FindWindowW returns HWND %x\n",hWnd));
	return hWnd;
}

HWND WINAPI
FindWindowExW(HWND hwndParent, HWND hwndChildAfter,
	LPCTSTR lpszClass, LPCTSTR lpszWindow)
{
	HWND hWnd;
	APISTR((LF_APICALL,"FindWindowExW()\n"));
	hWnd =  FindWindowEx(hwndParent,hwndChildAfter,lpszClass,lpszWindow);
	APISTR((LF_APIRET,"FindWindowExW returns HWND %x\n",hWnd));
	return hWnd;
}

HDC
CreateDCA(LPCSTR lpszDriver, LPCSTR lpszDevice,
	 LPCSTR lpszOutput, const void *lpvInitData)
{
	return CreateDC(lpszDriver, lpszDevice, lpszOutput, lpvInitData);
}

void *
TWINmemset(void *addr,int fill,int len)
{
	return memset(addr,fill,len);
}

BOOL GetTextExtentPoint32A(HDC hDC,LPCSTR lpcstr,int cbstring,LPSIZE lpSize)
{
 	APISTR((LF_API,"GetTextExtentPoint32A(HDC=%x,%s,UINT=%x)\n",
 		hDC,lpcstr,cbstring));
 	return GetTextExtentPoint(hDC,(LPSTR)lpcstr,cbstring,lpSize);
}

void
Shell32End()
{}
void
User32End()
{}
void
Kernel32End()
{}
void
Gdi32End()
{}



/*------------------------- < Full Revision History > ----------------------
** $Log: lib32.c,v $
** Revision 1.12  1999/12/24 08:32:23  robf
** added logging to GetCommandLineW
**
** Revision 1.11  1999/11/29 22:40:18  robf
** added GetCPInfo
**
** Revision 1.10  1999/04/30 06:26:58  robf
** improper use of RELEASEMODULEINFO
**
** Revision 1.9  1999/04/26 21:17:44  robf
** added GetOEMCP
**
** Revision 1.8  1999/04/26 09:00:15  robf
** debug update, parameters mismatch
**
** Revision 1.7  1999/04/12 02:43:58  robf
** handle atom based string names
**
** Revision 1.6  1999/04/11 07:55:04  robf
** removed xxxA functions
**
** Revision 1.5  1999/04/08 21:45:49  jwhite
** Added comments to describe a change for xxxA vs xxxW support.
**
**
**-------------------------------------------------------------------------*/


